<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Admin' ) ) {

  class TagGroups_Premium_Admin {

      /**
      * Prepares variable for echoing as string
      *
      *
      * @param mixed $var Mixed type that needs to be echoed as string.
      * @return return string
      */
      public static function echo_var( $var = '' )
      {

        if ( is_bool( $var ) ) {

          return $var ? 'true' : 'false';

        } elseif ( is_array( $var ) )  {

          return print_r( $var, true );

        } else {

          return (string) $var;

        }

      }


      /**
      * Adds the screen option slider to the tag groups
      *
      * @param void
      * @return void
      */
      public static function add_screen_option() {

        $option = 'per_page';

        if ( method_exists( 'TagGroups_Admin', 'get_items_per_page' ) ) {

          $items_per_page = TagGroups_Admin::get_items_per_page();

        } else {

          $items_per_page = 10;

        }

        $args = array(
          'label' => 'Tag Groups',
          'default' => $items_per_page,
          'option' => 'tag_groups_per_page'
        );

        add_screen_option( $option, $args );

      }


      /**
      * Save the settings from the screen options
      *
      * We save it in an option field so that the value will be available in an ajax-called function
      *
      * @param mixed $status
      * @param string $option
      * @param mixed $value
      * @return mixed
      */
      public static function set_screen_option( $status, $option, $value ) {

        if ( 'tag_groups_per_page' == $option ) {

          $items_per_page_all_users = get_option( 'tag_groups_per_page', array() );

          $user = get_current_user_id();

          $items_per_page_all_users[ $user ] = $value;

          update_option( 'tag_groups_per_page', $items_per_page_all_users );

        };

        return $status;

      }


      /**
      * performs maintenance on request
      *
      *
      *
      * @param void
      * @return void
      */
      public static function do_maintenance()
      {

        global $tag_group_posts, $tag_group_terms;

        /**
        * Remove tag ids from post meta that don't exist. (e.g. deleted while the plugin was deactivated)
        */
        $count = $tag_group_posts->fix_all_incorrect_post_terms();

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

          error_log( sprintf( '[Tag Groups Premium] %d incorrect terms removed from posts.', $count ) );

        }

        /**
        * Remove post ids from tag meta that don't exist. (e.g. deleted while the plugin was deactivated)
        */
        $count = $tag_group_terms->remove_missing_groups();

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

          error_log( sprintf( '[Tag Groups Premium] %d missing posts removed from terms.', $count ) );

        }

        return true;

      }


      /**
      * Encourage the user to purge also the object cache, if applicable.
      *
      * @param integer $group_id
      * @return void
      */
      public static function encourage_cleaning_object_cache( $group_id = null )
      {

        if ( current_user_can( 'manage_options' ) && get_option( 'tag_group_object_cache', false ) ) {

          TagGroups_Admin_Notice::add( 'cache', __( sprintf( 'For a consistent user experience it is recommended to clear the object cache. <button %s>Do it now</button>', 'type="button" id="tag_groups_premium_clear_cache" class="tg_pointer"' ), 'tag-groups-premium' ) );

        }

      }


      /**
      *  Clears the object cache
      *
      * @param void
      * @return void
      */
      public static function clear_object_cache()
      {

        if ( current_user_can( 'manage_options' ) && get_option( 'tag_group_object_cache', false ) ) {

          TagGroups_Premium_Cache::purge();

        }

      }

    }

  }
