<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists('TagGroups_Premium_Meta_Box') ) {

  class TagGroups_Premium_Meta_Box {

    public function __construct() {

    }

    /**
    *   Adds the meta box to the respective page, part 1: select post type and prepare saving
    *
    * @param void
    * @return void
    */
    static function add_meta_box_hook() {

      $tag_group_meta_box_taxonomy = get_option( 'tag_group_meta_box_taxonomy', array() );

      $tag_group_post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $tag_group_meta_box_taxonomy );

      foreach ($tag_group_post_types as $post_type) {

        add_action( 'add_meta_boxes_' . $post_type , array( 'TagGroups_Premium_Meta_Box', 'add_post_meta_box'), 10, 2 );

      }

    }


    /**
    * Adds the meta box to the respective page, part 2: register the meta box
    *
    * @param object $post
    * @return void
    */
    static function add_post_meta_box( $post ) {

      // taxonomies that should have a meta box
      $tag_group_meta_box_taxonomy = get_option( 'tag_group_meta_box_taxonomy', array() );

      $taxonomy_names = get_object_taxonomies( $post->post_type );

      // get the taxonomies for that meta box
      $taxonomies = array_intersect( $tag_group_meta_box_taxonomy, $taxonomy_names);
      // only one possible
      $taxonomy_label = get_taxonomy( TagGroups_Base::get_first_element( $taxonomies ) )->labels->name;

      add_meta_box(
        'tag-groups-posts-meta-box',
        esc_html( $taxonomy_label ),
        array( 'TagGroups_Premium_Meta_Box', 'render_meta_box'),
        $post->post_type,
        'side',
        'low'
      );
    }


    /**
    * Renders the actual meta box
    *
    * @param object $post
    * @return html output
    */
    static function render_meta_box( $post ) {

      global $tag_group_groups;

      /**
      * Retrieve options
      */
      $tag_group_meta_box_taxonomy = get_option( 'tag_group_meta_box_taxonomy', array() );
      $tag_group_meta_box_add_term = get_option( 'tag_group_meta_box_add_term', 1 );
      $tag_group_meta_box_change_group = get_option( 'tag_group_meta_box_change_group', 1 );
      $tag_group_role_mb_override = get_option( 'tag_group_role_mb_override', 'no_one' );
      $tag_group_role_edit_tags = get_option( 'tag_group_role_edit_tags', 'manage_options' );

      $term_group_ids = $tag_group_groups->get_group_ids_by_position();

      $tag_group_meta_box_include = get_option( 'tag_group_meta_box_include', $term_group_ids );
      $tag_group_meta_box_open_all = get_option( 'tag_group_meta_box_open_all', 0 );

      if ( $tag_group_role_mb_override != 'no_one' && current_user_can( $tag_group_role_mb_override ) ) {

        $override_restrictions = true;

      } else {

        $override_restrictions = false;

      }

      $taxonomy_names = get_object_taxonomies( $post->post_type );

      // get the taxonomies for that meta box
      $taxonomies = array_intersect( $tag_group_meta_box_taxonomy, $taxonomy_names );

      $taxonomy = TagGroups_Base::get_first_element( $taxonomies );

      // We can have only one taxonomy per post screen
      if ( empty( $taxonomy ) ) {

        $taxonomy = 'post_tag'; // worst case fallback

      }

      $term_group_labels = $tag_group_groups->get_labels();

      if ( ! empty( $tag_group_meta_box_include ) ) {

        $term_group_ids = array_intersect( $term_group_ids, $tag_group_meta_box_include );

      }

      $term_group_labels[-1] = __( 'Add a group', 'tag-groups-premium' );


      wp_nonce_field( 'tag-groups-posts-meta-box', 'tag-groups-posts-meta-box-nonce' );

      ?>
      <input type="hidden" name="tg_taxonomy" value="<?php echo $taxonomy ?>">
      <div class="tg_meta_box_all_groups_container" style="">
        <div id="tg_meta_box_intro">
          <div id="tg_meta_box_notifications" class="tg_meta_box_notifications"></div>
          <div id="tg_meta_box_errors" class="tg_meta_box_errors"></div>
          <div id="tg_meta_box_groups_selector" style="display:inline-block; width:100%;"><?php _e( 'Loading...', 'tag-groups-premium' ) ?></div>
        </div>

      </div>

      <script>
      var selectedGroupsTermGroup = [];
      var tagGroupsTermGroupIds = <?php echo json_encode( $term_group_ids ); ?>;
      var tagGroupsTermGroupIdsReversed = <?php echo json_encode( array_values( array_reverse( $term_group_ids ) ) ) ?>;
      var tagGroupsNumberOfGroups = <?php echo count( $term_group_ids ); ?>;
      var termGroupLabels = <?php echo json_encode( $term_group_labels ); ?>;
      var tagGroupsOpenAllGroupsThatHaveTermsVar = <?php echo get_option( 'tag_group_open_all_with_terms', 1 ) ? 'true' : 'false' ?>;
      var tagGroupsOpenAllGroupsVar = <?php echo $tag_group_meta_box_open_all ? 'true' : 'false' ?>;
      var tagGroupsPostTagsLoaded = false;
      var tagGroupsGroupTagsLoaded = 0;
      var tagGroupsPostTerms = {};
      var tagGroupsTermsOfTermGroup = {};

      // retrieve tags
      tagGroupsRetrieveTags();
      // refresh tags regularly
      setInterval(function() {
        tagGroupsRetrieveTags();
      }, <?php echo TAG_GROUPS_PREMIUM_MB_RELOAD_TAGS ?>);

      /**
      * Retrieve tags that are currently selected for this post and add them into the field
      */
      function tagGroupsRetrieveTags() {
        <?php
        $protocol = isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://';
        $ajax_link = admin_url( 'admin-ajax.php', $protocol );
        ?>

        /* get all available tags for this taxonomy */
        for ( var i in tagGroupsTermGroupIds ) {
          if ( tagGroupsTermGroupIds.hasOwnProperty(i) ) {
            jQuery.ajax( {
              url: "<?php echo $ajax_link ?>",
              dataType: "xml",
              data: {
                action: 'tg_ajax_get_terms_by_group',
                termgroup: tagGroupsTermGroupIds[i],
                taxonomy: '<?php echo $taxonomy ?>',
                postid: <?php echo $post->ID ?>
              },
              method:'post',
              success: function( data ) {
                var termgroup = jQuery(data).find('termgroup').text();
                tagGroupsTermsOfTermGroup[termgroup] = JSON.parse(jQuery(data).find('tags').text());
                tagGroupsGroupTagsLoaded++;
                // run only after loading screen, and when all post tags available
                if (tagGroupsPostTagsLoaded && tagGroupsGroupTagsLoaded >= tagGroupsNumberOfGroups) {
                  tagGroupsAddSelector();
                  if (tagGroupsOpenAllGroupsVar) {
                    tagGroupsOpenAllGroups();
                  } else if (tagGroupsOpenAllGroupsThatHaveTermsVar) {
                    tagGroupsOpenAllGroupsThatHaveTerms();
                    tagGroupsOpenAllGroupsThatHaveTermsVar = false;
                  }
                }
              },
              error: function(xhr, textStatus, errorThrown) {
                console.log('[Tag Groups Premium] Error: ' + xhr.responseText);
              }
            } );
          }
        }


        /* get tags for this post */
        jQuery.ajax( {
          url: "<?php echo $ajax_link ?>",
          dataType: "xml",
          data: {
            action: 'tg_ajax_get_terms_for_post',
            postid: <?php echo $post->ID; ?>,
            taxonomy: '<?php echo $taxonomy ?>'
          },
          method:'post',
          success: function( data ) {
            tagGroupsPostTerms = JSON.parse(jQuery(data).find('tags').text());
            var postid = jQuery(data).find('postid').text();
            tagGroupsPostTagsLoaded = true;
            // run only after loading screen, and when group tags available
            if (tagGroupsGroupTagsLoaded >= tagGroupsNumberOfGroups) {
              tagGroupsAddSelector();
              if (tagGroupsOpenAllGroupsVar) {
                tagGroupsOpenAllGroups();
              } else if (tagGroupsOpenAllGroupsThatHaveTermsVar) {
                tagGroupsOpenAllGroupsThatHaveTerms();
                tagGroupsOpenAllGroupsThatHaveTermsVar = false;
              }
            }
          },
          error: function(xhr, textStatus, errorThrown) {
            console.log('[Tag Groups Premium] Error: ' + xhr.responseText);
          }
        } );
      }

      jQuery(document).ready(function(){
        /* Add selector only after document is ready, otherwise we cannot process changes. */
        /* Open autocomplete-selector with all tags on focus */
        jQuery( '.tag-editor' ).live('focus', function(e) {
          setTimeout(function(t){
            jQuery(t).find('.ui-autocomplete-input').autocomplete('search', '');
          }, 200, this )
        });

        /* Add all terms of a group */
        jQuery('.tg_meta_box_add_all').live('click', function(){
          var termgroup = jQuery(this).attr('data-termgroup');
          tg_add_all_tags(termgroup);
        });

        <?php if ( ! $tag_group_meta_box_open_all ) : ?>
        /* A group has been selected. */
        jQuery('#select_tag_group').live('change', function(){
          var termgroup = jQuery(this).val();
          /* If all groups are in use, hide the selector. */
          if (selectedGroupsTermGroup.length == tagGroupsNumberOfGroups) {
            jQuery('#tg_meta_box_intro').hide();
          } else {
            jQuery('#tg_meta_box_intro').show();
          }
          tagGroupsCreateGroupBox(termgroup);
        });

        /* Close a group */
        jQuery('.tg_meta_box_undo').live('click', function(){
          var termgroup = jQuery(this).attr('data-termgroup');
          tagGroupsRemoveGroup(termgroup);
        });

        /* Open all groups */
        jQuery('#tg_open_all').live('click', function(){
          for ( var i in tagGroupsTermGroupIds ) {
            if ( tagGroupsTermGroupIds.hasOwnProperty(i) ) {
              if ( jQuery('#tg_meta_box_terms_container_' + tagGroupsTermGroupIds[i]).length == 0 ) {
                tagGroupsCreateGroupBox(tagGroupsTermGroupIds[i]);
              }
            }
          }
        });

        /* Close all groups */
        jQuery('#tg_close_all').live('click', function(){
          jQuery('[id^=tg_meta_box_terms_container_]').each(function(){
            var termgroup = jQuery(this).attr('data-termgroup');
            tagGroupsRemoveGroup(termgroup);
          })
        });
        <?php endif; ?>
      });

      /* Open all groups that contain terms */
      function tagGroupsOpenAllGroupsThatHaveTerms(){
        for ( var i in tagGroupsTermGroupIdsReversed ) {
          if ( tagGroupsTermGroupIdsReversed.hasOwnProperty(i) ) {
            // get group tags
            if (typeof tagGroupsPostTerms[tagGroupsTermGroupIdsReversed[i]] !== 'undefined' && tagGroupsPostTerms[tagGroupsTermGroupIdsReversed[i]].length > 0) {
              if ( jQuery('#tg_meta_box_terms_container_' + tagGroupsTermGroupIdsReversed[i]).length == 0
              && tagGroupsPostTerms[tagGroupsTermGroupIdsReversed[i]].length > 0 ) {
                tagGroupsCreateGroupBox(tagGroupsTermGroupIdsReversed[i]);
              }
            }
          }
        }
      }

      /* Open all groups */
      function tagGroupsOpenAllGroups(){
        for ( var i in tagGroupsTermGroupIdsReversed ) {
          if ( tagGroupsTermGroupIdsReversed.hasOwnProperty(i) ) {
            if ( jQuery('#tg_meta_box_terms_container_' + tagGroupsTermGroupIdsReversed[i]).length == 0 ) {
              tagGroupsCreateGroupBox(tagGroupsTermGroupIdsReversed[i]);
            }
          }
        }
        jQuery('#tg_meta_box_groups_selector').hide();
      }

      /**
      * Removes the box for a particular tag group
      */
      function tagGroupsRemoveGroup(termgroup) {
        if ( tagGroupsOpenAllGroupsVar ) {
          return;
        }
        termgroup = parseInt(termgroup);
        // Remove tags that are in field but not in among original group tags from tagGroupsPostTerms
        // get selected tags
        var selected_tags_val = jQuery('#tg_meta_box_terms_container_' + termgroup ).val();

        if ( selected_tags_val != '' && tagGroupsTermsOfTermGroup[termgroup].length > 0 ) {
          var selected_tags = selected_tags_val.split(',');
          selected_tags.forEach(function(tag){
            var i = tagGroupsPostTerms[termgroup].indexOf(tag);
            if (i > -1 && tagGroupsTermsOfTermGroup[termgroup].indexOf(tag) == -1) {
              tagGroupsPostTerms[termgroup].splice(i,1);
            }
          });
        }
        /* Remove the div */
        jQuery('[data-termgroup='+termgroup+']').remove();

        /* Make group available again for selection  */
        var i = selectedGroupsTermGroup.indexOf(termgroup);
        selectedGroupsTermGroup.splice(i, 1);

        /* Reveal the selector */
        tagGroupsAddSelector();
        jQuery('#tg_meta_box_intro').show();
      }

      /**
      *	Creates the box for a particular tag group where tags can be added
      */
      function tagGroupsCreateGroupBox(termgroup) {
        termgroup = parseInt(termgroup);
        // add this termgroup to the array of groups that have a box open
        selectedGroupsTermGroup.push(termgroup);
        var intro = '<div class="tg_meta_box_group_container" data-termgroup="' + termgroup + '" style="padding:5px;">' +
        '<h3> <div class="tg_right">' +
        '<span class="tg_meta_box_add_all dashicons dashicons-text tg_pointer tg_left" data-termgroup="' + termgroup + '" title="<?php _e('Add all from group', 'tag-groups-premium' ) ?>"></span>&nbsp;';

        if ( ! tagGroupsOpenAllGroupsVar ) {
          intro += '<span class="tg_meta_box_undo dashicons dashicons-dismiss tg_pointer tg_left" data-termgroup="' + termgroup + '" title="<?php _e('Close', 'tag-groups-premium' ) ?>"></span>';
        }

        intro += '</div>' + termGroupLabels[termgroup] + '</h3><input type="text" id="tg_meta_box_terms_container_' + termgroup + '" name="tg_meta_box_tags_' + termgroup + '" class="tg_meta_box_tags_container tg_sortable_connected" data-termgroup="' + termgroup + '" data-role="tagsinput" style="height:1.5em;width:90%;" /></div>';

        jQuery('#tg_meta_box_intro').after(intro);
        if (typeof tagGroupsPostTerms[termgroup] !== 'undefined' && tagGroupsPostTerms[termgroup].length > 0 ) {
          // populate tag field with existing post tags from this group
          var post_terms_in_group_s = tagGroupsPostTerms[termgroup].join(',');
          jQuery('#tg_meta_box_terms_container_' + termgroup).val(post_terms_in_group_s);
        }
        jQuery( '#tg_meta_box_terms_container_' + termgroup ).tagEditor({
          placeholder: '<?php _e( 'Please enter your tags here.', 'tag-groups-premium' ) ?>',
          forceLowercase: false,
          <?php if ( ! $tag_group_meta_box_add_term && ! $override_restrictions ): ?>
          noEdit:true,
          <?php endif; ?>
          <?php if ( current_user_can( $tag_group_role_edit_tags ) && ( $tag_group_meta_box_change_group || $override_restrictions ) ): ?>
          connectWith: ".tag-editor",
          <?php endif; ?>
          autocomplete: {
            minLength: 0,
            position: { collision: 'flip' },
            source: function(request, response) {
              // clone the array
              var available_group_terms = JSON.parse(JSON.stringify(tagGroupsTermsOfTermGroup[termgroup]));
              // remove terms from array that are already in the input field
              var tags = jQuery('#tg_meta_box_terms_container_' + termgroup ).tagEditor('getTags')[0].tags;
              if (tags.length > 0) {
                // remove tag from available tags
                tags.forEach(function(tag){
                  var i = available_group_terms.indexOf(tag);
                  if (i>-1) {
                    available_group_terms.splice(i,1);
                  }
                });
              }
              response(jQuery.ui.autocomplete.filter(available_group_terms, request.term));
            }
          },
          beforeTagSave: function(field, editor, tags, tag, val){
            val = val.trim();
            if (''===val) return false;

            var valLowerCase = val.toLowerCase()
            // add tag to tagGroupsPostTerms[termgroup]
            // Check for not allowed tags
            // All checks have to be case-insensitive, since later processing with WP's term_exists() first searches the terms' slugs in the database, and therefore two tags with different cases count as the same
            // concat all tags
            var all_tags = [];
            for (var groupId in tagGroupsTermsOfTermGroup) {
              all_tags = all_tags.concat(tagGroupsTermsOfTermGroup[groupId]);
            }
            // make lower case
            var all_tags_lowerCase = all_tags.map(function(value) {
              return value.toLowerCase();
            });

            // make sure that tag is not already there
            // make lower case
            // var post_terms_lowerCase = tagGroupsPostTerms[termgroup].map(function(value) {
            //   return value.toLowerCase();
            // });

            if (tags.indexOf(valLowerCase) > -1) {
              jQuery('#tg_meta_box_errors').html('<?php _e( 'Tag already selected.', 'tag-groups-premium' ) ?>').slideDown().clearQueue();
              setTimeout(function() {jQuery('#tg_meta_box_errors').slideUp().html('');},5000);
              return false;
            }

            // don't allow to assign new tag to a group, except unassinged
            <?php if ( ! current_user_can( $tag_group_role_edit_tags ) ) : ?>
            if ( termgroup != 0 && all_tags_lowerCase.indexOf(valLowerCase) == -1 ) {
              jQuery('#tg_meta_box_errors').html('<?php _e( 'Not allowed to assign a tag to a group.', 'tag-groups-premium' ) ?>').slideDown().clearQueue();
              setTimeout(function() {jQuery('#tg_meta_box_errors').slideUp().html('');},5000);
              return false;
            }
            // don't allow to move by typing in another group
            // make lower case
            var group_terms_lowerCase = tagGroupsTermsOfTermGroup[termgroup].map(function(value) {
              return value.toLowerCase();
            });
            if ( all_tags_lowerCase.indexOf(valLowerCase)>-1 && group_terms_lowerCase.indexOf(valLowerCase)==-1 ) {
              jQuery('#tg_meta_box_errors').html('<?php _e( 'Not allowed to assign a tag to a group.', 'tag-groups-premium' ) ?>').slideDown().clearQueue();
              setTimeout(function() {jQuery('#tg_meta_box_errors').slideUp().html('');},5000);
              return false;
            }
            <?php endif; ?>

            // don't allow new tags
            <?php if ( ! $tag_group_meta_box_add_term && ! $override_restrictions ) : ?>
            if ( all_tags_lowerCase.indexOf(valLowerCase) == -1 ) {
              jQuery('#tg_meta_box_errors').html('<?php _e( 'Not allowed to add new tags.', 'tag-groups-premium' ) ?>').slideDown().clearQueue();
              setTimeout(function() {jQuery('#tg_meta_box_errors').slideUp().html('');},5000);
              return false;
            }
            <?php endif; ?>

            // restricted to original group
            <?php if ( ! $tag_group_meta_box_change_group && ! $override_restrictions ) : ?>
            // make lower case
            var group_terms_lowerCase = tagGroupsTermsOfTermGroup[termgroup].map(function(value) {
              return value.toLowerCase();
            });
            if ( all_tags_lowerCase.indexOf(valLowerCase)>-1 && group_terms_lowerCase.indexOf(valLowerCase)==-1 ) {
              jQuery('#tg_meta_box_errors').html('<?php _e( 'Not allowed to use tags from other groups.', 'tag-groups-premium' ) ?>').slideDown().clearQueue();
              setTimeout(function() {jQuery('#tg_meta_box_errors').slideUp().html('');},5000);
              return false;
            }
            <?php endif; ?>

            tagGroupsPostTerms[termgroup].push(val);
          },
          beforeTagDelete: function(field, editor, tags, val){
            // remove tag from tagGroupsPostTerms[termgroup]
            var i = tagGroupsPostTerms[termgroup].indexOf(val);
            if (i>-1) {
              tagGroupsPostTerms[termgroup].splice(i,1);
            }
          },
          onChange: function(field, editor, tags) {
            // coloring of new and moved tags
            // concat all existing tags
            var all_tags = [];
            for (var groupId in tagGroupsTermsOfTermGroup) {
              all_tags = all_tags.concat(tagGroupsTermsOfTermGroup[groupId]);
            };

            tags.forEach(function(tag){
              if ( all_tags.indexOf(tag) == -1 ) {
                jQuery('li', editor).each(function(){
                  var li = jQuery(this);
                  if (li.find('.tag-editor-tag').html() == tag) {
                    li.addClass('tg_new_tag');
                    li.attr('title', '<?php _e( 'New tag', 'tag-groups-premium' ) ?>');
                  }
                });
              } else if ( all_tags.indexOf(tag) > -1 && tagGroupsTermsOfTermGroup[termgroup].indexOf(tag) == -1 ) {
                jQuery('li', editor).each(function(){
                  var li = jQuery(this);
                  if (li.find('.tag-editor-tag').html() == tag) {
                    li.addClass('tg_moved_tag');
                    li.attr('title', '<?php _e( 'Added group', 'tag-groups-premium' ) ?>');
                  }
                });
              } else {
                // in case it's been moved back
                jQuery('li', editor).each(function(){
                  var li = jQuery(this);
                  if (li.find('.tag-editor-tag').html() == tag) {
                    li.removeClass('tg_moved_tag');
                  }
                });
              }
            });
          },
        })

        tagGroupsAddSelector();
      }

      /**
      * Add a <select> element with all still selectable tag groups
      */
      function tagGroupsAddSelector(){
        if (tagGroupsOpenAllGroupsVar) {
          return;
        }
        var output = '';
        var groupOptions = '';

        groupOptions += '<option value="-1">' + termGroupLabels[-1] + '</option>';

        for ( var i in tagGroupsTermGroupIds ) {
          if ( tagGroupsTermGroupIds.hasOwnProperty(i) ) { // include 0
            /* Add only groups that are not yet selected */
            if (selectedGroupsTermGroup.indexOf(tagGroupsTermGroupIds[i]) == -1 ) {
              groupOptions += '<option value="' + tagGroupsTermGroupIds[i] + '">' + termGroupLabels[tagGroupsTermGroupIds[i]] + '</option>';
            }
          }
        }
        if (selectedGroupsTermGroup.length < tagGroupsNumberOfGroups) {
          output += '<select id="select_tag_group">' + groupOptions + '</select>';
        }
        output += '<span style="margin:5px;">';

        if (selectedGroupsTermGroup.length < tagGroupsNumberOfGroups) {
          // "add all" button
          output += ' <span id="tg_open_all" class="dashicons dashicons-arrow-down-alt2 tg_pointer tg_right" title="<?php _e( 'Open all', 'tag-groups-premium' ) ?>"></span>';
        } else {
          output += ' <span id="tg_open_all_inactive" class="dashicons dashicons-arrow-down-alt2 tg_pointer tg_right tg_inactive"></span>';
        }
        if (selectedGroupsTermGroup.length > 0) {
          output += ' <span id="tg_close_all" class="dashicons dashicons-arrow-up-alt2 tg_pointer tg_right" title="<?php _e( 'Close all', 'tag-groups-premium' ) ?>"></span>';
        } else {
          output += ' <span id="tg_close_all_inactive" class="dashicons dashicons-arrow-up-alt2 tg_pointer tg_right tg_inactive"></span>';
        }
        output += '</span>';

        jQuery('#tg_meta_box_groups_selector').html('<div id="tg_select_group_container" style="display:inline-block;width:100%;">' + output + '</div>');
      }

      /**
      * Adds all tags of a group to the corresponding input field
      */
      function tg_add_all_tags(termgroup){
        // get all terms of this input box
        var selected_group_terms = jQuery('#tg_meta_box_terms_container_'+termgroup).val().split(',');
        // add them one by one to the tag editor
        tagGroupsTermsOfTermGroup[termgroup].forEach(function(term){
          // add only those that are not yet there, otherwise we get error message "Tag already selected"
          if (selected_group_terms.indexOf(term) == -1) {
            // Let the browser do other tasks as well
            setTimeout(function() {
              jQuery( '#tg_meta_box_terms_container_' + termgroup ).tagEditor('addTag',term,true);
            }, 5);
          }
        });
      }
      </script>
      <?php

      // If Gutenberg is active, we need to check if the default Tags panel should be hidden
      if ( function_exists( 'register_block_type' ) && get_option( 'tag_group_hide_tagsdiv', 0 ) ) {

        self::remove_gutenberg_tags_panel();

      }

    }


    /**
    * Save the meta data when the post is saved.
    *
    * @param int $post_id
    */
    static function save_meta_box( $post_id, $post ) {

      global $tag_group_groups;

      // Check if our nonce is set.
      if ( ! isset( $_POST['tag-groups-posts-meta-box-nonce'] ) ) {

        return $post_id;

      }

      // Verify that the nonce is valid.
      if ( ! wp_verify_nonce( $_POST['tag-groups-posts-meta-box-nonce'], 'tag-groups-posts-meta-box' ) ) {

        return $post_id;

      }

      // omit autosaves
      if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { // apparently not equivalent to wp_is_post_autosave( $post_id )

        return $post_id;

      }

      // omit revisions
      if ( 'revision' == $post->post_type ) {

        return $post_id;

      }


      // Check the user's permissions
      $post_type_object = get_post_type_object( $post->post_type );

      if ( is_object( $post_type_object ) && is_object( $post_type_object->cap ) && ! current_user_can( $post_type_object->cap->edit_post, $post_id ) ) {

        return $post_id;

      }

      // check if there was a multisite switch before
      if ( is_multisite() && ms_is_switched() ) {
        /**
        * since version 1.3: In rare cases, the content of $GLOBALS['_wp_switched_stack'] will never be reset => The post is saved, but without the tags. Solution: undo switching
        * Replacing: return $post_id;
        */
        $GLOBALS['_wp_switched_stack'] = array();
        $GLOBALS['switched'] = false;

      }

      /**
      * Retrieve permissions of Tag Groups Premium
      */
      $tag_group_meta_box_add_term = get_option( 'tag_group_meta_box_add_term', 1 );
      $tag_group_meta_box_change_group = get_option( 'tag_group_meta_box_change_group', 1 );
      $tag_group_role_mb_override = get_option( 'tag_group_role_mb_override', 'no_one' );
      $tag_group_role_edit_tags = get_option( 'tag_group_role_edit_tags', 'manage_options' );

      if ( $tag_group_role_mb_override != 'no_one' && current_user_can( $tag_group_role_mb_override ) ) {

        $override_restrictions = true;

      } else {

        $override_restrictions = false;

      }

      /**
      * Reduce the risk of interference from other plugins
      */
      remove_all_filters( 'get_terms_orderby' );
      remove_all_filters( 'list_terms_exclusions' );
      remove_all_filters( 'get_terms' );

      /**
      * Check if we have a taxonomy
      */
      if ( ! empty( $_POST['tg_taxonomy'] ) ) {

        $taxonomy = sanitize_title( $_POST['tg_taxonomy'] );

      } else {

        return $post_id;

      }


      $all_term_group_ids = $tag_group_groups->get_group_ids_by_position();

      // hidden groups are never $_POSTed, so we should not identify these tags as deleted
      $tag_group_meta_box_include = get_option( 'tag_group_meta_box_include', $all_term_group_ids );

      $meta_box_term_group_ids = array_intersect( $all_term_group_ids, $tag_group_meta_box_include );

      $new_terms = array(); // new in the post

      // retrieve post terms
      $post_term_ids = wp_get_post_terms( $post_id, $taxonomy, array("fields" => "ids") );

      $tg_post = new TagGroups_Premium_Post( $post_id );

      $current_post_terms = $tg_post->get_terms_by_group();

      $new_post_terms = array();

      // A) Check if we have to create a new term
      foreach ( $meta_box_term_group_ids as $term_group ) {

        if ( isset( $_POST[ 'tg_meta_box_tags_' . $term_group ] ) ) {

          $terms_s = sanitize_text_field( $_POST['tg_meta_box_tags_' . $term_group ] );

          $terms_a = explode( ',', $terms_s );

          $new_post_terms[ $term_group ] = array();

          /**
          * Processing all submitted terms
          */
          foreach ( $terms_a as $term_name ) {

            if ( empty( $term_name ) ) {

              continue;

            }

            /**
            * We might have created a new term
            */
            if ( ! term_exists( $term_name, $taxonomy ) ) {
              // Term is new -> We need to create it with right term_group

              // check if we are allowed to add tags
              if ( $tag_group_meta_box_add_term || $override_restrictions ) {

                // have to insert and add term_group in 2 steps
                $inserted_term = wp_insert_term( $term_name, $taxonomy );

                // $term_id needed for further processing
                $term_id = $inserted_term['term_id'];

                $tg_term = new TagGroups_Term( $term_id );

                if ( 0 == $term_group ) {

                  $tg_term->set_group( $term_group )->save();

                } else {

                  $tg_term->add_group( $term_group )->save();

                }

              }

            } else {

              $wp_term = get_term_by( 'name', $term_name, $taxonomy );

              $term_id = $wp_term->term_id;

              /**
              *   get_term_by does not consider WPML, need to "translate" the term ID
              */
              if ( defined( 'ICL_LANGUAGE_CODE' ) ) {

                $term_id = icl_object_id( $term_id, $taxonomy, true, ICL_LANGUAGE_CODE );

              }

            }

            /**
            * Save them as array of groups and term IDs
            */
            $new_post_terms[ $term_group ][] = $term_id;

          } // looping through submitted terms

        } // only submitted groups

      } // looping through all groups


      /**
      * B) Check if any tags in groups were changed
      */
      foreach ( $all_term_group_ids as $term_group ) {

        // if ( 0 == $term_group ) {
        //
        //   continue;
        //
        // }

        /**
        * Check if any tags in groups were removed
        */
        foreach ( $current_post_terms[ $term_group ] as $term_id ) {

          if ( isset( $new_post_terms[ $term_group ] ) ) { // group was open

            if ( ! in_array( $term_id, $new_post_terms[ $term_group ] ) ) {
              /**
              * The term has been removed from that group
              */

              /**
              * Check if it is still in any other group
              */
              $found_in_other_group = false;

              foreach ( $all_term_group_ids as $term_group_other ) {

                if ( $term_group_other == $term_group ) {

                  continue;

                }

                if ( isset( $new_post_terms[ $term_group_other ] ) && in_array( $term_id, $new_post_terms[ $term_group_other ] ) ) {
                  /**
                  * We found it - we don't need to know how often
                  */
                  $found_in_other_group = true;

                  break;

                }

              }

              if ( ! $found_in_other_group ) {
                /**
                * No other group has that term: It was completely removed.
                * We need to remove it from the post tags.
                */

                wp_remove_object_terms( $post_id, $term_id, $taxonomy );

                $post_term_ids = array_diff( $post_term_ids, array( $term_id ) );

              }

            }

          } else {

            /**
            * We have tags for that group, but it was not open in the Meta Box: We keep the previous tags
            */
            $carried_over_post_terms[ $term_group ] = $current_post_terms[ $term_group ];

          }

        }


        /**
        * Check if any tags in groups were added
        */
        if ( isset( $new_post_terms[ $term_group ] ) ) {

          foreach ( $new_post_terms[ $term_group ] as $term_id ) {

            if ( ! in_array( $term_id, $current_post_terms[ $term_group ] ) ) {
              /**
              * The term has been newly added or moved
              */

              /**
              * Check if term was in any other group
              */
              $found_in_other_group = 0;

              $found_in_group_ids = array();

              foreach ( $all_term_group_ids as $term_group_other ) {

                if ( $term_group_other == $term_group ) {

                  continue;

                }

                if ( in_array( $term_id, $current_post_terms[ $term_group_other ] ) && ! in_array( $term_id, $new_post_terms[ $term_group_other ] ) ) {
                  /**
                  * It was somewhere else, but has been removed
                  */
                  $found_in_other_group++;

                  $found_in_group_ids[] = $term_group_other;

                }

              }

              sort( $found_in_group_ids );

              if ( 0 == $found_in_other_group ) {
                /**
                * No other group had that term: It was newly added.
                * We need to add it to the post tags. Do it at the end.
                */

                /**
                * It might still be a new group
                */
                $tg_term = new TagGroups_Term( $term_id );

                if ( $tg_term->is_in_group( $term_group ) || current_user_can( $tag_group_role_edit_tags ) ) {
                  /**
                  * Add term to group, if not there
                  */
                  if ( 0 == $term_group ) {

                    $tg_term->set_group( $term_group )->save();

                  } else {

                    if ( ! $tg_term->is_in_group( $term_group ) ) {

                      $tg_term->add_group( $term_group )->save();

                    }

                  }

                  $new_terms[] = $term_id;

                } else {
                  /**
                  * We have to undo the move
                  */
                  $current_post_terms[ TagGroups_Base::get_first_element( $found_in_group_ids ) ][] = $term_id;

                  $new_post_terms[ $term_group ] = array_diff( $new_post_terms[ $term_group ], array( $term_id ) );

                }

              } elseif ( 1 == $found_in_other_group ) {
                /**
                * The term was moved
                */

                /**
                * Check if we are allowed to move a tag
                */
                if ( ! $tag_group_meta_box_change_group && ! $override_restrictions ) {
                  /**
                  * We have to undo the move
                  */
                  $current_post_terms[ TagGroups_Base::get_first_element( $found_in_group_ids ) ][] = $term_id;

                  $new_post_terms[ $term_group ] = array_diff( $new_post_terms[ $term_group ], array( $term_id ) );

                } else {

                  $tg_term = new TagGroups_Term( $term_id );

                  if ( $tg_term->is_in_group( $term_group ) || current_user_can( $tag_group_role_edit_tags ) ) {
                    /**
                    * Add term to group, if not there
                    */
                    if ( 0 == $term_group ) {

                      $tg_term->set_group( $term_group )->save();

                    } else {

                      if ( ! $tg_term->is_in_group( $term_group ) ) {

                        $tg_term->add_group( $term_group )->save();

                      }

                    }

                  } else {
                    /**
                    * We have to undo the move
                    */
                    $current_post_terms[ TagGroups_Base::get_first_element( $found_in_group_ids ) ][] = $term_id;

                    $new_post_terms[ $term_group ] = array_diff( $new_post_terms[ $term_group ], array( $term_id ) );

                  }

                }

              } else {
                /**
                * If it was moved, we don't know from where
                */

                if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

                  error_log( sprintf( '[Tag Groups Premium] Term with ID %d might have been moved but we don\'t know from where.' ), $term_id );

                }

                /**
                * Check if we are allowed to move a tag
                */
                if ( ! $tag_group_meta_box_change_group && ! $override_restrictions ) {
                  /**
                  * We have to undo the move
                  */
                  foreach ( $found_in_group_ids as $found_in_group_id ) {

                    $current_post_terms[ $found_in_group_ids[ $found_in_group_id ] ][] = $term_id;

                  }

                  $new_post_terms[ $term_group ] = array_diff( $new_post_terms[ $term_group ], array( $term_id ) );

                } else {

                  $tg_term = new TagGroups_Term( $term_id );

                  if ( $tg_term->is_in_group( $term_group ) || current_user_can( $tag_group_role_edit_tags ) ) {
                    /**
                    * Add term to group, if not there
                    */
                    if ( 0 == $term_group ) {

                      $tg_term->set_group( $term_group )->save();

                    } else {

                      if ( ! $tg_term->is_in_group( $term_group ) ) {

                        $tg_term->add_group( $term_group )->save();

                      }

                    }

                  } else {
                    /**
                    * We have to undo the move
                    */
                    foreach ( $found_in_group_ids as $found_in_group_id ) {

                      $current_post_terms[ $found_in_group_ids[ $found_in_group_id ] ][] = $term_id;

                    }

                    $new_post_terms[ $term_group ] = array_diff( $new_post_terms[ $term_group ], array( $term_id ) );

                  }

                }

              }

            }

          }

        }

      }



      /**
      *
      * Prepare the arrays
      */
      $terms_for_metadata = array();

      $assigned_terms = array();

      foreach ( $all_term_group_ids as $group_id ) {

        $term_ids = array();

        if ( isset( $carried_over_post_terms[ $group_id ] ) ) {

          $term_ids = array_merge( $term_ids, $carried_over_post_terms[ $group_id ] );

        }

        if ( isset( $new_post_terms[ $group_id ] ) ) {

          $term_ids = array_merge( $term_ids, $new_post_terms[ $group_id ] );

        }


        foreach ( $term_ids as $term_id ) {

          $terms_for_metadata[] = array(
            'term_id'  => $term_id,
            'term_group' => $group_id,
            'taxonomy'  => $taxonomy
          );

          $assigned_terms[] = $term_id;

        }

      }

      /**
      * C: Check terms in post meta if they should be in groups
      */
      foreach ( $post_term_ids as $post_term_id ) {

        if ( ! in_array( $post_term_id, $assigned_terms ) ) {

          $tg_term = new TagGroups_Term( $post_term_id );

          $groups = $tg_term->get_groups();

          if ( count( array_diff( $groups, array(0) ) ) > 0 ) {

            foreach ( $groups as $group ) {

              $terms_for_metadata[] = array(
                'term_id'  => $post_term_id,
                'term_group' => $group,
                'taxonomy'  => $taxonomy
              );

            }

            $assigned_terms[] = $post_term_id;

            $post_term_ids = array_diff( $post_term_ids, array( $post_term_id ) );

          }

        }

      }

      /**
      * D: Update the post terms: We must even re-submit the terms that previously have been there
      */
      $new_post_terms = array_values( array_unique( array_merge( $post_term_ids, $assigned_terms ) ) );

      wp_set_post_terms( $post_id, $new_post_terms, $taxonomy );

      /**
      * E: Update the post meta
      *
      */
      $unassigned_terms = $post_term_ids; //array_diff( $post_term_ids, $assigned_terms );

      $tg_post->set_terms( $terms_for_metadata, $unassigned_terms, 'id' )->save();

      do_action( 'purge_post_list_transients', $post_id, $post );

      return $post_id;

    }


    /**
    * Removes terms from data sent when saving a post
    *
    * Used if default tag box is disabled.
    *
    * @param type var Description
    * @return return type
    */
    static function remove_terms_from_post_data()
    {

      /**
      * Disable for Quick Edit
      */
      if ( ! empty( $_REQUEST['_inline_edit'] ) ) {

        return;

      }

      if ( ! empty( $_POST['tg_taxonomy'] ) ) {

        unset( $_POST['tax_input'][ sanitize_title( $_POST['tg_taxonomy'] ) ] );

      } else {

        $tag_group_meta_box_taxonomies = get_option( 'tag_group_meta_box_taxonomy', array() );

        foreach ( $tag_group_meta_box_taxonomies as $taxonomy ) {

          unset( $_POST['tax_input'][ $taxonomy ] );

        }

      }

    }


    /**
    * AJAX handler to get all available terms, separated into groups
    *
    * @param void
    * @return void
    */
    static function ajax_get_terms_by_group()
    {

      global $tag_group_groups;

      if ( ! empty( $_REQUEST['postid'] ) ) {

        $post_id = (int) trim( $_REQUEST['postid'] );

      } else {

        $post_id = 0;

      }

      if ( isset( $_REQUEST['termgroup'] ) ) {

        $termgroup = (int) trim( $_REQUEST['termgroup'] );

      }

      if ( ! empty( $_REQUEST['taxonomy'] ) ) {

        $taxonomy = sanitize_text_field( trim( $_REQUEST['taxonomy'] ) );

      } else {

        $taxonomy = 'post_tag';

      }

      $taxonomy_array = explode( ',', $taxonomy );

      $response = new WP_Ajax_Response;

      $tg_group = new TagGroups_Group( $termgroup );

      if ( ! $tg_group->exists() ) {

        wp_die();

      }

      $terms_name = $tg_group->get_group_terms( $taxonomy_array, false, 'names', $post_id );

      if ( is_array( $terms_name ) ) {

        $terms_name = array_map( 'html_entity_decode', $terms_name );

        /**
        * Remove characters that break the editor
        */
        $terms_name = array_map( function( $item ) {
          return str_replace( array( ',', ';' ), '', $item );
        },
        $terms_name );

        /**
        * Reduce multiple spaces to one
        */
        $terms_name = array_map( function( $item ) {
          return str_replace( array( '  ' ), ' ', $item );
        },
        $terms_name );

      }

      $response->add(
        array(
          'data' => 'success',
          'supplemental' => array(
            'tags' => json_encode( $terms_name ),
            'termgroup' => $termgroup,
            'postid' => $post_id
          )
        )
      );

      $response->send();

      wp_die();

    }


    /**
    * AJAX handler to get all tags for a particular post
    *
    * @param void
    * @return void
    */
    static function ajax_get_terms_for_post()
    {

      if ( isset( $_REQUEST['postid'] ) ) {

        $postid = (int) trim( $_REQUEST['postid'] );

      }

      if ( ! empty( $_REQUEST['taxonomy'] ) ) {

        $taxonomy = sanitize_title( trim( $_REQUEST['taxonomy'] ) );

      } else {

        $taxonomy = 'post_tag';

      }

      $response = new WP_Ajax_Response;

      $tg_post = new TagGroups_Premium_Post( $postid );

      $terms_name =  $tg_post->get_term_names_by_id( true );

      if ( is_array( $terms_name ) ) {

        foreach ( $terms_name as $key => $term_name ) {

          $terms_name[ $key ] = array_map( 'html_entity_decode', $term_name );

        }

      }

      /**
      * Remove characters that break the editor
      */
      $terms_name = array_map( function( $item ) {
        return str_replace( array( ',', ';' ), '', $item );
      },
      $terms_name );

      /**
      * Reduce multiple spaces to one
      */
      $terms_name = array_map( function( $item ) {
        return str_replace( array( '  ' ), ' ', $item );
      },
      $terms_name );

      $response->add(
        array(
          'data' => 'success',
          'supplemental' => array(
            'tags' => json_encode( $terms_name ),
            'postid' => $postid
          )
        )
      );

      $response->send();

      wp_die();

    }


    /**
    * Removes the default tag meta box from all pages where the meta box is set to appear
    *
    * @param void
    * @return void
    */
    static function remove_tag_meta_boxes()
    {

      $tag_group_meta_box_taxonomies = get_option( 'tag_group_meta_box_taxonomy', array() );

      if ( ! empty( $tag_group_meta_box_taxonomies ) ) {

        foreach ( $tag_group_meta_box_taxonomies as $taxonomy ) {

          $post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $taxonomy );

          foreach ( $post_types as $post_type ){

            remove_meta_box( 'tagsdiv-' . $taxonomy, $post_type, 'normal' );

          }

        }

      }

    }


    /**
    * Remove the "Tags" panel in the Gutenberg editor
    *
    * We need to make sure that the Classic Editor is not active
    *
    * @param void
    * @return void
    */
    static function remove_gutenberg_tags_panel() {
      ?>
      <script>
      jQuery(document).ready(function(){
        if ( typeof wp !== "undefined" && typeof wp.data !== "undefined" && typeof wp.data.dispatch( 'core/edit-post' ) !== "undefined" && wp.data.dispatch( 'core/edit-post' ) !== null ) {
          wp.data.dispatch( 'core/edit-post' ).removeEditorPanel( 'taxonomy-panel-post_tag' );
        }
      } );
      </script>
      <?php
    }

  } // class

}
