<?php
/**
* Tag Groups Premium
*
* @package    Tag Groups Premium
* @author     Christoph Amthor
* @copyright  2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license    see official vendor website
* @since      1.14.1
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Settings' ) ) {

  /**
  * This class modfiies the admin menu structure and content as required for the premium plugin.
  *
  */
  class TagGroups_Premium_Settings {


    /**
    * hooks into the filters to modify available tabs on admin setting pages
    *
    * @param void
    * @return void
    */
    public static function register_tab_filters()
    {

      add_filter( 'tag_groups_settings_front_end_tabs', array( 'TagGroups_Premium_Settings', 'modify_front_end_tabs' ) );

      add_filter( 'tag_groups_settings_back_end_tabs', array( 'TagGroups_Premium_Settings', 'modify_back_end_tabs' ) );

      add_filter( 'tag_groups_settings_tools_tabs', array( 'TagGroups_Premium_Settings', 'modify_tools_tabs' ) );

      add_filter( 'tag_groups_settings_troubleshooting_tabs', array( 'TagGroups_Premium_Settings', 'modify_troubleshooting_tabs' ) );

      add_filter( 'tag_groups_settings_about_tabs', array( 'TagGroups_Premium_Settings', 'modify_about_tabs' ) );

    }


    /**
    * mofifies the array that contains the Front End tabs
    *
    *
    * @param array $tabs
    * @return array
    */
    public static function modify_front_end_tabs( $tabs )
    {

      global $tag_groups_premium_fs_sdk;

      $tabs['themes-premium'] = __( 'Other Themes', 'tag-groups-premium' );

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') )  {

        $tabs['dpf'] = __( 'Dynamic Post Filter', 'tag-groups-premium' );

      }

      $tabs['cache'] = __( 'Cache', 'tag-groups-premium' );

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        $tabs['post-tags'] = __( 'Post Tags', 'tag-groups-premium' );

        if ( defined( 'WC_VERSION' ) ) {

          $tabs['woocommerce'] = __( 'WooCommerce', 'tag-groups-premium' );

        }

      }

      return $tabs;

    }


    /**
    * mofifies the array that contains the Back End tabs
    *
    *
    * @param array $tabs
    * @return array
    */
    public static function modify_back_end_tabs( $tabs )
    {

      global $tag_groups_premium_fs_sdk;

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        $tabs['meta-box'] = __( 'Meta Box', 'tag-groups-premium' );

        $tabs['permissions'] = __( 'Permissions', 'tag-groups-premium' );

      }

      return $tabs;

    }


    /**
    * mofifies the array that contains the Tools tabs
    *
    *
    * @param array $tabs
    * @return array
    */
    public static function modify_tools_tabs( $tabs )
    {

      $tabs['maintenance'] = __( 'Maintenance', 'tag-groups-premium' );

      return $tabs;

    }


    /**
    * mofifies the array that contains the Troubleshooting tabs
    *
    *
    * @param array $tabs
    * @return array
    */
    public static function modify_troubleshooting_tabs( $tabs )
    {

      unset( $tabs['faq'] );

      unset( $tabs['documentation'] );

      unset( $tabs['support'] );

      unset( $tabs['system'] ); // Unset and re-set to place it at the end

      unset( $tabs['first-aid'] );

      $tabs['first-aid-premium'] = __( 'First Aid', 'tag-groups-premium' );

      $tabs['faq-premium'] = __( 'FAQ and Common Issues', 'tag-groups-premium' );

      $tabs['documentation-premium'] = __( 'Documentation', 'tag-groups-premium' );

      $tabs['support-premium'] = __( 'Get Support', 'tag-groups-premium' );

      $tabs['system'] = __( 'System Information', 'tag-groups-premium' );

      return $tabs;

    }


    /**
    * mofifies the array that contains the About tabs
    *
    *
    * @param array $tabs
    * @return array
    */
    public static function modify_about_tabs( $tabs )
    {

      unset( $tabs['info'] );

      unset( $tabs['licenses'] );

      unset( $tabs['news'] );

      $tabs['info-premium'] = __( 'Info', 'tag-groups-premium' );

      $tabs['licenses-premium'] = __( 'Licenses', 'tag-groups-premium' );

      $tabs['news-premium'] = __( 'Development News', 'tag-groups-premium' );

      return $tabs;

    }


    /**
    * hooks into the filter to modify the admin menu structure
    *
    * @param void
    * @return void
    */
    public static function register_admin_structure_filter()
    {

      add_filter( 'tag_groups_admin_structure', array( 'TagGroups_Premium_Settings', 'modify_admin_structure' ) );

    }


    /**
    * modifies the array that contains the admin menu structure
    *
    * @param array $tag_groups_admin_structure
    * @return array
    */
    public static function modify_admin_structure( $tag_groups_admin_structure )
    {

      // remove the "Premium" menu
      unset( $tag_groups_admin_structure[7] );

      return $tag_groups_admin_structure;

    }


    /**
    * returns tab contents for tabs that are available for premium only
    *
    *
    * @param string $active_tab identifier for the content
    * @return string HTLM for the tab content
    */
    public static function get_content( $active_tab )
    {

      global $tag_groups_premium_fs_sdk, $tag_group_groups;

      switch ( $active_tab ) {

        case 'dpf':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') )  {

          $tag_group_dpf_template = get_option( 'tag_group_dpf_template', TagGroups_Premium_Shortcode_DPF::get_default_template() );


          $protocol = isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://';

          $ajax_link = admin_url( 'admin-ajax.php', $protocol );


          $ajax_template = json_encode( utf8_encode( TagGroups_Premium_Shortcode_DPF::get_default_template() ) );

          $view = new TagGroups_Premium_View( 'admin/settings_front_end_dpf' );

          $view->set( array(
            'tag_group_dpf_template'  => $tag_group_dpf_template,
            'ajax_link'               => $ajax_link,
            'ajax_template'           => $ajax_template
          ) );

          $view->render();

        }

        break;


        case 'themes-premium':

        $view = new TagGroups_Premium_View( 'admin/settings_front_end_themes_premium' );

        $view->render();

        break;


        case 'cache':

        $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

        $object_cache_options = array(
          0 => __( 'Off', 'tag-groups-premium' ),
          1 => __( 'Database', 'tag-groups-premium' ) . ' (' . __( 'recommended',  'tag-groups' ) . ')',
          3 => __( 'Filesystem', 'tag-groups-premium' ),
          9 => __( 'WP Object Cache', 'tag-groups-premium' ),
        );


        $view = new TagGroups_Premium_View( 'admin/settings_front_end_cache' );

        $view->set( array(
          'tag_group_object_cache'  => $tag_group_object_cache,
          'object_cache_options'    => $object_cache_options
        ) );

        $view->render();


        break;


        case 'meta-box':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          $tag_group_meta_box_change_group = get_option( 'tag_group_meta_box_change_group', 1 );

          $tag_group_meta_box_add_term = get_option( 'tag_group_meta_box_add_term', 1 );

          $tag_group_meta_box_open_all = get_option( 'tag_group_meta_box_open_all', 0 );

          $tag_group_hide_tagsdiv = get_option( 'tag_group_hide_tagsdiv', 1 );

          $tag_group_open_all_with_terms = get_option( 'tag_group_open_all_with_terms', 1 );

          $tag_group_meta_box_taxonomy = get_option( 'tag_group_meta_box_taxonomy', array() );

          $all_taxonomies = get_taxonomies( array( 'public' => true ), 'objects' );

          $non_hierarchical_taxonomies_names = array();

          foreach ( $all_taxonomies as $taxonomy ) {

            /**
            * We offer only non-hierarchical taxonomies
            */
            if ( ! empty( $taxonomy ) && is_object( $taxonomy ) && ! $taxonomy->hierarchical ) {

              $non_hierarchical_taxonomies_names[] = $taxonomy->name;

            }

          }

          $taxonomies = array_intersect( get_option( 'tag_group_taxonomy', array() ), $non_hierarchical_taxonomies_names );

          $tag_group_meta_box_include = get_option( 'tag_group_meta_box_include', $tag_group_groups->get_group_ids() );


          $view = new TagGroups_Premium_View( 'admin/settings_back_end_meta_box' );

          $view->set( array(
            'taxonomies'                        => $taxonomies,
            'tag_groups'                        => $tag_group_groups->get_all_with_position_as_key(),
            'tag_group_meta_box_include'        => $tag_group_meta_box_include,
            'tag_group_meta_box_change_group'   => $tag_group_meta_box_change_group,
            'tag_group_meta_box_add_term'       => $tag_group_meta_box_add_term,
            'tag_group_meta_box_open_all'       => $tag_group_meta_box_open_all,
            'tag_group_hide_tagsdiv'            => $tag_group_hide_tagsdiv,
            'tag_group_open_all_with_terms'     => $tag_group_open_all_with_terms,
            'tag_group_meta_box_taxonomy'       => $tag_group_meta_box_taxonomy
          ) );

          $view->render();

        }

        break;


        case 'post-tags':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          $groups = $tag_group_groups->get_all_with_position_as_key();

          $tag_group_display_groups_under_posts = get_option( 'tag_group_display_groups_under_posts', $tag_group_groups->get_group_ids() );

          $tag_group_display_groups_under_posts_title = get_option( 'tag_group_display_groups_under_posts_title', __( 'Tags', 'tag-groups-premium' ) );

          $tag_group_display_groups_under_posts_separator = get_option( 'tag_group_display_groups_under_posts_separator', '&nbsp;|&nbsp;' );

          $tag_group_display_groups_under_posts_single = get_option( 'tag_group_display_groups_under_posts_single', false );

          $tag_group_display_groups_under_posts_home = get_option( 'tag_group_display_groups_under_posts_home', false );

          $tag_group_display_groups_under_posts_archive = get_option( 'tag_group_display_groups_under_posts_archive', false );

          $tag_group_display_groups_under_posts_feed = get_option( 'tag_group_display_groups_under_posts_feed', false );

          $tag_group_display_groups_under_posts_priority = get_option( 'tag_group_display_groups_under_posts_priority', 10 );

          $tag_group_remove_the_post_terms = get_option( 'tag_group_remove_the_post_terms', false );


          $view = new TagGroups_Premium_View( 'admin/settings_front_end_post_tags' );

          $view->set( array(
            'groups'                                            => $groups,
            'tag_group_display_groups_under_posts'              => $tag_group_display_groups_under_posts,
            'tag_group_display_groups_under_posts_title'        => $tag_group_display_groups_under_posts_title,
            'tag_group_display_groups_under_posts_separator'    => $tag_group_display_groups_under_posts_separator,
            'tag_group_display_groups_under_posts_single'       => $tag_group_display_groups_under_posts_single,
            'tag_group_display_groups_under_posts_home'         => $tag_group_display_groups_under_posts_home,
            'tag_group_display_groups_under_posts_archive'      => $tag_group_display_groups_under_posts_archive,
            'tag_group_display_groups_under_posts_feed'         => $tag_group_display_groups_under_posts_feed,
            'tag_group_display_groups_under_posts_priority'     => $tag_group_display_groups_under_posts_priority,
            'tag_group_remove_the_post_terms'                   => $tag_group_remove_the_post_terms
          ) );

          $view->render();

        }

        break;


        case 'woocommerce':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          $tag_group_add_product_tags_to_attributes = get_option( 'tag_group_add_product_tags_to_attributes', false );

          $tag_group_remove_the_product_tags = get_option( 'tag_group_remove_the_product_tags', false );

          $tag_group_display_groups_under_products_separator = get_option( 'tag_group_display_groups_under_products_separator', ', ' );

          $view = new TagGroups_Premium_View( 'admin/settings_back_end_woocommerce' );

          $view->set( array(
            'tag_group_add_product_tags_to_attributes'            => $tag_group_add_product_tags_to_attributes,
            'tag_group_remove_the_product_tags'                   => $tag_group_remove_the_product_tags,
            'tag_group_display_groups_under_products_separator'   => $tag_group_display_groups_under_products_separator
          ) );

          $view->render();

        }

        break;


        case 'permissions':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          /**
          * Permissions to edit tag groups
          */
          $tag_group_role_edit_groups = get_option( 'tag_group_role_edit_groups', 'edit_pages' );

          /**
          * Permissions to edit groups on tag page
          */
          $tag_group_role_edit_tags = get_option( 'tag_group_role_edit_tags', 'manage_options' );

          /**
          * Permissions to override restrictions in Meta Box (adding new tags and moving tags to another group)
          */
          $tag_group_role_mb_override = get_option( 'tag_group_role_mb_override', 'no_one' );


          $edit_group_options = array(
            'manage_options' => __( 'Administrators', 'tag-groups-premium' ),
            'edit_pages' => __( 'Editors', 'tag-groups-premium' ),
            'publish_posts' => __( 'Authors', 'tag-groups-premium' ),
          );

          $edit_tags_options = array(
            'manage_options' => __( 'Administrators', 'tag-groups-premium' ),
            'edit_pages' => __( 'Editors', 'tag-groups-premium' ),
            'publish_posts' => __( 'Authors', 'tag-groups-premium' ),
          );

          $mb_override_options = array(
            'no_one' => __( 'No one', 'tag-groups-premium' ),
            'manage_options' => __( 'Administrators', 'tag-groups-premium' ),
            'edit_pages' => __( 'Editors', 'tag-groups-premium' ),
            'publish_posts' => __( 'Authors', 'tag-groups-premium' ),
          );

          $view = new TagGroups_Premium_View( 'admin/settings_back_end_permissions' );

          $view->set( array(
            'tag_group_role_edit_groups'    => $tag_group_role_edit_groups,
            'tag_group_role_edit_tags'      => $tag_group_role_edit_tags,
            'tag_group_role_mb_override'    => $tag_group_role_mb_override,
            'edit_group_options'            => $edit_group_options,
            'edit_tags_options'             => $edit_tags_options,
            'mb_override_options'           => $mb_override_options
          ) );

          $view->render();

        }

        break;


        case 'maintenance':

        if ( ! empty( $_GET['process-tasks'] ) ) {

          self::add_html_process();

        } else {

          $view = new TagGroups_Premium_View( 'admin/settings_tools_maintenance' );

          $view->set( 'tasks', 'fixgroups,fixmissinggroups,fixpostterms' );

          $view->render();

        }

        break;


        case 'info-premium':

        $tag_group_premium_first_activation_time = get_option( 'tag_group_premium_first_activation_time', false );

        if ( $tag_group_premium_first_activation_time && ( time() - $tag_group_premium_first_activation_time > 60*60*24*7 ) ) {

          $show_testimonial_appeal = true;

        } else {

          $show_testimonial_appeal = false;

        }

        $view = new TagGroups_Premium_View( 'admin/settings_about_info_premium' );

        $view->set( 'show_testimonial_appeal', $show_testimonial_appeal );

        $view->render();

        break;


        case 'licenses-premium':

        $view = new TagGroups_Premium_View( 'admin/settings_about_licenses_premium' );

        $view->render();

        break;


        case 'news-premium':

        $protocol = isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://';

        $admin_url = admin_url( 'admin-ajax.php', $protocol );

        $view = new TagGroups_Premium_View( 'admin/settings_about_news_premium' );

        $view->set( 'admin_url', $admin_url );

        $view->render();

        break;


        case 'first-aid-premium':

        if ( ! empty( $_GET['process-tasks'] ) ) {

          self::add_html_process();

        } else {

          $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

          $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_first_aid' );

          $view->set( 'tag_group_object_cache', $tag_group_object_cache );

          $view->set( 'tasks_migration', 'termmetacomma,migratetermmeta,postmetacomma,migratepostmeta' );

          $view->set( 'tasks_maintenance', 'fixgroups,fixmissinggroups,fixpostterms' );

          $view->set( 'tasks_rebuild', 'rebuildpostcount' );

          $view->set( 'tag_group_show_filter_tags', get_option( 'tag_group_show_filter_tags', true ) );

          $view->render();

        }

        break;


        case 'faq-premium':

        $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_faq_premium' );

        $view->render();

        break;


        case 'documentation-premium':

        $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_documentation_premium' );

        $view->set( 'tag_groups_premium_fs_sdk', $tag_groups_premium_fs_sdk );

        $view->render();

        break;


        case 'support-premium':

        $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_support_premium' );

        $view->render();

        break;

      }

    }


    /**
    * Process return data from forms of premium settings
    *
    * @param string @action
    * @return void
    */
    static function premium_settings_action( $action ) {

      global $tag_groups_premium_fs_sdk, $tag_group_groups, $tag_group_terms;

      switch ( $action ) {

        case 'woocommerce':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          // check nonce
          if ( ! isset( $_POST['tag-groups-woocommerce-nonce'] ) ||
          ! wp_verify_nonce( $_POST['tag-groups-woocommerce-nonce'], 'tag-groups-woocommerce' ) ) {

            die( "Security check" );

          }

          if ( ! empty( $_POST['tag_group_add_product_tags_to_attributes'] ) ) {

            update_option( 'tag_group_add_product_tags_to_attributes', 1 );

          } else {

            update_option( 'tag_group_add_product_tags_to_attributes', 0 );

          }


          if ( ! empty( $_POST['tag_group_remove_the_product_tags'] ) ) {

            update_option( 'tag_group_remove_the_product_tags', 1 );

          } else {

            update_option( 'tag_group_remove_the_product_tags', 0 );

          }


          if ( isset( $_POST['tag_group_display_groups_under_products_separator'] ) ) {

            $tag_group_display_groups_under_products_separator = wp_kses( $_POST['tag_group_display_groups_under_products_separator'], array() ) ; // sanitize_textarea_field() trims spaces

          } else {

            $tag_group_display_groups_under_products_separator = ', ';

          }

          update_option( 'tag_group_display_groups_under_products_separator', $tag_group_display_groups_under_products_separator );


          TagGroups_Admin_Notice::add( 'success', __( 'Your settings have been saved.', 'tag-groups-premium' ) );

        }

        do_action( 'tag_groups_woocommerce_saved' );

        break;


        case 'post-tags':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          // check nonce
          if ( ! isset( $_POST['tag-groups-post-tags-nonce'] ) ||
          ! wp_verify_nonce( $_POST['tag-groups-post-tags-nonce'], 'tag-groups-post-tags' ) ) {

            die( "Security check" );

          }


          if ( isset( $_POST['tag_group_display_groups_under_posts_title'] ) ) {

            $tag_group_display_groups_under_posts_title = sanitize_text_field( $_POST['tag_group_display_groups_under_posts_title'] ) ;

          } else {

            $tag_group_display_groups_under_posts_title = __( 'Tags', 'tag-groups-premium' );

          }

          update_option( 'tag_group_display_groups_under_posts_title', $tag_group_display_groups_under_posts_title );


          if ( isset( $_POST['tag_group_display_groups_under_posts_priority'] ) ) {

            $tag_group_display_groups_under_posts_priority = intval( $_POST['tag_group_display_groups_under_posts_priority'] ) ;

          } else {

            $tag_group_display_groups_under_posts_priority = 10;

          }

          update_option( 'tag_group_display_groups_under_posts_priority', $tag_group_display_groups_under_posts_priority );


          if ( isset( $_POST['tag_group_display_groups_under_posts_separator'] ) ) {

            // preserving space
            $tag_group_display_groups_under_posts_separator = wp_kses( $_POST['tag_group_display_groups_under_posts_separator'], array() ) ; // sanitize_textarea_field() trims spaces

          } else {

            $tag_group_display_groups_under_posts_separator = '&nbsp;|&nbsp;';

          }

          update_option( 'tag_group_display_groups_under_posts_separator', $tag_group_display_groups_under_posts_separator );


          if ( isset( $_POST['tag_group_display_groups_under_posts'] ) ) {

            $tag_group_display_groups_under_posts = array_map( 'intval', array_values( $_POST['tag_group_display_groups_under_posts'] ) );

          } else {

            $tag_group_display_groups_under_posts = array();

          }
          update_option( 'tag_group_display_groups_under_posts', $tag_group_display_groups_under_posts );


          if ( isset( $_POST['tag_group_display_groups_under_posts_single'] ) ) {

            $tag_group_display_groups_under_posts_single = intval( $_POST['tag_group_display_groups_under_posts_single'] );

          } else {

            $tag_group_display_groups_under_posts_single = array();

          }
          update_option( 'tag_group_display_groups_under_posts_single', $tag_group_display_groups_under_posts_single );


          if ( isset( $_POST['tag_group_display_groups_under_posts_home'] ) ) {

            $tag_group_display_groups_under_posts_home = intval( $_POST['tag_group_display_groups_under_posts_home'] );

          } else {

            $tag_group_display_groups_under_posts_home = array();

          }
          update_option( 'tag_group_display_groups_under_posts_home', $tag_group_display_groups_under_posts_home );


          if ( isset( $_POST['tag_group_display_groups_under_posts_archive'] ) ) {

            $tag_group_display_groups_under_posts_archive = intval( $_POST['tag_group_display_groups_under_posts_archive'] );

          } else {

            $tag_group_display_groups_under_posts_archive = array();

          }
          update_option( 'tag_group_display_groups_under_posts_archive', $tag_group_display_groups_under_posts_archive );


          if ( isset( $_POST['tag_group_display_groups_under_posts_feed'] ) ) {

            $tag_group_display_groups_under_posts_feed = intval( $_POST['tag_group_display_groups_under_posts_feed'] );

          } else {

            $tag_group_display_groups_under_posts_feed = array();

          }
          update_option( 'tag_group_display_groups_under_posts_feed', $tag_group_display_groups_under_posts_feed );


          if ( isset( $_POST['tag_group_remove_the_post_terms'] ) ) {

            $tag_group_remove_the_post_terms = intval( $_POST['tag_group_remove_the_post_terms'] );

          } else {

            $tag_group_remove_the_post_terms = array();

          }
          update_option( 'tag_group_remove_the_post_terms', $tag_group_remove_the_post_terms );

          TagGroups_Admin_Notice::add( 'success', __( 'Your settings have been saved.', 'tag-groups-premium' ) );

        }

        do_action( 'tag_groups_post_tags_saved' );

        break;


        case 'permissions':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          // check nonce
          if ( ! isset( $_POST['tag-groups-permissions-nonce'] ) ||
          ! wp_verify_nonce( $_POST['tag-groups-permissions-nonce'], 'tag-groups-permissions' ) ) {

            die( "Security check" );

          }

          /**
          * Permission to use the tag groups page
          */
          if ( isset( $_POST['tag_group_role_edit_groups'] ) ) {

            $tag_group_role_edit_groups = sanitize_title( $_POST['tag_group_role_edit_groups'] );

          } else {

            $tag_group_role_edit_groups = 'manage_options'; // use the most restrictive if the input cannot be understood

          }
          update_option( 'tag_group_role_edit_groups', $tag_group_role_edit_groups );

          /**
          * Permission to edit groups of tags
          */
          if ( isset( $_POST['tag_group_role_edit_tags'] ) ) {

            $tag_group_role_edit_tags = sanitize_title( $_POST['tag_group_role_edit_tags'] );

          } else {

            $tag_group_role_edit_tags = 'manage_options'; // use the most restrictive if the input cannot be understood

          }
          update_option( 'tag_group_role_edit_tags', $tag_group_role_edit_tags );

          /**
          * Permission to override the Meta Box restrictions
          */
          if ( isset( $_POST['tag_group_role_mb_override'] ) ) {

            $tag_group_role_mb_override = sanitize_title( $_POST['tag_group_role_mb_override'] );

          } else {

            $tag_group_role_mb_override = 'no_one';

          }
          update_option( 'tag_group_role_mb_override', $tag_group_role_mb_override );


          TagGroups_Admin_Notice::add( 'success', __( 'Your settings have been saved.', 'tag-groups-premium' ) );

        }

        do_action( 'tag_groups_permissions_saved' );

        break;


        case 'meta-box':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          // check nonce
          if ( ! isset( $_POST['tag-groups-meta-box-nonce'] ) ||
          ! wp_verify_nonce( $_POST['tag-groups-meta-box-nonce'], 'tag-groups-meta-box' ) ) {

            die( "Security check" );

          }

          if ( isset( $_POST['taxonomies'] ) ) {

            $selected_taxonomies = $_POST['taxonomies'];

          } else {

            $selected_taxonomies = array();

          }

          $taxonomies = TagGroups_Taxonomy::get_public_taxonomies();

          foreach ( $selected_taxonomies as $taxonomy_item ) {

            $taxonomy_item = stripslashes( sanitize_text_field( $taxonomy_item ) );

            if ( ! in_array( $taxonomy_item, $taxonomies ) ) {

              die( "Security check: taxonomies" );

            }
          }
          update_option( 'tag_group_meta_box_taxonomy', $selected_taxonomies );


          if ( isset( $_POST['tag_group_meta_box_include'] ) ) {

            $tag_group_meta_box_include = array_map( 'intval', array_values( $_POST['tag_group_meta_box_include'] ) );

          } else {

            // We don't allow here to select none

            $tag_group_meta_box_include = $tag_group_groups->get_group_ids();

          }
          update_option( 'tag_group_meta_box_include', $tag_group_meta_box_include );


          if ( isset( $_POST['tag_group_meta_box_open_all'] ) ) {

            $tag_group_meta_box_open_all = $_POST['tag_group_meta_box_open_all'] ? 1 : 0;

          } else {

            $tag_group_meta_box_open_all = 0;

          }
          update_option( 'tag_group_meta_box_open_all', $tag_group_meta_box_open_all );


          if ( isset( $_POST['tag_group_meta_box_add_term'] ) ) {

            $tag_group_meta_box_add_term = $_POST['tag_group_meta_box_add_term'] ? 1 : 0;

          } else {

            $tag_group_meta_box_add_term = 0;

          }
          update_option( 'tag_group_meta_box_add_term', $tag_group_meta_box_add_term );


          if ( isset( $_POST['tag_group_hide_tagsdiv'] ) ) {

            $tag_group_hide_tagsdiv = $_POST['tag_group_hide_tagsdiv'] ? 1 : 0;

          } else {

            $tag_group_hide_tagsdiv = 0;

          }
          update_option( 'tag_group_hide_tagsdiv', $tag_group_hide_tagsdiv );


          if ( isset( $_POST['tag_group_meta_box_change_group'] ) ) {

            $tag_group_meta_box_change_group = $_POST['tag_group_meta_box_change_group'] ? 1 : 0;

          } else {

            $tag_group_meta_box_change_group = 0;

          }
          update_option( 'tag_group_meta_box_change_group', $tag_group_meta_box_change_group );

          if ( isset( $_POST['tag_group_open_all_with_terms'] ) ) {

            $tag_group_open_all_with_terms = $_POST['tag_group_open_all_with_terms'] ? 1 : 0;

          } else {

            $tag_group_open_all_with_terms = 0;

          }
          $return = update_option( 'tag_group_open_all_with_terms', $tag_group_open_all_with_terms );


          TagGroups_Admin_Notice::add( 'success', __( 'Your settings have been saved.', 'tag-groups-premium' ) );

        }

        do_action( 'tag_groups_metabox_saved' );

        break;


        case 'object-cache':

        // check nonce
        if ( ! isset( $_POST['tag-groups-object-cache-nonce'] ) ||
        ! wp_verify_nonce( $_POST['tag-groups-object-cache-nonce'], 'tag-groups-object-cache' ) ) {
          die( "Security check" );
        }

        if ( isset( $_POST['object-cache'] )  ) {

          $tag_group_object_cache_before = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

          if ( $tag_group_object_cache_before != (int) $_POST['object-cache'] ) {

            TagGroups_Premium_Cache::purge();

          }

          update_option( 'tag_group_object_cache', (int) $_POST['object-cache'] );

          /**
          * Purge transients
          */
          $tag_group_terms->clear_term_cache();

          TagGroups_Premium::purge_all_transients();

        }

        TagGroups_Admin_Notice::add( 'success', __( 'Your settings have been saved.', 'tag-groups-premium' ) );

        do_action( 'tag_groups_object_cache_saved' );

        break;


        case 'dpf-template':

        if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

          // check nonce
          if ( ! isset( $_POST['tag-groups-dpf-template-nonce'] ) ||
          ! wp_verify_nonce( $_POST['tag-groups-dpf-template-nonce'], 'tag-groups-dpf-template' ) ) {

            die( "Security check" );

          }

          if ( isset( $_POST['tag_group_dpf_template'] )  ) {

            update_option( 'tag_group_dpf_template', TagGroups_Premium_Shortcode_DPF::sanitize_html_field( $_POST['tag_group_dpf_template'] ) );

          }

          TagGroups_Admin_Notice::add( 'success', __( 'Your settings have been saved.', 'tag-groups-premium' ) );

        }

        do_action( 'tag_groups_dpf_saved' );

        break;


        case 'purge-object-cache':

        // check nonce
        if ( ! isset( $_POST['tag-groups-purge-object-cache-nonce'] ) ||
        ! wp_verify_nonce( $_POST['tag-groups-purge-object-cache-nonce'], 'tag-groups-purge-object-cache' ) ) {

          die( "Security check" );

        }

        /**
        * Purge transients
        */
        $tag_group_terms->clear_term_cache();

        TagGroups_Premium::purge_all_transients();

        TagGroups_Premium_Cache::purge();


        TagGroups_Admin_Notice::add( 'success', __( 'Your cache has been purged.', 'tag-groups-premium' ) );

        break;

      }

    }


    /**
    * Returns the part of the setting page where the shortcodes are explained
    *
    * @param void
    * @return string
    */
    static function premium_shortcodes( $html ) {

      global $tag_groups_premium_fs_sdk;

      $view = new TagGroups_Premium_View( 'admin/settings_front_end_premium_shortcodes' );

      $view->set( 'tag_groups_premium_fs_sdk', $tag_groups_premium_fs_sdk );

      return $view->return_html();

    }


    /**
    * Modifies the list of searchable setting topics to comprise also premium topics
    *
    * @param array $topics
    * @return array
    */
    public static function tag_groups_setting_topics( $topics )
    {

      global $tag_groups_premium_fs_sdk;

      unset( $topics['premium'] );

      unset( $topics['faq'] );

      unset( $topics['documentation'] );

      unset( $topics['support'] );

      unset( $topics['info'] );

      unset( $topics['licenses'] );

      unset( $topics['news'] );

      unset( $topics['first-aid'] );


      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        $topics['shortcodes']['keywords'] = array_merge(
          $topics['shortcodes']['keywords'],
          array(
            __( 'dynamic post filter', 'tag-groups-premium' ),
            __( 'shuffle box', 'tag-groups-premium' ),
            __( 'table tag cloud', 'tag-groups-premium' ),
            __( 'combined tag cloud', 'tag-groups-premium' ),
            __( 'tag groups menu', 'tag-groups-premium' ),
            __( 'post list', 'tag-groups-premium' ),
          )
        );

      } else {

        $topics['shortcodes']['keywords'] = array_merge(
          $topics['shortcodes']['keywords'],
          array(
            __( 'shuffle box', 'tag-groups-premium' ),
            __( 'table tag cloud', 'tag-groups-premium' ),
            __( 'combined tag cloud', 'tag-groups-premium' ),
            __( 'tag groups menu', 'tag-groups-premium' ),
          )
        );

      }

      $topics['themes-premium'] = array(
        'title' => __( 'Other Themes', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-front-end' ,
        'keywords'  => array(
          __( 'tag cloud', 'tag-groups-premium' ),
          'CSS',
          'style',
          __( 'colors', 'tag-groups-premium' ),
        ),
      );

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        $topics['dpf'] = array(
          'title' => __( 'Dynamic Post Filter', 'tag-groups-premium' ),
          'page' => 'tag-groups-settings-front-end' ,
          'keywords'  => array(
            __( 'posts', 'tag-groups-premium' ),
            'ajax',
            __( 'template', 'tag-groups-premium' ),
          ),
        );

      }

      $topics['cache'] = array(
        'title' => __( 'Cache', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-front-end' ,
        'keywords'  => array(
          __( 'object cache', 'tag-groups-premium' ),
          'transient',
          __( 'speed', 'tag-groups-premium' ),
          __( 'performance', 'tag-groups-premium' ),
          // __( 'colors', 'tag-groups-premium' ),
        ),
      );

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        $topics['post-tags'] = array(
          'title' => __( 'Post Tags', 'tag-groups-premium' ),
          'page' => 'tag-groups-settings-front-end' ,
          'keywords'  => array(
          ),
        );

        if ( defined('WC_VERSION') ) {

          $topics['woocommerce'] = array(
            'title' => __( 'WooCommerce', 'tag-groups-premium' ),
            'page' => 'tag-groups-settings-front-end' ,
            'keywords'  => array(
              __( 'object cache', 'ecommerce' ),
              __( 'product tags', 'ecommerce' ),
              __( 'product attributes', 'ecommerce' ),
            ),
          );

        }

        $topics['meta-box'] = array(
          'title' => __( 'Meta Box', 'tag-groups-premium' ),
          'page' => 'tag-groups-settings-back-end' ,
          'keywords'  => array(
            __( 'tag input', 'tag-groups-premium' ),
            __( 'group menu', 'tag-groups-premium' ),
            __( 'post edit screen', 'tag-groups-premium' ),
            __( 'tag box', 'tag-groups-premium' ),
          ),
        );

        $topics['permissions'] = array(
          'title' => __( 'Permissions', 'tag-groups-premium' ),
          'page' => 'tag-groups-settings-back-end' ,
          'keywords'  => array(
            __( 'Meta Box', 'tag-groups-premium' ),
            __( 'restrict creation', 'tag-groups-premium' ),
            __( 'user role', 'tag-groups-premium' ),
            'metabox',
          ),
        );

      }

      $topics['maintenance'] = array(
        'title' => __( 'Maintenance', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-tools' ,
        'keywords'  => array(
          __( 'Troubleshooting', 'tag-groups-premium' ),
          __( 'check', 'tag-groups-premium' ),
          __( 'tool', 'tag-groups-premium' ),
        ),
      );

      $topics['first-aid-premium'] = array(
        'title' => __( 'First Aid', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-troubleshooting' ,
        'keywords'  => array(
          __( 'troubleshooting', 'tag-groups-premium' ),
          __( 'purge cache', 'tag-groups-premium' ),
          __( 'clean cache', 'tag-groups-premium' ),
          __( 'migrate', 'tag-groups-premium' ),
          __( 'missing posts', 'tag-groups-premium' ),
          __( 'help', 'tag-groups-premium' ),
          __( 'problem', 'tag-groups-premium' ),
          __( 'tag filter', 'tag-groups' ),
        ),
      );

      $topics['faq-premium'] = array(
        'title' => __( 'FAQ and Common Issues', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-troubleshooting' ,
        'keywords'  => array(
          __( 'frequently asked questions', 'tag-groups-premium' ),
          __( 'help', 'tag-groups-premium' ),
          __( 'bug', 'tag-groups-premium' ),
          __( 'problem', 'tag-groups-premium' ),
          __( 'troubleshooting', 'tag-groups-premium' ),
          __( 'support', 'tag-groups-premium' ),
        ),
      );

      $topics['documentation-premium'] = array(
        'title' => __( 'Documentation', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-troubleshooting' ,
        'keywords'  => array(
          __( 'instructions', 'tag-groups-premium' ),
          __( 'help', 'tag-groups-premium' ),
          __( 'problem', 'tag-groups-premium' ),
          __( 'troubleshooting', 'tag-groups-premium' ),
          __( 'support', 'tag-groups-premium' ),
          'Gutenberg',
          'CSS',
          'style',
          'PHP',
          'REST API'
        ),
      );

      $topics['support-premium'] = array(
        'title' => __( 'Get Support', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-troubleshooting' ,
        'keywords'  => array(
          __( 'support', 'tag-groups-premium' ),
          __( 'contact', 'tag-groups-premium' ),
          __( 'bug', 'tag-groups-premium' ),
          __( 'problem', 'tag-groups-premium' ),
          __( 'help', 'tag-groups-premium' ),
        ),
      );

      $topics['info-premium'] = array(
        'title' => __( 'Info', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-about' ,
        'keywords'  => array(
          __( 'author', 'tag-groups-premium' ),
          __( 'version', 'tag-groups-premium' ),
          __( 'contact', 'tag-groups-premium' ),
          __( 'about', 'tag-groups-premium' ),
          __( 'testimonial', 'tag-groups-premium' ),
        ),
      );

      $topics['licenses-premium'] = array(
        'title' => __( 'Licenses', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-about' ,
        'keywords'  => array(
          __( 'Credits', 'tag-groups-premium' ),
        ),
      );

      $topics['news-premium'] = array(
        'title' => __( 'Development News', 'tag-groups-premium' ),
        'page' => 'tag-groups-settings-about' ,
        'keywords'  => array(
          __( 'blog', 'tag-groups-premium' ),
          __( 'updates', 'tag-groups-premium' ),
        ),
      );

      return $topics;

    }


    /**
    * adds the processing of tasks
    *
    * @param void
    * @return string
    */
    public static function add_html_process()
    {

      global $tag_group_terms, $tag_group_premium_terms;

      $tasks_whitelist = array(
        'migratepostmeta'   => __( 'Migrating the post meta', 'tag-groups-premium' ),
        'migratetermmeta'   => __( 'Migrating the term meta', 'tag-groups-premium' ),
        'fixgroups'         => __( 'Fixing incorrect tag groups', 'tag-groups-premium' ),
        'fixpostterms'      => __( 'Fixing incorrect post tags', 'tag-groups-premium' ),
        'fixmissinggroups'  => __( 'Fixing incorrect groups in tags', 'tag-groups-premium' ),
        'postmetacomma'     => __( 'Migrating the post meta to the new format', 'tag-groups-premium' ),
        'termmetacomma'     => __( 'Migrating the term meta to the new format', 'tag-groups-premium' ),
        'checkbadterms'     => __( 'Checking the tag names', 'tag-groups-premium' ),
        'rebuildpostcount'  => __( 'Rebuilding the post counts', 'tag-groups-premium' ),
      );

      $progress = array();

      $totals = array();

      $languages = array();

      $tasks = explode( ',', $_GET['process-tasks'] );

      $tasks = array_map( 'sanitize_title', $tasks );

      $tasks = array_intersect( $tasks, array_keys( $tasks_whitelist ) );

      $task_html = '';

      foreach ( $tasks as $key => $task ) {

        if ( 'rebuildpostcount' == $task ) {

          $tag_group_terms->clear_term_cache( -1 ); // Don't rebuild via cron

          $languages_a = apply_filters( 'wpml_active_languages', NULL, '' );

          if ( empty( $languages_a ) ) {

            $languages_a = array( 0 => 'all' );

          }

          foreach ( $languages_a as $language_code => $language_info ) {

            $totals[ $task . '_' . $language_code ] = TagGroups_Premium_Process::get_task_total( $task . '_' . $language_code, $language_code );

            $languages[ $task . '_' . $language_code ] = $language_code;

            $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_process_task_language' );

            $view->set( array(
              'task'                  => $task,
              'tasks_whitelist'       => $tasks_whitelist,
              'language_code'         => $language_code,
              'language_info'         => $language_info
            ) );

            $task_html .= $view->return_html();

          }

          unset( $tasks[ $key ] );

          foreach ( $languages_a as $language_code => $language_info ) {

            $tasks[] = $task . '_' . $language_code;

          }

        } else {

          $totals[ $task ] = TagGroups_Premium_Process::get_task_total( $task );


          $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_process_task_no_language' );

          $view->set( array(
            'task'            => $task,
            'tasks_whitelist' => $tasks_whitelist
          ) );

          $task_html .= $view->return_html();

        }

      }


      /**
      * Add the Javascript part that loops through the tasks and through the chunks within each task
      *
      * Keep some timeout between the calls so that we don't block the browser
      * Parameters: chunk length and timeouts for chunk and task
      */
      $protocol = isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://';

      $ajax_link = admin_url( 'admin-ajax.php', $protocol );

      if ( defined( 'TAG_GROUPS_CHUNK_SIZE' ) ) {

        $chunk_length = (int) TAG_GROUPS_CHUNK_SIZE;

      } else {

        $chunk_length = 30;

      }

      if ( defined( 'TAG_GROUPS_CHUNK_TIMEOUT' ) ) {

        $timeout_chunk = (int) TAG_GROUPS_CHUNK_TIMEOUT;

      } else {

        $timeout_chunk = 10 * 1000; // 10 seconds - for really slow networks

      }

      if ( defined( 'TAG_GROUPS_TASK_TIMEOUT' ) ) {

        $timeout_task = (int) TAG_GROUPS_TASK_TIMEOUT;

      } else {

        $timeout_task = 15 * 60 * 1000; // 15 minutes - can be long, but user needs to keep window open

      }


      $bad_terms = $tag_group_premium_terms->get_bad_terms();


      /**
      * The result messages will be revealed by the Javascript routine
      */
      $view = new TagGroups_Premium_View( 'admin/settings_troubleshooting_process' );

      $view->set( array(
        'task_html'       => $task_html,
        'ajax_link'       => $ajax_link,
        'tasks'           => $tasks,
        'totals'          => $totals,
        'languages'       => $languages,
        'timeout_task'    => $timeout_task,
        'timeout_chunk'   => $timeout_chunk,
        'chunk_length'    => $chunk_length,
        'bad_terms'       => $bad_terms
      ) );

      $view->render();

    }


  }

}
