<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
* @since      1.8.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Term' ) ) {

  class TagGroups_Premium_Term {

    /**
    * WP term_id
    *
    * @var int
    */
    private $term_id;

    /**
    * WP $taxonomy
    *
    * @var string|array
    */
    private $taxonomy;

    /**
    * term groups
    *
    * @var array
    */
    private $group_ids;



    function __construct( $term_id = null ) {

      if ( isset( $term_id ) ) {

        $this->term_id = $term_id;

        $this->load();

      }

    }


    /**
    * Get group data of a term, enabled for multiple groups
    *
    * @deprecated since 1.24.0
    * @param int $this->term_id
    * @return self
    */
    public function load() {

      $groups = get_term_meta( $this->term_id, '_cm_term_group_array', true );

      if ( '' == $groups ) { // not found

        return false;

      } else {

        $groups_a = explode( ',', $groups );

        // remove empty values
        $groups_a = array_filter( $groups_a, function( $v ) { return $v != ''; } );

        // must be ints and no funny keys
        $groups_a = array_values( array_map( 'intval', $groups_a ) );

        $this->group_ids = $groups_a;

      }

      return $this;

    }


    /**
    * Save group data of a term, enabled for multiple groups
    *
    * @deprecated since 1.24.0
    * @param int $this->term_id
    * @param array $groups Array of integers
    * @return self
    */
    public function save()
    {

      update_term_meta( $this->term_id, '_cm_term_group_array', ',' . implode( ',', $this->group_ids ) . ',' );

      /**
      * save also first element to field term_group of term, for backwards compatibility (downgrading from premium to base)
      */
      $args = array(
        'term_group' => reset( $this->group_ids ) // NOTE: $this->group_ids is shortened by first element
      );

      remove_all_actions('edit_terms');
      remove_all_actions('edited_terms');
      remove_all_actions('edit_term_taxonomy');
      remove_all_actions('edited_term_taxonomy');
      remove_all_actions('edit_term');
      remove_all_actions("edit_{$this->taxonomy}");
      remove_all_actions('edited_term');
      remove_all_actions("edited_{$this->taxonomy}");

      wp_update_term( $this->term_id, $this->taxonomy, $args );

      /**
      * Goes through all posts if the group information for this term is still correct
      */
      // now done through hook

      return $this;

    }


    /**
    * Get the value of WP $taxonomy
    *
    * @return string|array
    */
    public function get_taxonomy()
    {
      return $this->taxonomy;
    }

    /**
    * Set the value of WP $taxonomy
    *
    * @param string|array taxonomy
    *
    * @return self
    */
    public function set_taxonomy( $taxonomy )
    {
      $this->taxonomy = $taxonomy;

      return $this;
    }

    /**
    * Get the value of term groups
    *
    * @return array
    */
    public function get_group_ids()
    {
      return $this->group_ids;
    }

    /**
    * Set the value of term groups
    *
    * @param array group_ids
    *
    * @return self
    */
    public function set_group_ids( $group_ids )
    {
      $this->group_ids = $group_ids;

      return $this;
    }

  }

}
