<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists('TagGroups_Premium') ) {

  class TagGroups_Premium {

    /**
    * Array of classes of the base plugin that are required for this plugin
    *
    * @var array
    */
    private $required_class_names;

    /**
    * Whether all preconditions have been met
    *
    * @var boolean
    */
    public $is_launched;

    public function __construct() {

      // $this->required_class_names = array(
      //   'TagGroups_Shortcode',
      //   'ChattyMango_Cache',
      //   'TagGroups_Admin',
      //   'TagGroups_Base',
      //   'TagGroups_Term',
      //   'TagGroups_Group',
      //   'TagGroups_Taxonomy',
      //   'TagGroups_Feed'
      // );
      //
      // $this->is_launched = false;

      // $result = $this->check_preconditions();

      if ( $result !== false) {

        // $result->check_values()->register_hooks();

        $result->register_hooks();

        $this->is_launched = true;

      } else {

        return;

      }


    }



    /**
    *   Prevent errors that stem from wrong values, e.g. based on earlier bugs
    *
    * @deprecated should not really happen. Better fix in maintenance
    * @param void
    * @return void
    */
    public static function check_values() {

      $tag_group_meta_box_taxonomy = get_option( 'tag_group_meta_box_taxonomy', array() );

      // Prevent some weird errors
      if ( ! is_array( $tag_group_meta_box_taxonomy ) ) {

        update_option( 'tag_group_meta_box_taxonomy', array( $tag_group_meta_box_taxonomy ) );

      }

      return $this;

    }


    /**
    * Retrieves all transients created by Tag Groups Premium and deletes what is expired
    *
    * @param void
    * @return int
    */
    public static function purge_expired_transients()
    {

      global $wpdb;

      $count = 0;

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( '[Tag Groups Premium] Purging expired transients.' );

      }

      $transients = $wpdb->get_col(
        "
        SELECT REPLACE(option_name, '_transient_timeout_', '') AS transient_name
        FROM {$wpdb->options}
        WHERE option_name LIKE '_transient_timeout_tag_groups_%'
        AND option_value -1000000 < unix_timestamp()
        "
      );

      foreach( $transients as $transient ) {

        if ( false === get_transient( $transient ) ) {

          $count++;

        }

      }

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( sprintf( '[Tag Groups Premium] Purged %d expired transients.', $count ) );

      }

      return $count;

    }


    /**
    * Retrieves all transients created by Tag Groups Premium and deletes them
    *
    * @param void
    * @return int
    */
    public static function purge_all_transients()
    {

      global $wpdb;

      $count = 0;

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( '[Tag Groups Premium] Purging all transients.' );

      }

      $transients = $wpdb->get_col(
        "
        SELECT REPLACE(option_name, '_transient_', '') AS transient_name
        FROM {$wpdb->options}
        WHERE option_name LIKE '_transient_tag_groups_%'
        "
      );

      foreach( $transients as $transient ) {

        if ( true === delete_transient( $transient ) ) {

          $count++;

        }

      }

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( sprintf( '[Tag Groups Premium] Purged %d transients.', $count ) );

      }

      return $count;

    }


    /**
    * Retrieves all transients for post listings and deletes them
    *
    * @param void
    * @return int
    */
    public static function purge_post_list_transients( $post_id, $post )
    {

      global $wpdb;

      // omit autosaves
      if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { // apparently not equivalent to wp_is_post_autosave( $post_id )

        return $post_id;

      }

      // omit revisions
      if ( 'revision' == $post->post_type ) {

        return $post_id;

      }

      $count = 0;

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( '[Tag Groups Premium] Purging post transients.' );

      }

      // Dynamic Post Filter (Toggle)
      $transients = $wpdb->get_col(
        "
        SELECT REPLACE(option_name, '_transient_', '') AS transient_name
        FROM {$wpdb->options}
        WHERE option_name LIKE '_transient_tag_groups_dpf_%'
        "
      );

      foreach( $transients as $transient ) {

        if ( true === delete_transient( $transient ) ) {

          $count++;

        }

      }

      // Post List
      $transients = $wpdb->get_col(
        "
        SELECT REPLACE(option_name, '_transient_', '') AS transient_name
        FROM {$wpdb->options}
        WHERE option_name LIKE '_transient_tag_groups_post_list_%'
        "
      );

      foreach( $transients as $transient ) {

        if ( true === delete_transient( $transient ) ) {

          $count++;

        }

      }

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( sprintf( '[Tag Groups Premium] Purged %d post transients.', $count ) );

      }

      return $count;

    }


    /**
    * executes the routines to migrate posts from the base to the premium plugin
    *
    *
    * @param void
    * @return void
    */
    public static function run_post_migration()
    {

      global $tag_group_posts, $tag_group_terms;

      /**
      * We will need TagGroups_Admin to convert the taxonomies to post_types
      */
      if ( class_exists( 'TagGroups_Admin' ) ) {

        /**
        * Add term information to the post meta
        */
        if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

          error_log( '[Tag Groups Premium] Checking if posts need to be migrated.' );

          $start_time = microtime( true );

        }

        $count_post = $tag_group_posts->convert_to_post_meta();

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

          error_log( sprintf( '[Tag Groups Premium] %d post(s) migrated in %d milliseconds.', $count_post, round( ( microtime( true ) - $start_time ) * 1000 ) ) );

        }

        if ( $count_post > 0 ) {

          $tag_group_terms->clear_term_cache();

        }


        if ( ! defined( 'TAG_GROUPS_DISABLE_CACHE_REBUILD' ) || TAG_GROUPS_DISABLE_CACHE_REBUILD ) {

          // schedule rebuild of cache in the background - do it soon
          TagGroups_Premium_Cron::schedule_in_secs( 1, 'tag_groups_rebuild_post_counts' );

        }

      }

    }


  }

}
