<?php
/**
* Tag Groups Premium
*
* @package    Tag Groups Premium
* @author     Christoph Amthor
* @copyright  2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license    see official vendor website
* @since      1.10.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_API' ) ) {

  class TagGroups_Premium_API {

    /**
    * Displays the terms of a post in a table
    *
    *
    * @param type var Description
    * @return return type
    */
    public static function post_terms( $args = array() )
    {

      global $tag_group_groups;

      /**
      * parsing arguments and setting default values
      */
      if ( isset( $args['post_id'] ) ) {

        $post_id = intval( $args['post_id'] );

      } else {

        $post_id = get_the_ID();

      }

      if ( isset( $args['order'] ) ) {

        $order = sanitize_title( $args['order'] );

      } else {

        $order = 'asc';

      }

      if ( isset( $args['order_by'] ) ) {

        $order_by = sanitize_title( $args['order_by'] );

      } else {

        $order_by = 'natural';

      }

      if ( isset( $args['prepend'] ) ) {

        // preserving space
        $prepend = wp_kses( $args['prepend'], array() );

      } else {

        $prepend = '';

      }

      if ( isset( $args['separator'] ) ) {

        // preserving space
        $separator = wp_kses( $args['separator'], array() );

      } else {

        $separator = '&nbsp;|&nbsp;';

      }

      if ( isset( $args['include'] ) && is_array( $args['include'] ) ) {

        $include_array = array_map( 'intval', $args['include'] );

      } else {

        $include_array = array();

      }

      if ( isset( $args['return'] ) ) {

        $return = $args['return'];

      } else {

        $return = 'echo';

      }

      /**
      * constructing the output
      */
      if ( ! empty( $post_id ) ) {

        $tag_group_display_groups_under_posts_title = get_option( 'tag_group_display_groups_under_posts_title', __( 'Tags', 'tag-groups-premium' ) );

        $output_rows = '';

        $tg_post = new TagGroups_Premium_Post( $post_id );

        $groups = $tg_post->get_terms_by_group();

        $group_ids = $tag_group_groups->get_group_ids_by_position();

        foreach ( $group_ids as $group_id ) {

          if ( isset( $groups[ $group_id ] ) ) {

            $terms = $groups[ $group_id ];

          } else {

            $terms = array();

          }

          if ( ! empty( $include_array ) && ! in_array( $group_id, $include_array ) ) {

            continue;

          }

          if ( count( $terms ) > 0 ) {

            $tg_group = new TagGroups_Group( $group_id );

            $output_rows .= "<tr>\n";

            $output_rows .= "<td class='tag-groups-premium-post-tags-group'>" . $tg_group->get_label() . "</td>\n<td class='tag-groups-premium-post-tags'>";

            $first_in_group = true;

            $terms_name = array();

            $terms_array = array();

            foreach ( $terms as $term_id ) {

              $term = new TagGroups_Term( $term_id );

              $terms_array[] = array(
                'term_id'   =>  $term_id,
                'name'      =>  $term->get_name(),
                'taxonomy'  =>  $term->get_taxonomy(),
                'slug'      =>  $term->get_slug()
              );

            }

            if ( 'natural' == $order_by ) {

              $terms_array = self::natural_sorting( $terms_array, $order );

            }

            foreach ( $terms_array as $term ) {

              $tag_link = get_term_link( $term['slug'], $term['taxonomy'] );

              if ( ! is_string( $tag_link ) ) {

                continue;

              }

              if ( ! $first_in_group ) {

                $output_rows .= '<span class="tag-groups-post-tags-separator">' . $separator . '</span>';

              }

              $first_in_group = false;

              if ( ! isset( $args['add_premium_filter'] ) || $args['add_premium_filter'] ) {

                if ( mb_strpos( $tag_link, '?' ) === false ) {

                  $tag_link = esc_url( $tag_link . '?term_group=' . $group_id . '&term_id=' . $term['term_id'] );

                } else {

                  $tag_link = esc_url( $tag_link . '&term_group=' . $group_id . '&term_id=' . $term['term_id'] );

                }

              }

              $output_rows .= '<span class="tag-groups-post-tags-tag"><a href="' . $tag_link . '"><span class="tag-groups-post-tags-prepend">' . $prepend . '</span><span class="tag-groups-post-tags-label">' . $term['name'] . '</span></a></span>';

            }

            $output_rows .= "</td>\n</tr>\n";

          }

        }

        /**
        * Combining frame and rows
        */
        $output = '';

        if ( ! empty( $output_rows ) ) {


          $output .= "<div class='tag-groups-premium-post-tags-div'>\n<table class='tag-groups-premium-post-tags-table'>\n";

          if ( ! empty( $tag_group_display_groups_under_posts_title ) ) {

            $output .= "<h4 class='tag-groups-premium-post-tags-title'>" . $tag_group_display_groups_under_posts_title . "</h4>\n";

          }

          $output .= $output_rows;

          $output .= "</table>\n</div>\n";

        }

        /**
        * Add filter for customization
        */
        $output = apply_filters( 'tag_groups_post_terms', $output );

        if ( 'echo' === $return ) {

          echo $output;

        } else {

          return $output;

        }

      }

    }


    /**
    * Helper for natural sorting of names
    *
    * Inspired by _wp_object_name_sort_cb
    *
    * @param array $terms
    * @param string $order asc or desc
    * @return array
    */
    static function natural_sorting( $terms_name, $order )
    {
      $factor = ( 'desc' == strtolower( $order ) ) ? -1 : 1;

      // "use" requires PHP 5.3+
      uasort( $terms_name, function( $a, $b ) use ( $factor ) {

        return $factor * strnatcasecmp( $a['name'], $b['name'] );

      });

      $terms_name = array_values( $terms_name );

      return $terms_name;

    }


  }

}
