<?php
/**
* Tag Groups Premium
*
* @package    Tag Groups Premium
* @author     Christoph Amthor
* @copyright  2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license    see official vendor website
* @since      1.15.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/


if ( ! class_exists( 'TagGroups_Premium_Post_Group' ) ) {

  class TagGroups_Premium_Post_Group {

    /**
    * Group that corresponds to this WooCommerce object
    *
    * @var object
    */
    private $group;

    /**
    * Array of WP Term objects that correspond to the supplied list of IDs
    *
    * @var array
    */
    private $terms;


    /**
    *
    *
    * @param int $group_id ID of the tag group
    * @param array $term_ids List of tag IDs (usually post tags)
    * @return void
    */
    function __construct( $group_id, $term_ids ) {

      $this->group = new TagGroups_Group( $group_id );

      $this->terms = array();

      foreach ( $term_ids as $term_id ) {

        $this->terms[] = get_term( $term_id );

      }

    }


    /**
    * Dummy function that always returns false
    *
    * @param void
    * @return boolean
    */
    public function is_taxonomy()
    {
      return false;
    }


    /**
    * Returns the label of the group
    *
    * @param void
    * @return string
    */
    public function get_name()
    {
      return $this->group->get_label();
    }


    /**
    * Returns the terms as clickable links
    *
    * @param void
    * @return array
    */
    public function get_terms()
    {
      return $this->terms;
    }


    /**
    * Returns whether there are any terms saved in this object
    *
    * @param void
    * @return boolean
    */
    public function has_terms()
    {
      return empty( $this->terms ) ? false : true;
    }

  }

}
