<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Shortcode' ) ) {

  class TagGroups_Premium_Shortcode {

    /**
    * Register the shortcodes with WordPress: frontend
    *
    * @param void
    * @return void
    */
    static function register_frontend() {

      global $tag_groups_premium_fs_sdk;

      /**
      * Table tag cloud
      */
      add_shortcode( 'tag_groups_table', array( 'TagGroups_Premium_Shortcode_Table', 'tag_groups_table' ) );

      if ( function_exists( 'register_block_type' ) ) {

        register_block_type( 'chatty-mango/tag-groups-premium-cloud-table', array(
          'render_callback' => array( 'TagGroups_Premium_Shortcode_Table', 'tag_groups_table' ),
        ) );

      }

      /**
      * Combined tag cloud
      */
      add_shortcode( 'tag_groups_combined_cloud', array( 'TagGroups_Premium_Shortcode_Combined', 'tag_groups_combined_cloud' ) );

      if ( function_exists( 'register_block_type' ) ) {

        register_block_type( 'chatty-mango/tag-groups-premium-cloud-combined', array(
          'render_callback' => array( 'TagGroups_Premium_Shortcode_Combined', 'tag_groups_combined_cloud' ),
        ) );

      }

      /**
      * Shuffle Box
      */
      add_shortcode( 'tag_groups_shuffle_box', array( 'TagGroups_Premium_Shortcode_Shuffle_Box', 'tag_groups_shuffle_box') );

      if ( function_exists( 'register_block_type' ) ) {

        register_block_type( 'chatty-mango/tag-groups-premium-shuffle-box', array(
          'render_callback' => array( 'TagGroups_Premium_Shortcode_Shuffle_Box', 'tag_groups_shuffle_box' ),
        ) );

      }

      /**
      * Groups Menu
      */
      add_shortcode( 'tag_groups_menu', array( 'TagGroups_Premium_Shortcode_Menu', 'tag_groups_menu' ) );


      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        /**
        * Dynamic Post Filter
        */
        add_shortcode( 'tag_groups_dpf', array( 'TagGroups_Premium_Shortcode_DPF', 'tag_groups_dpf' ) );

        add_shortcode( 'tag_groups_dpf_body', array( 'TagGroups_Premium_Shortcode_DPF', 'tag_groups_dpf_body' ) );

        if ( function_exists( 'register_block_type' ) ) {

          register_block_type( 'chatty-mango/tag-groups-premium-dpf', array(
            'render_callback' => array( 'TagGroups_Premium_Shortcode_DPF', 'tag_groups_dpf' ),
          ) );

        }

        /**
        * Post List
        */
        add_shortcode( 'tag_groups_post_list', array( 'TagGroups_Premium_Shortcode_Post_List', 'tag_groups_post_list' ) );

        if ( function_exists( 'register_block_type' ) ) {

          register_block_type( 'chatty-mango/tag-groups-premium-post-filter', array(
            'render_callback' => array( 'TagGroups_Premium_Shortcode_Post_List', 'tag_groups_post_list' ),
          ) );

        }

        /**
        * Dynamic Post Filter with toggles
        */
        add_shortcode( 'tag_groups_dpf_toggle_menu', array( 'TagGroups_Premium_Shortcode_DPF_Toggle', 'tag_groups_dpf_toggle_menu' ) );

        add_shortcode( 'tag_groups_dpf_toggle_body', array( 'TagGroups_Premium_Shortcode_DPF_Toggle', 'tag_groups_dpf_toggle_body' ) );

        add_shortcode( 'tag_groups_dpf_toggle_messages', array( 'TagGroups_Premium_Shortcode_DPF_Toggle', 'tag_groups_dpf_toggle_messages' ) );

        if ( function_exists( 'register_block_type' ) ) {

          // not yet

        }


      }

    }


    /**
    * Register the shortcodes with WordPress: backend (used for AJAX calls)
    *
    * @param void
    * @return void
    */
    static function register_backend() {

      global $tag_groups_premium_fs_sdk;

      // each Ajax action for both logged-in and not-logged-in users
      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') )  {

        add_action( 'wp_ajax_nopriv_tg_ajax_get_posts', array( 'TagGroups_Premium_Shortcode_DPF', 'tg_ajax_get_posts' ) );

        add_action( 'wp_ajax_tg_ajax_get_posts', array( 'TagGroups_Premium_Shortcode_DPF', 'tg_ajax_get_posts' ) );

        add_action( 'wp_ajax_nopriv_tg_ajax_dpf_toggle_get_posts', array( 'TagGroups_Premium_Shortcode_DPF_Toggle', 'tg_ajax_dpf_toggle_get_posts' ) );

        add_action( 'wp_ajax_tg_ajax_dpf_toggle_get_posts', array( 'TagGroups_Premium_Shortcode_DPF_Toggle', 'tg_ajax_dpf_toggle_get_posts' ) );

        add_action( 'wp_ajax_nopriv_tg_ajax_get_terms_by_group', array( 'TagGroups_Premium_Meta_Box', 'ajax_get_terms_by_group' ) );

        add_action( 'wp_ajax_tg_ajax_get_terms_by_group', array( 'TagGroups_Premium_Meta_Box', 'ajax_get_terms_by_group' ) );

      }

    }


  } //class

}
