<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
* @since      1.8.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Terms' ) ) {

  class TagGroups_Premium_Terms {


    /**
    * Converts all WP-native term_group attributes to the term meta format, if no term meta format was found.
    * Term meta must use _cm_term_group_array as key.
    *
    * @deprecated since 1.24.0
    * @param void
    * @return int number of processed items
    */
    public function convert_to_term_meta( $count_only = false, $offset = null, $length = null )
    {

      if ( $count_only ) {
        // In case we restarted the tasks, we need to get fresh data

        $terms = false;


      } else {

        $terms = get_transient( 'tag_groups_convert_to_term_meta_terms' );

      }

      if ( $terms === false ) {

        /**
        * Process only those that don't have any meta
        */
        $args = array(
          'hide_empty'  => false,
          'taxonomy'  => TagGroups_Taxonomy::get_enabled_taxonomies(),
          'meta_query' => array(
            array(
              'key'     => '_cm_term_group_array',
              'compare' => 'NOT EXISTS'
            ),
          ),
        );

        $terms = get_terms( $args );

        // Try to keep for 10 minutes so that our offset always starts with unprocessed items.
        set_transient( 'tag_groups_convert_to_term_meta_terms', $terms, 600 );

      }

      if ( $count_only ) {

        if ( is_array( $terms ) ) {

          return count( $terms );

        } else {

          return 0;

        }

      }

      if ( isset( $offset ) && isset( $length ) ) {

        $terms = array_slice( $terms, $offset, $length );

      }

      $count = 0;

      if ( is_array( $terms ) ) {

        foreach ( $terms as $term ) {

          /**
          *  Fast way of saving: not necessary to use method save()
          */
          $result = update_term_meta( $term->term_id, '_cm_term_group_array', ',' . $term->term_group . ',' );

          if ( is_int( $result ) || $result === true ) {

            $count++;

          }

        }

      }

      return $count;

    }


    /**
    * Converts term meta to new format that is faster to process
    *
    * @param void
    * @return int number of processed items
    */
    public function term_meta_add_comma( $count_only = false, $offset = null, $length = null )
    {

      if ( $count_only ) {
        // In case we restarted the tasks, we need to get fresh data

        $term_ids = false;


      } else {

        $term_ids = get_transient( 'tag_groups_pterm_meta_add_comma_ids' );

      }


      if ( $term_ids === false ) {

        /**
        * Process only those that don't have any meta
        */
        $args_new = array(
          'hide_empty'  => false,
          'fields'      => 'ids',
          'taxonomy'    => TagGroups_Taxonomy::get_enabled_taxonomies(),
          'meta_query'  => array(
            'relation'  => 'AND',
            array(
              'key'     => '_cm_term_group_array',
              'compare' => 'EXISTS'
            ),
            array(
              'key'     => '_cm_term_group_array',
              'value'   => '^,.+,$',
              'compare' => 'REGEXP' // not possible to construct with LIKE and ',%,'
            ),
          ),
        );

        $args_all = array(
          'hide_empty'  => false,
          'fields'      => 'ids',
          'taxonomy'  => TagGroups_Taxonomy::get_enabled_taxonomies(),
          'meta_query'  => array(
            'relation'  => 'AND',
            array(
              'key'     => '_cm_term_group_array',
              'compare' => 'EXISTS'
            ),
          ),
        );

        $term_ids_new = get_terms( $args_new );

        $term_ids_all = get_terms( $args_all );

        $term_ids = array_diff( $term_ids_all, $term_ids_new );

        // Try to keep for 10 minutes so that our offset always starts with unprocessed items.
        set_transient( 'tag_groups_pterm_meta_add_comma_ids', $term_ids, 600 );

      }

      if ( $count_only ) {

        if ( is_array( $term_ids ) ) {

          return count( $term_ids );

        } else {

          return 0;

        }

      }

      if ( isset( $offset ) && isset( $length ) ) {

        $term_ids = array_slice( $term_ids, $offset, $length );

      }

      $count = 0;

      if ( is_array( $term_ids ) ) {

        foreach ( $term_ids as $term_id ) {

          $groups = get_term_meta( $term_id, '_cm_term_group_array', true );

          // Check if this is the old format without commas at beginning and end
          if ( substr( $groups, 0, 1 ) != ',' || substr( $groups, -1, 1 ) != ',' || $groups == ',,' ) {

            $groups_a = explode( ',', $groups );

            // remove empty values
            $groups_a = array_filter( $groups_a, function( $v ) { return $v != ''; } );

            if ( empty( $groups_a ) ) {

              $groups_a = array( 0 );

            }

            /**
            *  Fast way of saving: not necessary to use method save()
            */

            update_term_meta( $term_id, '_cm_term_group_array', ',' . implode( ',', $groups_a ) . ',' );

            $count++;

          }

        }

      }

      return $count;

    }


    /**
    * Modifies the query to show on term archives only posts that belong to the right tag group
    *
    *
    * @param object $query WP Query object
    * @return return type
    */
    public function term_archive_filter( $query )
    {

      if ( $query->is_main_query()  )  {

        if ( ! empty( $_GET[ 'term_group' ] ) && ! empty( $_GET[ 'term_id' ] ) ) {

          $term_group = (int) $_GET[ 'term_group' ];

          $term_id = (int) $_GET[ 'term_id' ];

          $query->set(
            'meta_query',
            array(
              array(
                'key' => '_cm_post_terms_' . $term_group,
                'value' => ',' . $term_id . ',', // WP adds %...% ; see https://core.trac.wordpress.org/browser/tags/5.1/src/wp-includes/class-wp-meta-query.php#L549
                'compare' => 'LIKE'
              )
            )
          );

        } elseif ( ! empty( $_GET[ 'term_group' ] ) ) {

          $term_group = (int) $_GET[ 'term_group' ];

          $query->set(
            'meta_query',
            array(
              array(
                'key' => '_cm_post_terms_' . $term_group,
                'compare' => 'EXISTS'
              )
            )
          );

        }

      }

      return $query;

    }


    // TODO: add unit test
    /**
    * Uses count_posts() to fill the transient cache of post counts
    *
    * @param void
    * @return void
    */
    public function fill_post_counts_transients()
    {

      global $sitepress;

      // Check if we have languages (WPML and Polylang)
      $languages = apply_filters( 'wpml_active_languages', NULL, '' );

      if ( ! empty( $languages ) ) {

        foreach ( $languages as $language_code => $language_info ) {

          $post_counts = $this->get_post_counts( $language_code );

        }

        if ( ! empty( $sitepress ) && is_object( $sitepress ) && defined( 'ICL_LANGUAGE_CODE' ) ) {

          $sitepress->switch_lang( ICL_LANGUAGE_CODE );

        }

      } else {

        $post_counts = $this->get_post_counts();

      }

    }


    /**
    * Returns how often a term is used for published posts, considering the group
    *
    * Wrapper that first checks if a transient can deliver the required information.
    * To replace $tag->count, because the count can be different for each group
    * Saves the result to the transient if it had to be created.
    *
    * @param string|null $language_code
    * @return array 2-dimensional Array of term ID and tag group ID
    */
    public function get_post_counts( $language_code = null )
    {

      $transient_value = get_transient( TagGroups_WPML::get_tag_groups_post_counts_transient_name( $language_code ) );

      if ( $transient_value === false) {

        $post_counts = $this->count_posts( $language_code );

        set_transient( TagGroups_WPML::get_tag_groups_post_counts_transient_name( $language_code ), $post_counts );

        return $post_counts;

      } else {

        return $transient_value;

      }

    }


    // TODO: add unit test
    /**
    * Returns how often a term is used for published posts, considering the group
    *
    * Does the actual counting
    *
    * @param string $language_code
    * @return array 2-dimensional Array of term ID and tag group ID
    */
    public function count_posts( $language_code = null ) {

      global $tag_group_groups, $sitepress;

      $post_counts = array();


      /**
      * 1. get all IDs of published posts
      */
      $transient_key = 'tag_groups_count_posts';

      if ( ! empty( $language_code ) ) {

        $transient_key .= '_' . $language_code;

      }

      $post_ids = get_transient( $transient_key );

      if ( $post_ids === false ) {

        /**
        * We need an array with numeric indices, because wp-includes/taxonomy.php will search for $taxonomy[0]
        */
        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        $post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $enabled_taxonomies );

        $args =
        array(
          'post_type'   => $post_types,
          'numberposts' => -1,
          'posts_per_page' => -1,
          'post_status' => array( 'publish' ),
          'fields'      => 'ids'
        );


        // If this code is run via cron, we need to set the language manually
        if ( ! empty( $language_code ) ) {

          if ( ! empty( $sitepress ) && is_object( $sitepress ) ) {

            $sitepress->switch_lang( $language_code );

          } elseif ( function_exists( 'pll_current_language') ) {

            $args['lang'] = $language_code;

          }

        }


        $query = new WP_Query( $args );

        $post_ids = $query->posts;

        set_transient( $transient_key, $post_ids, 180 );

      }


      /**
      * 2. get all term groups
      */
      $all_term_groups = $tag_group_groups->get_group_ids();


      if ( ! empty( $post_ids ) && is_array( $post_ids ) ) {

        /**
        * We are reusing the object so that we needn't load the transients again
        */

        $post_o = new TagGroups_Premium_Post();

        foreach ( $post_ids as $post_id ) {

          $terms_by_group = $post_o->set_post_id( $post_id )->load()->get_terms_by_group();

          foreach( $terms_by_group as $group => $terms ) {

            foreach ( $terms as $term ) {

              if ( ! isset( $post_counts[ $term ] ) ) {

                $post_counts[ $term ] = array();

              }

              if ( ! isset( $post_counts[ $term ][ $group ] ) ) {

                $post_counts[ $term ][ $group ] = 0;

              }

              $post_counts[ $term ][ $group ]++;

            }

          }

        }

        $post_o->save_transient();

      }

      return $post_counts;

    }


    // TODO: add unit test
    /**
    * Returns how often a term is used for published posts, considering the group
    *
    * Does the actual counting
    *
    * @param string $language_code
    * @return int Number of posts processed
    */
    public static function count_posts_resumable( $language_code = null, $count_only = false, $offset = null, $length = null ) {

      global $tag_group_groups, $sitepress;

      $count = 0;

      if ( empty( $length ) || $count_only ) {

        $post_counts = array();

      } else {

        $post_counts = get_transient( TagGroups_WPML::get_tag_groups_post_counts_transient_name( $language_code ) );

      }


      /**
      * 1. get all IDs of published posts
      */
      $transient_key = 'tag_groups_count_posts_resumable';

      if ( ! empty( $language_code ) ) {

        $transient_key .= '_' . $language_code;

      }

      if ( $count_only ) {

        $post_ids = false;

      } else {

        $post_ids = get_transient( $transient_key );

      }

      if ( $post_ids === false ) {

        /**
        * We need an array with numeric indices, because wp-includes/taxonomy.php will search for $taxonomy[0]
        */
        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        $post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $enabled_taxonomies );

        $args =
        array(
          'post_type'   => $post_types,
          'numberposts' => -1,
          'posts_per_page' => -1,
          'post_status' => array( 'publish' ),
          'fields'      => 'ids'
        );


        // If this code is run via cron, we need to set the language manually
        if ( ! empty( $language_code ) ) {

          if ( ! empty( $sitepress ) && is_object( $sitepress ) ) {

            $sitepress->switch_lang( $language_code );

          } elseif ( function_exists( 'pll_current_language') ) {

            $args['lang'] = $language_code;

          }

        }


        $query = new WP_Query( $args );

        $post_ids = $query->posts;

        // Try to keep for 10 minutes so that our offset always starts with unprocessed items.
        set_transient( $transient_key, $post_ids, 600 );

      }

      if ( $count_only ) {

        return count( $post_ids );

      }

      if ( isset( $offset ) && isset( $length ) ) {

        $post_ids = array_slice( $post_ids, $offset, $length );

      }

      /**
      * 2. get all term groups
      */
      $all_term_groups = $tag_group_groups->get_group_ids_by_position();


      if ( ! empty( $post_ids ) && is_array( $post_ids ) ) {

        /**
        * We are reusing the object so that we needn't load the transients again
        */

        $post_o = new TagGroups_Premium_Post();

        foreach ( $post_ids as $post_id ) {

          $count++;

          $terms_by_group = $post_o->set_post_id( $post_id )->load()->get_terms_by_group();

          foreach( $terms_by_group as $group => $terms ) {

            foreach ( $terms as $term ) {

              if ( ! isset( $post_counts[ $term ] ) ) {

                $post_counts[ $term ] = array();

              }

              if ( ! isset( $post_counts[ $term ][ $group ] ) ) {

                $post_counts[ $term ][ $group ] = 0;

              }

              $post_counts[ $term ][ $group ]++;

            }

          }

        }

        $post_o->save_transient();

      }

      set_transient( TagGroups_WPML::get_tag_groups_post_counts_transient_name( $language_code ), $post_counts );

      return $count;

    }


    /**
    * Clear the transient cache tag_groups_post_counts
    *
    *
    * @param integer $group_id
    * @return void
    */
    public function renew_tag_groups_post_counts_cache( $group_id = null )
    {

      $languages = apply_filters( 'wpml_active_languages', NULL, '' );

      if ( ! empty( $languages ) ) {

        foreach ( $languages as $language_code => $language_info ) {

          delete_transient( 'tag_groups_post_counts-' . $language_code );

        }

      } else {

        delete_transient( 'tag_groups_post_counts' );

      }


      if ( ! defined( 'TAG_GROUPS_DISABLE_CACHE_REBUILD' ) || TAG_GROUPS_DISABLE_CACHE_REBUILD ) {

        // schedule rebuild of cache
        return TagGroups_Premium_Cron::schedule_in_secs( 120, 'tag_groups_rebuild_post_counts' );

      }

      return true;

    }


    /**
    * Finds terms that might cause some trouble
    *
    *
    * @param void
    * @return array
    */
    public static function get_bad_terms()
    {

      $bad_terms = array();

      remove_all_actions( 'parse_query ' );

      $terms = get_terms( array(
        'taxonomy'  => TagGroups_Taxonomy::get_enabled_taxonomies(),
        'hide_empty' => false,
        'fields'  => 'names'
      ) );

      foreach ( $terms as $term ) {

        $term_decoded = html_entity_decode( $term );

        if (
          strpos( $term_decoded, ',' ) !== false
          || strpos( $term_decoded, ';' ) !== false
        ) {

          $bad_terms[] = $term;

        }

      }

      return $bad_terms;

    }

  }

}
