<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Shortcode_Combined' ) ) {

  class TagGroups_Premium_Shortcode_Combined extends TagGroups_Shortcode {


    /**
    * Shortcode that outputs the tags in a combined cloud
    *
    * @param array $atts
    * @return string
    */
    static function tag_groups_combined_cloud( $atts = array() ) {

      global $tag_group_groups, $tag_group_premium_terms;

      $shortcode_id = 'tag_groups_combined_cloud';

      extract( shortcode_atts( array(
        'adjust_separator_size' => false,
        'amount' => 0,
        'append' => '',
        'assigned_class' => null,
        'custom_title' => null,
        'exclude_terms' => '',
        'groups_post_id' => -1,
        'hide_empty' => true,
        'include' => '',
        'include_terms' => '',
        'largest' => 22,
        'link_target' => '',
        'link_append' => '',
        'not_assigned_name' => 'not assigned',
        'order' => 'ASC',
        'orderby' => 'name',
        'prepend' => '',
        'separator_size' => 12,
        'separator' => '',
        'show_not_assigned' => false,
        'show_tag_count' => true,
        'smallest' => 12,
        'source' => 'shortcode',
        'div_id' => null,
        'div_class' => 'tag-groups-cloud-combined',
        'tags_post_id' => -1,
        'taxonomy' => null,
      ), $atts ) );

      $div_id_output = $div_id ? ' id="' . TagGroups_Base::sanitize_html_classes( $div_id ) . '"' : '';

      $div_class_output = $div_class ? ' class="' . TagGroups_Base::sanitize_html_classes( $div_class ) . '"' : ' class="tg_table_align_top"';

      if ( is_array( $atts ) ) {

        asort( $atts );

      }


      /*
      *  applying parameter tags_post_id
      */
      if ( $tags_post_id == 0 ) {

        // retrieve the current post ID
        $tags_post_id = get_the_ID();

      }

      /*
      *  applying parameter groups_post_id
      */
      if ( $groups_post_id == 0 ) {

        $groups_post_id = get_the_ID();

      }

      $cache_key = md5( 'combined' . serialize( $atts ) . serialize( $tags_post_id ) . serialize( $groups_post_id ) );

      // check for a cached version (premium plugin)
      $html = apply_filters( 'tag_groups_hook_cache_get', false, $cache_key );

      if ( ! $html ) {

        $include_tags_post_id_groups = array();

        $assigned_terms = array();

        $tag_group_ids = $tag_group_groups->get_group_ids_by_position();


        if ( 'shortcode' == $source ) {

          $prepend = html_entity_decode( $prepend );

          $append = html_entity_decode( $append );

          $separator = html_entity_decode( $separator );

        }

        // natural sorting: retrieve from database by name, then sort by PHP
        if ( 'natural' == $orderby ) {

          $natural_sorting = true;

          $orderby = 'name';

        } else {

          $natural_sorting = false;

        }

        if ( $smallest < 1 ) {

          $smallest = 1;

        }

        if ( $largest < $smallest ) {

          $largest = $smallest;

        }

        if ( $amount < 0 ) {

          $amount = 0;

        }

        if ( ! empty( $show_not_assigned ) ) {

          $start_group = 0;

        } else {

          $start_group = 1;

        }

        // remove prepended '?'
        if ( mb_strpos( $link_append, '?' ) === 0 ) {

          $link_append = mb_substr( $link_append, 1 );

        }

        if ( isset( $taxonomy ) ) {

          if ( empty( $taxonomy ) ) {

            unset( $taxonomy );

          } else {

            $taxonomy_array = array_map( 'trim', explode( ',', $taxonomy ) );

          }

        }

        /**
        * Consider only taxonomies that
        * 1. are among $enabled_taxonomies
        * 2. actually exist
        * 3. are among $taxonomy_array, if isset
        */

        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        if ( ! empty( $taxonomy_array ) ) {

          $taxonomies = array_intersect( $enabled_taxonomies, $taxonomy_array );

          if ( empty( $taxonomies ) ) {

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              error_log( sprintf( '[Tag Groups Premium] Wrong taxonomy in shortcode "tag_groups_table": %s', $taxonomy ) );

            }

            return '';

          }

        } else {

          $taxonomies = $enabled_taxonomies;

        }

        /**
        * Reduce the risk of interference from other plugins by removing filters
        */
        remove_all_filters( 'get_terms_orderby' );
        remove_all_filters( 'list_terms_exclusions' );
        remove_all_filters( 'get_terms' );

        // format for WP including versions before 4.5
        $posttags = get_terms(
          $taxonomies,
          array(
            'hide_empty' => $hide_empty,
            'orderby' => $orderby,
            'order' => $order,
            'include' => $include_terms,
            'exclude' => $exclude_terms
          )
        );

        /**
        * In case of errors: return empty array
        */
        if ( ! is_array( $posttags ) ) {

          $posttags = array();

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            error_log( '[Tag Groups Premium] Error retrieving tags with get_terms.' );

          }

        }


        if ( $include !== '' ) {

          $include_array = explode( ',', str_replace( ' ', '', $include ) );

        } else {

          $include_array = array();

        }

        if ( $separator_size < 1 ) {

          $separator_size = 12;

        } else {

          $separator_size = (int) $separator_size;

        }


        if ( $tags_post_id > 0 ) {

          $result = self::add_tags_of_post( $tags_post_id, $taxonomies, $posttags, $assigned_class );

          $assigned_terms = $result['assigned_terms'];

          $posttags = $result['posttags'];

          $include_tags_post_id_groups = $result['include_tags_post_id_groups'];

        }


        /*
        *  applying parameter groups_post_id
        */
        if ( $groups_post_id > 0 ) {

          $include_array = self::add_groups_of_post( $groups_post_id, $taxonomies, $include_array );

        } elseif ( count( $include_array ) == 0 ) {

          $include_array = $tag_group_ids;

        }


        // apply natural sorting and random sorting
        if ( $natural_sorting ) {

          $posttags = self::natural_sorting( $posttags, $order );

        } elseif ( 'random' == $orderby ) {

          $posttags = self::random_sorting( $posttags );

        }

        $html = '';

        $min_max = self::determine_min_max( $posttags, $amount, $include_array, $include_tags_post_id_groups );

        /**
        * We need here absolute min and max, independent of groups
        */
        $min = 0;

        $max = 0;

        foreach ( $min_max as $min_max_item ) {

          if ( 0 == $min || $min_max_item['min'] < $min ) {

            $min = $min_max_item['min'];

          }

          if ( $min_max_item['max'] > $max ) {

            $max = $min_max_item['max'];

          }

        }

        $post_counts = array();

        if ( class_exists( 'TagGroups_Premium_Term' ) && method_exists( $tag_group_premium_terms, 'get_post_counts' ) ) {

          $post_counts = $tag_group_premium_terms->get_post_counts();

        }

        /*
        *  render the content
        */
        $count_amount = 0;

        foreach ( $posttags as $tag ) {

          $other_tag_classes = '';

          $description = '';

          if ( ! empty( $amount ) && $count_amount >= $amount ) {

            break;

          }

          $term_o = new TagGroups_Term( $tag );

          // check if tag has posts for this set of groups
          if ( $term_o->is_in_group( $include_array ) ) {

            /**
            * We are using here simple, group-agnostic tag counts, since each tag links to the full set of posts that use this tag.
            */
            $tag_count = $tag->count;

            if ( ! $hide_empty || $tag_count > 0 ) {

              $tag_link = get_term_link( $tag );

              if ( ! empty( $link_append ) ) {

                if ( mb_strpos( $tag_link, '?' ) === false ) {

                  $tag_link = esc_url( $tag_link . '?' . $link_append );

                } else {

                  $tag_link = esc_url( $tag_link . '&' . $link_append );

                }

              }

              /**
              * Note: We cannot append a parameter to separate terms by group on the archive page, since we don't use groups here.
              */
              $font_size = self::font_size( $tag_count, $min, $max, $smallest, $largest );

              $font_size_tag = $adjust_separator_size ? $font_size : $separator_size;

              if ( $count_amount > 0 && ! empty( $separator ) ) {

                $html .= '<span style="font-size:' . $font_size_tag . 'px">' . $separator . '</span> ';

              }

              if ( ! empty( $assigned_class ) ) {

                if ( ! empty( $assigned_terms[ $tag->term_id ] ) ) {

                  $other_tag_classes = ' ' . $assigned_class . '_1';

                } else {

                  $other_tag_classes = ' ' . $assigned_class . '_0';

                }

              }

              if ( ! empty( $custom_title ) ) {

                // use the provided template
                $description = ! empty( $tag->description ) ? esc_html( $tag->description ) : '';

                $title = preg_replace("/(\{description\})/", $description, $custom_title);

                $title = preg_replace("/(\{count\})/", $tag_count, $title);

              } else {

                // use just description and number
                $description = ! empty( $tag->description ) ? esc_html( $tag->description ) . ' ' : '';

                $tag_count_brackets = $show_tag_count ? '(' . $tag_count . ')' : '';

                $title = $description . $tag_count_brackets;

              }

              // replace placeholders in prepend and append
              if ( ! empty( $prepend ) ) {

                $prepend_output = preg_replace("/(\{count\})/", $tag_count, $prepend );

              } else {

                $prepend_output = '';

              }

              if ( ! empty( $append ) ) {

                $append_output = preg_replace("/(\{count\})/", $tag_count, $append );

              } else {

                $append_output = '';

              }

              // adding link target
              $link_target_html = ! empty( $link_target ) ? 'target="' . $link_target . '"' : '';

              // assembling a tag
              $html .= '<span class="tag-groups-tag' . $other_tag_classes . '" style="font-size:' . $font_size . 'px"><a href="' . $tag_link . '" ' . $link_target_html . ' title="' . $title . '"  class="' . $tag->slug . '">';

              if ( '' != $prepend_output ) {

                $html .= '<span class="tag-groups-prepend" style="font-size:' . $font_size . 'px">' . htmlentities( $prepend_output, ENT_QUOTES, "UTF-8" ) . '</span>';

              }

              $html .= apply_filters( 'tag_groups_cloud_tag_outer', '<span class="tag-groups-label" style="font-size:' . $font_size . 'px">' . apply_filters( 'tag_groups_cloud_tag_inner', $tag->name, $tag->term_id, $shortcode_id ) . '</span>', $tag->term_id, $shortcode_id );

              if ( '' != $append_output ) {

                $html .= '<span class="tag-groups-append" style="font-size:' . $font_size . 'px">' . htmlentities( $append_output, ENT_QUOTES, "UTF-8" ) . '</span>';

              }

              $html .= '</a></span> ';

              $count_amount++;

            }

          }

        }

        // create a cached version (premium plugin)
        do_action( 'tag_groups_hook_cache_set', $cache_key, $html );

      }

      /*
      *  render the div
      */
      $html = '<div' . $div_id_output . $div_class_output . '>' . $html . '</div>';

      apply_filters( 'tag_groups_cloud_html', $html, $shortcode_id, $atts );

      return $html;

    }

  } //class

}
