<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Shortcode_Menu' ) ) {

  class TagGroups_Premium_Shortcode_Menu extends TagGroups_Shortcode {


    /**
    * Shortcode that outputs the tag groups as a menu
    *
    * @param array $atts
    * @return string
    */
    static function tag_groups_menu( $atts = array() ) {

      global $tag_group_groups;

      $group_labels = $tag_group_groups->get_all_term_group_label();

      $term_groups = $tag_group_groups->get_group_ids_by_position();

      extract( shortcode_atts( array(
        'custom_url' => null,
        'div_class' => 'tag-groups-menu',
        'div_id' => 'tag-groups-menu-' . uniqid(),
        'include' => '',
        'not_assigned_name' => 'not assigned',
        'shop' => '',
        'source' => 'shortcode',
        'submit_label' => __( 'search' ),
        'taxonomy' => null, // we accept only ONE taxonomy
        'type' => 'select',
      ), $atts ) );


      if ( empty( $include ) ) {

        $include_array = $term_groups;

      } else {

        $include_array = array_map( 'intval', explode( ',', $include ) );

        if ( $diff = array_diff( $include_array, $term_groups ) ) {

          if ( defined( 'WP_DEBUG') && WP_DEBUG ) {

            error_log( sprintf( '[Tag Groups Premium] Wrong values for include with "tag_groups_menu": %s', implode( ',', $diff ) ) );

          }

          $include_array = $term_groups;

        }

      }


      if ( $custom_url ) {

        $base_url = $custom_url;

      } elseif ( 'wc' == $shop && class_exists( 'WooCommerce' ) ) {
        /**
        * We use the URL of the WooCommerce shop page
        */

        $base_url = get_permalink( wc_get_page_id( 'shop' ) );

      } elseif ( '' == $taxonomy ) {

        $base_url = get_post_type_archive_link( 'post' );

      } else {
        /**
        * We use the URL of the posts page
        */
        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        if ( ! in_array( $taxonomy, $enabled_taxonomies) ) {

          if ( defined( 'WP_DEBUG') && WP_DEBUG ) {

            error_log( sprintf( '[Tag Groups Premium] Wrong taxonomy with "tag_groups_menu": %s', $taxonomy ) );

          }

          $base_url = get_post_type_archive_link( 'post' );

        } else {

          $post_types = post_types_from_taxonomies( array( $taxonomy ) );

          $base_url = get_post_type_archive_link( TagGroups_Base::get_first_element( $post_types ) );

        }

      }


      $html = '<div id="' . $div_id . '" class="' . $div_class . '">';


      if ( 'select' == $type || 'autoselect' == $type ) {


        $html .= '<form action="' . $base_url . '" method="GET" id="' . $div_id . '_form">';

        $html .= '<select name="term_group" autocomplete="off">';

        /**
        * We process the labels in their order by position
        */

        foreach ( $group_labels as $group_id => $group_label ) {

          if ( ! in_array( $group_id, $include_array ) ) {

            continue;

          }

          if ( 0 == $group_id ) {

            $group_label == $not_assigned_name;

          }

          /**
          * Preselect the option that corresponds to the URL parameter
          */
          if ( isset( $_GET[ 'term_group' ] ) && $_GET[ 'term_group' ] == $group_id ) {

            $selected = ' selected="selected"';

          } else {

            $selected = '';

          }

          $html .= '<option value="' . $group_id . '"' . $selected . '>' . $group_label . '</option>';

        }

        $html .= '</select>';

        if ( 'select' == $type ) {
          /**
          * Form with a submit button
          */

          $html .= '<input type="submit" value="' . $submit_label . '">';

          $html .= '</form>';

        } else {
          /**
          * Form with auto-submission on change
          */

          $html .= '</form>';

          $view = new TagGroups_Premium_View( 'partials/autosubmit_js_snippet' );

          $view->set( 'div_id', $div_id );

          $html .= $view->return_html();

        }

      }


      $html .= '</div>';

      return $html;

    }


  } //class

}
