<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Shortcode_Shuffle_Box' ) ) {

  class TagGroups_Premium_Shortcode_Shuffle_Box extends TagGroups_Shortcode {


    /**
    * Shortcode that outputs the tags in a filterable tag cloud
    *
    * @param array $atts
    * @return string
    */
    static function tag_groups_shuffle_box( $atts = array() ) {

      global $tag_group_groups, $tag_group_premium_terms;

      $shortcode_id = 'tag_groups_shuffle_box';

      extract( shortcode_atts( array(
        'add_premium_filter' => 0,
        'amount' => 0,
        'append' => '',
        'custom_title' => '',
        'div_class' => 'cm-shuffle-box-theme-default',
        'div_id' => 'tag-groups-shuffle-box-' . uniqid(),
        'groups_post_id' => -1,
        'hide_empty' => true,
        'include' => '',
        'initial_group' => -1,
        'largest' => 22,
        'layout_mode' => 'fitRows', // 'fitRows', 'masonry', 'vertical'
        'link_append' => '',
        'link_target' => '',
        'not_assigned_name' => __( 'not assigned', 'tag-groups-premium' ),
        'order' => 'ASC',
        'orderby' => 'name',
        'placeholder' => __( 'search', 'tag-groups-premium' ),
        'prepend' => '',
        'show_all_name' => __( 'all groups', 'tag-groups-premium' ),
        'show_filter_all_groups' => 1,
        'show_group_filter' => 1,
        'show_tag_count' => 1,
        'show_text_filter' => 1,
        'smallest' => 12,
        'source' => 'shortcode',
        'tags_post_id' => -1,
        'taxonomy' => null,
      ), $atts ) );

      /**
      * In case we use the WPML plugin: consider the language
      */
      if ( defined( 'ICL_LANGUAGE_CODE' ) ) {

        $wpml_language = (string) ICL_LANGUAGE_CODE;

      } else {

        $wpml_language = '';

      }

      if ( is_array( $atts ) ) {

        asort( $atts );

      }

      /*
      *  applying parameter tags_post_id
      */
      if ( $tags_post_id == 0 ) {

        // retrieve the current post ID
        $tags_post_id = get_the_ID();

      }

      /*
      *  applying parameter groups_post_id
      */
      if ( $groups_post_id == 0 ) {

        $groups_post_id = get_the_ID();

      }

      $div_id = sanitize_html_class( $div_id );

      $div_id_output = $div_id ? ' id="' . $div_id . '"' : '';

      $div_class_output = ' class="cm-shuffle-box-container ' . sanitize_html_class( $div_class ) . '"';

      $cache_key = md5( 'shuffle_box' . serialize( $atts ) . serialize( $tags_post_id ) . serialize( $groups_post_id ) );

      // check for a cached version (premium plugin)
      $html = apply_filters( 'tag_groups_hook_cache_get', false, $cache_key );

      if ( ! $html ) {

        $include_array = array();

        $html_header = array();

        $html_div = array();

        $data = $tag_group_groups->get_all_with_position_as_key();

        $tag_group_ids = $tag_group_groups->get_group_ids_by_position();


        if ( 'shortcode' == $source ) {

          $prepend = html_entity_decode( $prepend );

          $append = html_entity_decode( $append );

        }

        // natural sorting: retrieve from database by name, then sort by PHP
        if ( 'natural' == $orderby ) {

          $natural_sorting = true;

          $orderby = 'name';

        } else {

          $natural_sorting = false;

        }

        if ( $smallest < 1 ) {

          $smallest = 1;

        }

        if ( $largest < $smallest ) {

          $largest = $smallest;

        }

        if ( $amount < 0 ) {

          $amount = 0;

        }



        $js_identifier = str_replace( '-', '_', sanitize_title( $div_id ) );


        // remove prepended '?'
        if ( mb_strpos( $link_append, '?' ) === 0 ) {

          $link_append = mb_substr( $link_append, 1 );

        }

        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        if ( ! empty( $taxonomy ) ) {

          $taxonomies = array_intersect( $enabled_taxonomies, array_map( 'trim', explode( ',', $taxonomy ) ) );

        } else {

          $taxonomies = $enabled_taxonomies;

        }


        if ( empty( $taxonomies ) ) {

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            error_log( sprintf( '[Tag Groups Premium] Wrong taxonomy in shortcode "tag_groups_table": %s', $taxonomy ) );

          }

          return '';

        }


        /**
        * Reduce the risk of interference from other plugins by removing filters
        */
        remove_all_filters( 'get_terms_orderby' );
        remove_all_filters( 'list_terms_exclusions' );
        remove_all_filters( 'get_terms' );

        // format for WP including versions before 4.5
        $posttags = get_terms(
          $taxonomies,
          array(
            'hide_empty' => $hide_empty,
            'orderby' => $orderby,
            'order' => $order,
          )
        );

        /**
        * In case of errors: return empty array
        */
        if ( ! is_array( $posttags ) ) {

          $posttags = array();

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            error_log( '[Tag Groups Premium] Error retrieving tags with get_terms.' );

          }

        }

        if ( $include !== '' ) {

          $include_array = array_intersect( $tag_group_ids, explode( ',', str_replace( ' ', '', $include ) ) );

        } else {

          $include_array = $tag_group_groups->get_group_ids_by_position();

          // We are removing "not assigned", if not explicitely required

          unset( $include_array[0] );

        }

        if ( empty( $include_array ) ) {

          $include_array = array();

        }


        if ( $tags_post_id > 0 ) {

          $result = self::add_tags_of_post( $tags_post_id, $taxonomies, $posttags, $assigned_class );

          $assigned_terms = $result['assigned_terms'];

          $posttags = $result['posttags'];

          $include_tags_post_id_groups = $result['include_tags_post_id_groups'];

        }


        /*
        *  applying parameter groups_post_id
        */
        if ( $groups_post_id > 0 ) {

          $include_array = self::add_groups_of_post( $groups_post_id, $taxonomies, $include_array );

        } elseif ( count( $include_array ) == 0 ) {

          $include_array = $tag_group_ids;

        }


        // apply natural sorting and random sorting
        if ( $natural_sorting ) {

          $posttags = self::natural_sorting( $posttags, $order );

        } elseif ( 'random' == $orderby ) {

          $posttags = self::random_sorting( $posttags );

        }


        if ( $show_text_filter ) {

          $html_header[] .= '<input class="cm-shuffle-box-input cm-shuffle-box-quicksearch" value="" placeholder="' . $placeholder . '" autocomplete="off">';

        }



        if ( $show_group_filter ) {

          if ( $show_filter_all_groups ) {

            $html_header[] .= '<button class="cm-shuffle-box-button cm-shuffle-box-button--1" onclick="javascript:cmShuffleBoxSetGroup' . $js_identifier . '(-1);">' . $show_all_name . '</button>';

          }


          for ( $i = 0; $i <= $tag_group_groups->get_max_position(); $i++ ) {

            if ( in_array( $data[ $i ]['term_group'], $include_array ) ) {

              if ( $i == 0 ) {

                $group_name = $not_assigned_name;

              } else {

                $group_name = $data[ $i ]['label']; // self::translate_string_wpml( 'Group Label ID ' . $data[ $i ]['term_group'], $data[ $i ]['label'] );

              }

              $html_header[] .= '<button class="cm-shuffle-box-button cm-shuffle-box-button-' . $data[ $i ]['term_group'] . '" onclick="javascript:cmShuffleBoxSetGroup' . $js_identifier . '(' . $data[ $i ]['term_group'] . ');">' . $group_name . '</button>';

              if ( $initial_group == -1 && ! $show_filter_all_groups ) {

                $initial_group = $data[ $i ]['term_group'];

              }

            }

          }

        }


        $min_max = self::determine_min_max( $posttags, $amount, $include_array );

        /**
        * We need here absolute min and max, independent of groups
        */
        $min = 0;

        $max = 0;

        foreach ( $min_max as $min_max_item ) {

          if ( 0 == $min || $min_max_item['min'] < $min ) {

            $min = $min_max_item['min'];

          }

          if ( $min_max_item['max'] > $max ) {

            $max = $min_max_item['max'];

          }

        }


        $post_counts = array();

        if ( class_exists( 'TagGroups_Premium_Term' ) && method_exists( $tag_group_premium_terms, 'get_post_counts' ) ) {

          $post_counts = $tag_group_premium_terms->get_post_counts();

        }

        /*
        *  render the content
        */
        $count_amount = 0;

        foreach ( $posttags as $tag ) {

          $description = '';

          if ( ! empty( $amount ) && $count_amount >= $amount ) {

            break;

          }

          $term_o = new TagGroups_Term( $tag );

          // check if tag has posts for the selected groups
          if ( $term_o->is_in_group( $include_array ) ) {

            $tag_count = $tag->count;


            if ( ! $hide_empty || $tag_count > 0 ) {

              $tag_link = get_term_link( $tag );

              if ( ! empty( $link_append ) ) {

                if ( mb_strpos( $tag_link, '?' ) === false ) {

                  $tag_link = esc_url( $tag_link . '?' . $link_append );

                } else {

                  $tag_link = esc_url( $tag_link . '&' . $link_append );

                }

              }


              $font_size = self::font_size( $tag_count, $min, $max, $smallest, $largest );


              if ( ! empty( $custom_title ) ) {

                // use the provided template
                $description = ! empty( $tag->description ) ? esc_html( $tag->description ) : '';

                $title = preg_replace("/(\{description\})/", $description, $custom_title);

                $title = preg_replace("/(\{count\})/", $tag_count, $title);

              } else {

                // use just description and number
                $description = ! empty( $tag->description ) ? esc_html( $tag->description ) . ' ' : '';

                $tag_count_brackets = $show_tag_count ? '(' . $tag_count . ')' : '';

                $title = $description . $tag_count_brackets;

              }

              // replace placeholders in prepend and append
              if ( ! empty( $prepend ) ) {

                $prepend_output = preg_replace("/(\{count\})/", $tag_count, $prepend );

              } else {

                $prepend_output = '';

              }

              if ( ! empty( $append ) ) {

                $append_output = preg_replace("/(\{count\})/", $tag_count, $append );

              } else {

                $append_output = '';

              }


              $link_target_html = ! empty( $link_target ) ? 'target="' . $link_target . '"' : '';

              $groups_of_term = $term_o->get_groups();

              $group_classes = array_map( function( $g ) { return 'cm-shuffle-box-group-' . $g; } , $groups_of_term );

              $html_tag = '';

              if ( '' != $prepend_output ) {

                $html_tag .= '<span class="tag-groups-prepend" style="font-size:' . $font_size . 'px">' . htmlentities( $prepend_output, ENT_QUOTES, "UTF-8" ) . '</span>';

              }

              $html_tag .= apply_filters( 'tag_groups_cloud_tag_outer', '<span class="tag-groups-label" style="font-size:' . $font_size . 'px">' . apply_filters( 'tag_groups_cloud_tag_inner', $tag->name, $tag->term_id, $shortcode_id ) . '</span>', $tag->term_id, $shortcode_id );

              if ( '' != $append_output ) {

                $html_tag .= '<span class="tag-groups-append" style="font-size:' . $font_size . 'px">' . htmlentities( $append_output, ENT_QUOTES, "UTF-8" ) . '</span>';

              }

              $html_div[] = '<span class="tag-groups-tag ' . implode( ' ', $group_classes ) . '" style="font-size:' . $font_size . 'px">' .
              '<a href="' . $tag_link . '" ' . $link_target_html . ' title="' . $title . '"  class="' . $tag->slug . '" data-href="' . $tag_link . '" data-termid="' . $tag->term_id . '">' .
              $html_tag  .
              '</a>' .
              '</span>';

              $count_amount++;

            }

          }

        }


        /*
        * assemble content
        */
        $view = new TagGroups_Premium_View( 'shortcodes/shuffle_box' );

        $view->set( array(
          'html_header' => $html_header,
          'div_id_inner'  =>  $div_id . '_inner',
          'html_div'  => $html_div,
          'js_identifier' => $js_identifier,
          'layout_mode' => $layout_mode,
          'initial_group' => $initial_group,
          'add_premium_filter'  => $add_premium_filter
        ) );

        $html = $view->return_html();

        // create a cached version (premium plugin)
        do_action( 'tag_groups_hook_cache_set', $cache_key, $html );

      }


      $html = '<div' . $div_id_output . $div_class_output . '>' . $html . '</div>';

      apply_filters( 'tag_groups_cloud_html', $html, $shortcode_id, $atts );

      return $html;

    }


  } //class

}
