<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Shortcode_Table' ) ) {

  class TagGroups_Premium_Shortcode_Table extends TagGroups_Shortcode {

    /**
    * Shortcode that outputs the tags in a table
    *
    * @param array $atts
    * @return string
    */
    static function tag_groups_table( $atts = array() ) {

      global $tag_group_groups, $tag_group_premium_terms;

      $shortcode_id = 'tag_groups_table';

      extract( shortcode_atts( array(
        'adjust_separator_size' => false,
        'add_premium_filter' => 0,
        'amount' => 0,
        'append' => '',
        'assigned_class' => null,
        'custom_title' => null,
        'exclude_terms' => '',
        'group_in_class' => 0,
        'groups_post_id' => -1,
        'hide_empty' => true,
        'hide_empty_columns' => false,
        'include' => '',
        'include_terms' => '',
        'largest' => 22,
        'link_target' => '',
        'link_append' => '',
        'not_assigned_name' => 'not assigned',
        'order' => 'ASC',
        'orderby' => 'name',
        'prepend' => '',
        'responsive_breakpoint' => 800,
        'separator_size' => 12,
        'separator' => '',
        'show_not_assigned' => false,
        'show_all_groups' => false,
        'show_tag_count' => true,
        'smallest' => 12,
        'source' => 'shortcode',
        'table_id' => 'tag-groups-cloud-table-' . uniqid(),
        'table_class' => 'tag-groups-cloud-table',
        'tags_post_id' => -1,
        'taxonomy' => null,
        'td_class' => null,
        'th_class' => null,
      ), $atts ) );

      $table_id_output = $table_id ? ' id="' . TagGroups_Base::sanitize_html_classes( $table_id ) . '"' : '';

      $table_class_output = $table_class ? ' class="' . TagGroups_Base::sanitize_html_classes( $table_class ) . '"' : ' class="tg_table_align_top"';

      if ( is_array( $atts ) ) {

        asort( $atts );

      }


      /*
      *  applying parameter tags_post_id
      */
      if ( $tags_post_id == 0 ) {

        // retrieve the current post ID
        $tags_post_id = get_the_ID();

      }

      /*
      *  applying parameter groups_post_id
      */
      if ( $groups_post_id == 0 ) {

        $groups_post_id = get_the_ID();

      }

      $cache_key = md5( 'table' . serialize( $atts ) . serialize( $tags_post_id ) . serialize( $groups_post_id ) );

      // check for a cached version (premium plugin)
      $html = apply_filters( 'tag_groups_hook_cache_get', false, $cache_key );

      if ( ! $html ) {

        $html_table_headers = array();

        $html_table_cells = array();

        $post_id_terms = array();

        $assigned_terms = array();

        $include_tags_post_id_groups = array();

        $data = $tag_group_groups->get_all_with_position_as_key();

        $tag_group_ids = $tag_group_groups->get_group_ids_by_position();


        if ( 'shortcode' == $source ) {

          $prepend = html_entity_decode( $prepend );

          $append = html_entity_decode( $append );

          $separator = html_entity_decode( $separator );

        }

        // natural sorting: retrieve from database by name, then sort by PHP
        if ( 'natural' == $orderby ) {

          $natural_sorting = true;

          $orderby = 'name';

        } else {

          $natural_sorting = false;

        }

        if ( $smallest < 1 ) {

          $smallest = 1;

        }

        if ( $largest < $smallest ) {

          $largest = $smallest;

        }

        if ( $amount < 0 ) {

          $amount = 0;

        }

        if ( ! empty( $show_not_assigned ) ) {

          $start_group = 0;

        } else {

          $start_group = 1;

        }

        // remove prepended '?'
        if ( mb_strpos( $link_append, '?' ) === 0 ) {

          $link_append = mb_substr( $link_append, 1 );

        }

        if ( ! empty( $taxonomy ) ) {

          $taxonomy_array = array_map( 'trim', explode( ',', $taxonomy ) );

        }

        /**
        * Consider only taxonomies that
        * 1. are among $enabled_taxonomies
        * 2. actually exist
        * 3. are among $taxonomy_array, if isset
        */
        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        if ( ! empty( $taxonomy_array ) ) {

          $taxonomies = array_intersect( $enabled_taxonomies, $taxonomy_array );

          if ( empty( $taxonomies ) ) {

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              error_log( sprintf( '[Tag Groups Premium] Wrong taxonomy in shortcode "tag_groups_table": %s', $taxonomy ) );

            }

            return '';

          }

        } else {

          $taxonomies = $enabled_taxonomies;

        }

        /**
        * Reduce the risk of interference from other plugins by removing filters
        */
        remove_all_filters( 'get_terms_orderby' );
        remove_all_filters( 'list_terms_exclusions' );
        remove_all_filters( 'get_terms' );

        // format for WP including versions before 4.5
        $posttags = get_terms(
          $taxonomies,
          array(
            'hide_empty' => $hide_empty,
            'orderby' => $orderby,
            'order' => $order,
            'include' => $include_terms,
            'exclude' => $exclude_terms
          )
        );

        /**
        * In case of errors: return empty array
        */
        if ( ! is_array( $posttags ) ) {

          $posttags = array();

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            error_log( '[Tag Groups Premium] Error retrieving tags with get_terms.' );

          }

        }

        $td_class_output = $td_class ? ' class="' . TagGroups_Base::sanitize_html_classes( $td_class ) . '"' : '';

        if ( $include !== '' ) {

          $include_array = explode( ',', str_replace( ' ', '', $include ) );

        } else {

          $include_array = array();

        }

        if ( $separator_size < 1 ) {

          $separator_size = 12;

        } else {

          $separator_size = (int) $separator_size;

        }


        if ( $tags_post_id > 0 ) {

          $result = self::add_tags_of_post( $tags_post_id, $taxonomies, $posttags, $assigned_class );

          $assigned_terms = $result['assigned_terms'];

          $posttags = $result['posttags'];

          $include_tags_post_id_groups = $result['include_tags_post_id_groups'];

        }


        /*
        *  applying parameter groups_post_id
        */
        if ( $groups_post_id > 0 ) {

          $include_array = self::add_groups_of_post( $groups_post_id, $taxonomies, $include_array );

        } elseif ( count( $include_array ) == 0 ) {

          $include_array = $tag_group_ids;

        }


        // apply natural sorting and random sorting
        if ( $natural_sorting ) {

          $posttags = self::natural_sorting( $posttags, $order );

        } elseif ( 'random' == $orderby ) {

          $posttags = self::random_sorting( $posttags );

        }


        $html = '';

        /*
        *  render the table header
        */
        $html_table_headers[] = '<tr>';

        for ( $i = $start_group; $i <= $tag_group_groups->get_max_position(); $i++ ) {

          if ( $show_all_groups || in_array( $data[ $i ]['term_group'], $include_array ) ) {

            if ( $i == 0 ) {

              $group_name = $not_assigned_name;

            } else {

              $group_name = $data[ $i ]['label']; // self::translate_string_wpml( 'Group Label ID ' . $data[ $i ]['term_group'], $data[ $i ]['label'] );

            }

            $th_class_group = $th_class;

            if ( ! empty( $group_in_class ) ) {

              $th_class_group .= ' ' . sanitize_html_class( ' tg_header_group_id_' . $data[ $i ]['term_group'] ) . ' ' . sanitize_html_class( 'tg_header_group_label_' . strtolower( $data[ $i ]['label'] ) );

            }

            $th_class_output = $th_class_group ? ' class="' . TagGroups_Base::sanitize_html_classes( $th_class_group ) . '"' : '';

            $html_table_headers[] .= '<th' . $th_class_output . '>' . htmlentities( $group_name, ENT_QUOTES, "UTF-8" ) . '</th>';

          }

        }

        $html_table_headers[] .= '</tr>';

        $post_counts = array();

        if ( class_exists( 'TagGroups_Premium_Term' ) && method_exists( $tag_group_premium_terms, 'get_post_counts' ) ) {

          $post_counts = $tag_group_premium_terms->get_post_counts();

        }

        $min_max = self::determine_min_max( $posttags, $amount, $tag_group_ids, $include_tags_post_id_groups, $data, $post_counts );

        /*
        *  render the table content
        */
        for ( $i = $start_group; $i <= $tag_group_groups->get_max_position(); $i++ ) {

          $count_amount = 0;

          if ( $show_all_groups || empty( $include_array ) || in_array( $data[ $i ]['term_group'], $include_array ) ) {

            $html_table_cells[ $i ] = '';

            if ( 'count' == $orderby && ! empty( $post_counts ) ) {
              // We have to sort the tags according to the post counts for this particular group

              $posttags = self::sort_within_groups( $posttags, $data[ $i ]['term_group'], $post_counts, $order );

            }

            foreach ( $posttags as $tag ) {

              $other_tag_classes = '';

              $description = '';

              if ( ! empty( $amount ) && $count_amount >= $amount ) {

                break;

              }

              $term_o = new TagGroups_Term( $tag );

              // check if tag has posts for this particular group
              if ( $term_o->is_in_group( $data[ $i ]['term_group'] ) ) {

                if ( empty( $include_tags_post_id_groups ) || in_array( $tag->term_id, $include_tags_post_id_groups[ $data[ $i ]['term_group'] ] ) ) {

                  // check if tag has posts for this particular group
                  if ( ! empty( $post_counts ) ) {

                    if ( isset( $post_counts[ $tag->term_id ][ $data[ $i ]['term_group'] ] ) ) {

                      $tag_count = $post_counts[ $tag->term_id ][ $data[ $i ]['term_group'] ];

                    } else {

                      $tag_count = 0;

                    }

                  } else {

                    $tag_count = $tag->count;

                  }

                  if ( ! $hide_empty || $tag_count > 0 ) {

                    $tag_link = get_term_link( $tag );

                    if ( ! empty( $link_append ) ) {

                      if ( mb_strpos( $tag_link, '?' ) === false ) {

                        $tag_link = esc_url( $tag_link . '?' . $link_append );

                      } else {

                        $tag_link = esc_url( $tag_link . '&' . $link_append );

                      }

                    }

                    /**
                    * Append a parameter to separate terms by group on the archive page
                    */
                    if ( class_exists( 'TagGroups_Premium_Term' ) && $add_premium_filter ) {

                      if ( mb_strpos( $tag_link, '?' ) === false ) {

                        $tag_link = esc_url( $tag_link . '?term_group=' . $data[ $i ]['term_group'] . '&term_id=' . $tag->term_id );

                      } else {

                        $tag_link = esc_url( $tag_link . '&term_group=' . $data[ $i ]['term_group'] . '&term_id=' . $tag->term_id );

                      }

                    }

                    $font_size = self::font_size( $tag_count, $min_max[ $tag_group_ids[ $i ] ]['min'], $min_max[ $tag_group_ids[ $i ]]['max'], $smallest, $largest );

                    $font_size_tag = $adjust_separator_size ? $font_size : $separator_size;

                    if ( $count_amount > 0 && ! empty( $separator ) ) {

                      $html_table_cells[ $i ] .= '<span style="font-size:' . $font_size_tag . 'px">' . $separator . '</span> ';

                    }

                    if ( ! empty( $assigned_class ) ) {

                      if ( ! empty( $assigned_terms[ $tag->term_id ] ) ) {

                        $other_tag_classes = ' ' . $assigned_class . '_1';

                      } else {

                        $other_tag_classes = ' ' . $assigned_class . '_0';

                      }

                    }

                    if ( ! empty( $custom_title ) ) {

                      // use the provided template
                      $description = ! empty( $tag->description ) ? esc_html( $tag->description ) : '';

                      $title = preg_replace("/(\{description\})/", $description, $custom_title);

                      $title = preg_replace("/(\{count\})/", $tag_count, $title);

                    } else {

                      // use just description and number
                      $description = ! empty( $tag->description ) ? esc_html( $tag->description ) . ' ' : '';

                      $tag_count_brackets = $show_tag_count ? '(' . $tag_count . ')' : '';

                      $title = $description . $tag_count_brackets;

                    }

                    // replace placeholders in prepend and append
                    if ( ! empty( $prepend ) ) {

                      $prepend_output = preg_replace("/(\{count\})/", $tag_count, $prepend );

                    } else {

                      $prepend_output = '';

                    }

                    if ( ! empty( $append ) ) {

                      $append_output = preg_replace("/(\{count\})/", $tag_count, $append );

                    } else {

                      $append_output = '';

                    }

                    // adding link target
                    $link_target_html = ! empty( $link_target ) ? 'target="' . $link_target . '"' : '';

                    // adding class for group
                    if ( ! empty( $group_in_class ) ) {

                      $other_tag_classes .= ' ' . sanitize_html_class( ' tg_tag_group_id_' . $data[ $i ]['term_group'] ) . ' ' . sanitize_html_class( 'tg_tag_group_label_' . strtolower( $data[ $i ]['label'] ) );

                    }

                    // assembling a tag
                    $html_table_cells[ $i ] .= '<span class="tag-groups-tag' . $other_tag_classes . '" style="font-size:' . $font_size . 'px"><a href="' . $tag_link . '" ' . $link_target_html . ' title="' . $title . '"  class="' . $tag->slug . '">';


                    if ( '' != $prepend_output ) {

                      $html_table_cells[ $i ] .= '<span class="tag-groups-prepend" style="font-size:' . $font_size . 'px">' . htmlentities( $prepend_output, ENT_QUOTES, "UTF-8" ) . '</span>';

                    }

                    $html_table_cells[ $i ] .= apply_filters( 'tag_groups_cloud_tag_outer', '<span class="tag-groups-label" style="font-size:' . $font_size . 'px">' . apply_filters( 'tag_groups_cloud_tag_inner', $tag->name, $tag->term_id, $shortcode_id ) . '</span>', $tag->term_id, $shortcode_id );

                    if ( '' != $append_output ) {

                      $html_table_cells[ $i ] .= '<span class="tag-groups-append" style="font-size:' . $font_size . 'px">' . htmlentities( $append_output, ENT_QUOTES, "UTF-8" ) . '</span>';

                    }

                    $html_table_cells[ $i ] .= '</a></span> ';

                    $count_amount++;

                  }

                }

              }

            }

            $html_table_cells[ $i ] = '<td' . $td_class_output . '>' . $html_table_cells[ $i ] . '</td>';

          }



          if ( $hide_empty_columns && ! $count_amount ) {

            unset( $html_table_headers[ $i ] );

            unset( $html_table_cells[ $i ] );

          }

        }



        /**
        * assemble header
        */
        $html .= '<thead>' . implode( "\n", $html_table_headers ) . '</thead>';

        /**
        * assemble rows
        */
        $html .= '<tbody><tr>' . implode( "\n", $html_table_cells ) . '</tr></tbody>';


        // create a cached version (premium plugin)
        do_action( 'tag_groups_hook_cache_set', $cache_key, $html );

      }

      $html = '<table' . $table_id_output . $table_class_output . '>' . $html . '</table>';

      /**
      * add basictable for responsiveness
      */
      if ( $responsive_breakpoint ) {

        $view = new TagGroups_Premium_View( 'partials/responsive_table_js_snippet' );

        $view->set( array(
          'responsive_breakpoint'  => $responsive_breakpoint,
          'table_id'               => $table_id
        ) );

        $html .= $view->return_html();

      }

      apply_filters( 'tag_groups_cloud_html', $html, $shortcode_id, $atts );

      return $html;

    }


  } //class

}
