/**
* BLOCK: chatty-mango-guten-dpf
*
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
*/

//	Import CSS.
import './style.scss';
import './editor.scss';

import Select from 'react-select';
import apiFetch from '@wordpress/api-fetch';

const { __ } = wp.i18n;

const {
  createBlock,
  registerBlockType,
} = wp.blocks;

const {
  InspectorControls,
  PlainText,
} = wp.editor;

const {
  SelectControl,
  PanelBody,
  ToggleControl,
  RangeControl
} = wp.components;

const { Component } = wp.element;

const {
  siteUrl,
  siteLang,
  pluginUrl,
  premiumPluginUrl,
  hasPremium,
  plan,
} = ChattyMangoTagGroupsGlobal;

const helpUrl = 'https://documentation.chattymango.com/documentation/';
const helpProduct = 'tag-groups-premium';
const helpComponent = 'dynamic-post-filter/dynamic-post-filter-parameters/';
const logoUrl = premiumPluginUrl + '/assets/images/cm-tgp-icon-64x64.png';

class TagGroupsHelp extends Component {
  render() {
    let href = helpUrl + helpProduct + "/" + helpComponent;

    if ( '' != siteLang ) {
      href += "?lang=" + siteLang;
    }

    href += "#" + this.props.topic;

    let tooltip = __( 'Click for help!' );

    return (
      <div>
      <a href={href} target="_blank" style={{textDecoration: 'none'}} title={tooltip}>
      <span className="dashicons dashicons-editor-help tg_right chatty-mango-help-icon"></span>
      </a>
      </div>
    );
  }
}


class tagGroupsPremiumDPFParameters extends Component {

  // Method for setting the initial state.
  static getInitialState( attributes ) {
    let selectedGroups = []; // empty means all
    let selectedOpen = [];
    let selectedTaxonomies = ['post_tag'];
    let selectedStaticTaxonomy = ['post_tag'];
    let selectedStaticTerms = [];

    // We need arrays for the select elements.
    if ( attributes.include ) {
      selectedGroups = attributes.include.split(",")
    }
    if ( attributes.open ) {
      selectedOpen = attributes.open.split(",")
    }

    if ( attributes.taxonomy ) {
      selectedTaxonomies = attributes.taxonomy.split(",")
    }

    if ( attributes.taxonomy ) {
      selectedStaticTerms = attributes.static_terms.split(",")
    }

    if ( attributes.static_taxonomy ) {
      selectedStaticTaxonomy = attributes.static_taxonomy;
    }

    return {
      allTerms: [],
      allTaxonomies: {},
      taxonomies: [],
      groups: [],
      selectedStaticTerms,
      selectedTaxonomies,
      selectedGroups,
    };
  }


  // Constructing our component. With super() we are setting everything to 'this'.
  // Now we can access the attributes with this.props.attributes
  constructor() {
    super( ...arguments );

    this.groupsEndPoint = siteUrl + '/wp-json/tag-groups/v1/groups';
    this.termsEndPoint = siteUrl + '/wp-json/tag-groups/v1/terms';
    this.taxonomiesEndPoint = siteUrl + '/wp-json/tag-groups/v1/taxonomies';
    this.allTaxonomiesEndPoint = siteUrl + '/wp-json/wp/v2/taxonomies'

    this.state = this.constructor.getInitialState( this.props.attributes );

    // Bind so we can use 'this' inside the method.
    this.getAllTermsFromApi = this.getAllTermsFromApi.bind( this );
    this.getAllTaxonomiesFromApi = this.getAllTaxonomiesFromApi.bind( this );
    this.getTaxonomiesFromApi = this.getTaxonomiesFromApi.bind( this );
    this.getGroupsFromApi = this.getGroupsFromApi.bind( this );
    this.handleChangeInclude = this.handleChangeInclude.bind( this );
    this.handleChangeOpen = this.handleChangeOpen.bind( this );
    this.handleChangeTaxonomy = this.handleChangeTaxonomy.bind( this );
    this.handleChangeStaticTaxonomy = this.handleChangeStaticTaxonomy.bind( this );
    this.handleChangeStaticTerms = this.handleChangeStaticTerms.bind( this );
    this.toggleOptionPager = this.toggleOptionPager.bind( this );
    this.toggleOptionSumoSelect = this.toggleOptionSumoSelect.bind( this );
    this.toggleOptionDisplayAmount = this.toggleOptionDisplayAmount.bind( this );
    this.toggleOptionHideCloseButton = this.toggleOptionHideCloseButton.bind( this );
    this.toggleOptionGroupsOnly = this.toggleOptionGroupsOnly.bind( this );

    this.props.attributes.operator && this.props.setAttributes( { operator: this.props.attributes.operator.toUpperCase() } );


    // Load data from REST API.
    this.getGroupsFromApi();
    this.getAllTaxonomiesFromApi();
    this.getAllTermsFromApi();
    this.getTaxonomiesFromApi();
  }


  handleChangeInclude( options ) {
    let selectedGroups = options.map( function( option ) {
      // if ( ! isNaN( option.value ) ) {
      return String( option.value );
      // }
    });

    // Set the state
    this.setState( { selectedGroups } );

    // Set the attributes
    this.props.setAttributes( {
      include: selectedGroups.join(',')
    } );

  }

  handleChangeOpen( options ) {
    let selectedOpen = options.map( function( option ) {
      return String( option.value );
    });

    // Set the state
    this.setState( { selectedOpen } );

    // Set the attributes
    this.props.setAttributes( {
      open: selectedOpen.join(',')
    } );

  }

  handleChangeTaxonomy( options ) {
    let selectedTaxonomies = options.map( function( option ) {
      if ( typeof option.value === 'string' ) {
        return option.value;
      }
    });

    // Set the state
    this.setState( { selectedTaxonomies } );

    // Set the attributes
    this.props.setAttributes( {
      taxonomy: selectedTaxonomies.join(',')
    });
  }

  handleChangeStaticTaxonomy( option ) {

    let selectedStaticTaxonomy = option.value;

    // Set the state
    this.setState( { selectedStaticTaxonomy } );

    // Set the attributes
    this.props.setAttributes( {
      static_taxonomy: selectedStaticTaxonomy
    });
  }

  handleChangeStaticTerms( options ) {

    let selectedStaticTerms = options.map( function( option ) {
      return option.value;
    });

    // Set the state
    this.setState( { selectedStaticTerms } );

    // Set the attributes
    this.props.setAttributes( {
      static_terms: selectedStaticTerms.join(',')
    });
  }


  /**
  * Loading Groups
  */
  getGroupsFromApi() {
    // retrieve the groups
    apiFetch( { path: this.groupsEndPoint } ).then( groups => {
      if ( groups ) {
        this.setState({ groups });
      }
    });
  }


  /**
  * Loading Taxonomies (own REST API endpoint)
  */
  getTaxonomiesFromApi() {
    // retrieve the taxonomies
    apiFetch( { path: this.taxonomiesEndPoint } ).then( taxonomies => {
      if ( taxonomies ) {
        this.setState({ taxonomies });
      }
    });
  }


  /**
  * Loading static taxonomies (own REST API endpoint)
  */
  getAllTaxonomiesFromApi() {
    // retrieve the taxonomies
    apiFetch( { path: this.allTaxonomiesEndPoint } ).then( allTaxonomies => {
      if ( allTaxonomies ) {
        this.setState({ allTaxonomies });
      }
    });
  }


  /**
  * Loading Terms (own REST API endpoint)
  */
  getAllTermsFromApi() {
    var termsEndPoint = this.termsEndPoint + '?taxonomy=public&short=1';
    if ( this.props.attributes.allTaxonomies ) {
      termsEndPoint += '&taxonomy=' + this.props.attributes.allTaxonomies;
    }
    // retrieve the terms
    apiFetch( { path: termsEndPoint } ).then( allTerms => {
      if ( allTerms ) {
        this.setState({ allTerms });
      }
    });
  }


  toggleOptionPager() {
    let pager = ( 1 === this.props.attributes.pager ) ? 0 : 1;
    this.props.setAttributes( { pager } );
  }

  toggleOptionSumoSelect() {
    let sumoselect = ( 1 === this.props.attributes.sumoselect ) ? 0 : 1;
    this.props.setAttributes( { sumoselect } );
  }

  toggleOptionDisplayAmount() {
    let display_amount = ( 1 === this.props.attributes.display_amount ) ? 0 : 1;
    this.props.setAttributes( { display_amount } );
  }

  toggleOptionHideCloseButton() {
    let hide_close_buttons = ( 1 === this.props.attributes.hide_close_buttons ) ? 0 : 1;
    this.props.setAttributes( { hide_close_buttons } );
  }

  toggleOptionGroupsOnly() {
    let groups_only = ( 1 === this.props.attributes.groups_only ) ? 0 : 1;
    this.props.setAttributes( { groups_only } );
  }

  render() {
    const {
      attributes,
      setAttributes
    } = this.props;

    const {
      article_class,
      caching_time,
      default_option_label,
      display_amount,
      div_class,
      groups_header,
      groups_only,
      hide_close_buttons,
      include,
      message_amount_plural,
      message_amount_singular,
      message_only_existing,
      message_same_group,
      message_nothing_found,
      message_load_more,
      message_go_back,
      open,
      operator,
      order,
      orderby,
      pager,
      persistent_filter,
      placeholder,
      posts_per_page,
      posts_placeholder,
      static_taxonomy,
      static_terms,
      sumoselect,
      taxonomy,
      template,
      transition,
    } = attributes;

    let optionsTaxonomies = [], optionsStaticTaxonomy = [], optionsStaticTerms = [], optionsGroups = [], optionsOpen = [];

    if( this.state.groups && this.state.groups.length > 0 ) {
      this.state.groups.forEach( ( group ) => {
        optionsGroups.push({ value:group.term_group, label:group.label });
      });
    }

    if( this.state.groups && this.state.groups.length > 0 ) {
      this.state.groups.forEach( ( group ) => {
        if ( this.state.selectedGroups.length == 0 || this.state.selectedGroups.indexOf( String( group.term_group ) ) > -1 ) {
          optionsOpen.push( { value:group.term_group, label:group.label } );
        }
      });
    }

    if( this.state.taxonomies && this.state.taxonomies.length > 0 ) {
      this.state.taxonomies.forEach( ( taxonomy ) => {
        optionsTaxonomies.push({ value:taxonomy.slug, label:taxonomy.name });
      });
    }

    if( this.state.allTaxonomies ) {
      for ( var key in this.state.allTaxonomies ) {
        if ( this.state.allTaxonomies.hasOwnProperty( key ) ) {
          optionsStaticTaxonomy.push({ value:this.state.allTaxonomies[key].slug, label:this.state.allTaxonomies[key].name });
        }
      }
    }

    if( this.state.allTerms ) {
      this.state.allTerms.forEach( ( staticTerm ) => {
        if ( staticTerm.taxonomy == this.state.selectedStaticTaxonomy ) {
          optionsStaticTerms.push({ value:staticTerm.id, label:staticTerm.name });
        }
      });
    }

    if ( attributes.source !== 'gutenberg' ) {
      setAttributes({ source: 'gutenberg' });
    }

    return [
      (
        <InspectorControls key='inspector'>
        <div className='chatt-mango-inspector-control'>
        <PanelBody title={ __( 'Tags and Terms' ) } initialOpen={false}>
        <TagGroupsHelp topic="taxonomy"/>
        <label htmlFor="tg_input_taxonomy">
        { __( 'Include taxonomies' ) }
        </label>
        <Select
        id="tg_input_taxonomy"
        onChange={this.handleChangeTaxonomy}
        value={this.state.selectedTaxonomies}
        options={ optionsTaxonomies }
        multi={ true }
        closeOnSelect={ false}
        removeSelected={ true }
        />
        <TagGroupsHelp topic="operator"/>
        <label htmlFor="tg_input_operator">
        { __( 'Logical Operator' ) }
        </label>
        <Select
        id="tg_input_operator"
        onChange={ ( option ) => { if ( option ) setAttributes( { operator: option.value } ) } }
        value={ ( operator && ( typeof operator === 'string' ) ) ? operator : 'IN' }
        options={ [
          { value:'IN', label:__( 'any tag matches' ) },
          { value:'AND', label:__( 'all tags match' ) },
          { value:'IN AND', label:__( 'at least one tag matches from each group' ) },
        ] }
        />
        </PanelBody>

        <PanelBody title={ __( 'Groups' ) } initialOpen={false}>
        <TagGroupsHelp topic="include"/>
        <label htmlFor="tg_input_include">
        { __( 'Include groups' ) }
        </label>
        <Select
        id="tg_input_include"
        onChange={ this.handleChangeInclude }
        value={ this.state.selectedGroups }
        options={ optionsGroups }
        multi={ true }
        closeOnSelect={ false}
        removeSelected={ true }
        />
        { ( this.state.selectedGroups.length > 0 ) &&
          <div>
          <TagGroupsHelp topic="open"/>
          <label htmlFor="tg_input_open">
          { __( 'Initially open' ) }
          </label>
          <Select
          id="tg_input_open"
          onChange={ this.handleChangeOpen }
          value={ this.state.selectedOpen }
          options={ optionsOpen }
          multi={ true }
          closeOnSelect={ false}
          removeSelected={ true }
          />
          </div>
        }
        <TagGroupsHelp topic="groups_only"/>
        <ToggleControl
        label={ __( 'Show groups only.' ) }
        checked={ groups_only }
        onChange={ this.toggleOptionGroupsOnly }
        />
        <TagGroupsHelp topic="hide_close_buttons"/>
        <ToggleControl
        label={ __( 'Hide the close button.' ) }
        checked={ hide_close_buttons }
        onChange={ this.toggleOptionHideCloseButton }
        />
        </PanelBody>

        <PanelBody title={ __( 'Posts' ) } initialOpen={false}>
        <TagGroupsHelp topic="posts_per_page"/>
        <RangeControl
        label={ __( 'Posts per page' ) }
        value={ posts_per_page ? Number( posts_per_page ) : 22 }
        onChange={ ( posts_per_page ) => setAttributes( { posts_per_page } ) }
        min={ 1 }
        max={ 100 }
        />
        <TagGroupsHelp topic="pager"/>
        <ToggleControl
        label={ __( 'Use a pager.' ) }
        checked={ pager }
        onChange={ this.toggleOptionPager }
        />
        <TagGroupsHelp topic="caching_time"/>
        <RangeControl
        label={ __( 'Caching time (minutes)' ) }
        value={ caching_time ? Number( caching_time ) : 10 }
        onChange={ ( caching_time ) => setAttributes( { caching_time } ) }
        min={ 0 }
        max={ 600 }
        />
        <TagGroupsHelp topic="orderby"/>
        <label htmlFor="tg_input_orderby">
        { __( 'Order posts by' ) }
        </label>
        <Select
        id="tg_input_orderby"
        onChange={ ( option ) => { if ( option ) setAttributes( { orderby: option.value } ) } }
        value={ orderby && typeof orderby === 'string' ? orderby : 'name' }
        options={ [
          { value:'author', label:__('Author') },
          { value:'date', label:__('Date') },
          { value:'modified', label:__('Modified') },
          { value:'title', label:__('Title') },
          { value:'comment_count', label:__('Comment count') },
        ] }
        />
        <TagGroupsHelp topic="order"/>
        <label htmlFor="tg_input_order">
        { __( 'Sort order' ) }
        </label>
        <Select
        id="tg_input_order"
        onChange={ ( option ) => { if ( option ) setAttributes( { order: option.value } ) } }
        value={ order && typeof order === 'string' ? order.toUpperCase() : 'DESC' }
        options={ [
          { value:'ASC', label:__('Ascending') },
          { value:'DESC', label:__('Descending') }
        ] }
        />
        <TagGroupsHelp topic="display_amount"/>
        <ToggleControl
        label={ __( 'Display the total amount of posts.' ) }
        checked={ display_amount }
        onChange={ this.toggleOptionDisplayAmount }
        />
        <TagGroupsHelp topic="template"/>
        <label htmlFor="tg_input_template">
        { __( 'Post template' ) }
        </label>
        <PlainText
        id="tg_input_template"
        className="input-control"
        value={ template ? template : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( template ) => setAttributes( { template } ) }
        className="cm-three-line-input"
        />
        <TagGroupsHelp topic="transition"/>
        <label htmlFor="tg_input_transition">
        { __( 'Transition' ) }
        </label>
        <Select
        id="tg_input_transition"
        onChange={ ( option ) => { if ( option ) setAttributes( { transition: option.value } ) } }
        value={ ( transition && ( typeof transition === 'string' ) ) ? transition : '' }
        options={ [
          { value:'', label:__( 'none' ) },
          { value:'fade', label:__( 'fade' ) },
        ] }
        />
        <TagGroupsHelp topic="static_taxonomy"/>
        <label htmlFor="tg_input_static_taxonomy">
        { __( 'Static taxonomies' ) }
        </label>
        <Select
        id="tg_input_static_taxonomy"
        onChange={ this.handleChangeStaticTaxonomy }
        value={ this.state.selectedStaticTaxonomy }
        options={ optionsStaticTaxonomy }
        multi={ false }
        closeOnSelect={ true }
        />
        { this.state.selectedStaticTaxonomy &&
          <div>
          <TagGroupsHelp topic="static_terms"/>
          <label htmlFor="tg_input_static_terms">
          { __( 'Static terms' ) }
          </label>
          <Select
          id="tg_input_static_terms"
          onChange={ this.handleChangeStaticTerms }
          value={ this.state.selectedStaticTerms }
          options={ optionsStaticTerms }
          multi={ true }
          closeOnSelect={ false }
          removeSelected={ true }
          />
          </div>
        }
        </PanelBody>

        <PanelBody title={ __( 'Labels and Messages' ) } initialOpen={false}>
        { (!groups_only) &&
          <div>
          <TagGroupsHelp topic="placeholder"/>
          <label htmlFor="tg_input_placeholder">
          { __( 'Placeholder for tag field' ) }
          </label>
          <PlainText
          id="tg_input_placeholder"
          className="input-control"
          value={ placeholder ? placeholder : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( placeholder ) => setAttributes( { placeholder } ) }
          />
          </div>
        }
        <TagGroupsHelp topic="posts_placeholder"/>
        <label htmlFor="tg_input_posts_placeholder">
        { __( 'Placeholder for posts field' ) }
        </label>
        <PlainText
        id="tg_input_posts_placeholder"
        className="input-control"
        value={ posts_placeholder ? posts_placeholder : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( posts_placeholder ) => setAttributes( { posts_placeholder } ) }
        />
        <TagGroupsHelp topic="default_option_label"/>
        <label htmlFor="tg_input_default_option_label">
        { __( 'Label for default option' ) }
        </label>
        <PlainText
        id="tg_input_default_option_label"
        className="input-control"
        value={ default_option_label ? default_option_label : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( default_option_label ) => setAttributes( { default_option_label } ) }
        />
        <TagGroupsHelp topic="groups_header"/>
        <label htmlFor="tg_input_groups_header">
        { __( 'Header above groups' ) }
        </label>
        <PlainText
        id="tg_input_groups_header"
        className="input-control"
        value={ groups_header ? groups_header : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( groups_header ) => setAttributes( { groups_header } ) }
        />
        { (!groups_only) &&
          <div>
          <TagGroupsHelp topic="message_only_existing"/>
          <label htmlFor="tg_input_message_only_existing">
          { __( 'Alert to use only existing tags' ) }
          </label>
          <PlainText
          id="tg_input_message_only_existing"
          className="input-control"
          value={ message_only_existing ? message_only_existing : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_only_existing ) => setAttributes( { message_only_existing } ) }
          />
          <TagGroupsHelp topic="message_same_group"/>
          <label htmlFor="tg_input_message_same_group">
          { __( 'Alert to use only tags from correct group' ) }
          </label>
          <PlainText
          id="tg_input_message_same_group"
          className="input-control"
          value={ message_same_group ? message_same_group : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_same_group ) => setAttributes( { message_same_group } ) }
          />
          </div>
        }
        <TagGroupsHelp topic="message_nothing_found"/>
        <label htmlFor="tg_input_message_nothing_found">
        { __( 'Message if nothing was found' ) }
        </label>
        <PlainText
        id="tg_input_message_nothing_found"
        className="input-control"
        value={ message_nothing_found ? message_nothing_found : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( message_nothing_found ) => setAttributes( { message_nothing_found } ) }
        />
        { pager == true &&
          <div>
          <TagGroupsHelp topic="message_load_more"/>
          <label htmlFor="tg_input_message_load_more">
          { __( 'Link to load more posts' ) }
          </label>
          <PlainText
          id="tg_input_message_load_more"
          className="input-control"
          value={ message_load_more ? message_load_more : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_load_more ) => setAttributes( { message_load_more } ) }
          />
          <TagGroupsHelp topic="message_go_back"/>
          <label htmlFor="tg_input_message_go_back">
          { __( 'Link to load previous posts' ) }
          </label>
          <PlainText
          id="tg_input_message_go_back"
          className="input-control"
          value={ message_go_back ? message_go_back : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_go_back ) => setAttributes( { message_go_back } ) }
          />
          </div>
        }
        { display_amount == true &&
          <div>
          <TagGroupsHelp topic="message_amount_singular"/>
          <label htmlFor="tg_input_message_amount_singular">
          { __( 'Message for one post in total' ) }
          </label>
          <PlainText
          id="tg_input_message_amount_singular"
          className="input-control"
          value={ message_amount_singular ? message_amount_singular : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_amount_singular ) => setAttributes( { message_amount_singular } ) }
          />
          <TagGroupsHelp topic="message_amount_plural"/>
          <label htmlFor="tg_input_message_amount_plural">
          { __( 'Message for {count} posts in total' ) }
          </label>
          <PlainText
          id="tg_input_message_amount_plural"
          className="input-control"
          value={ message_amount_plural ? message_amount_plural : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_amount_plural ) => setAttributes( { message_amount_plural } ) }
          />
          </div>
        }
        </PanelBody>

        <PanelBody title={ __( 'Other' ) } initialOpen={false}>
        <TagGroupsHelp topic="persistent_filter"/>
        <RangeControl
        label={ __( 'Remember selected filter and post (minutes)' ) }
        value={ persistent_filter ? Number( persistent_filter ) : 0 }
        onChange={ ( persistent_filter ) => setAttributes( { persistent_filter } ) }
        min={ 0 }
        max={ 600 }
        />
        </PanelBody>

        <PanelBody title={ __( 'Advanced Styling' ) } initialOpen={false}>
        <TagGroupsHelp topic="sumoselect"/>
        <ToggleControl
        label={ __( 'Use styling for selector.' ) }
        checked={ sumoselect }
        onChange={ this.toggleOptionSumoSelect }
        />

        <div>
        <TagGroupsHelp topic="div_class"/>
        <label htmlFor="tg_input_div_class">
        { '<div class="...">' }
        </label>
        </div>
        <PlainText
        id="tg_input_div_class"
        className="input-control"
        value={ div_class ? div_class : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( div_class ) => setAttributes( { div_class } ) }
        />
        <div>
        <TagGroupsHelp topic="article_class"/>
        <label htmlFor="tg_input_article_class">
        { '<article class="...">' }
        </label>
        </div>
        <PlainText
        id="tg_input_article_class"
        className="input-control"
        value={ article_class ? article_class : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( article_class ) => setAttributes( { article_class } ) }
        />
        </PanelBody>
        <div className="chatty-mango-help-transform">
        <TagGroupsHelp topic="transform-your-block-for-more-options"/>
        <div dangerouslySetInnerHTML={{ __html: __( 'If you want to customize further options, you need to transform the block into a <b>shortcode block</b>.' ) }}>
        </div>
        </div>
        </div>
        </InspectorControls>
      ),
      <div className="chatty-mango-editor">
      <table style={{border:'none'}}>
      <tr>
      <td>
      <img src={logoUrl} alt='logo' style={{float:'left', margin:15}}/>
      </td>
      <td>
      <h3>{ __( 'Dynamic Post Filter' ) }</h3>
      <div className="cm-gutenberg dashicons-before dashicons-admin-generic">
      { __( 'Select this block and customize the filter in the Inspector.' ) }
      </div>
      <div className="cm-gutenberg dashicons-before dashicons-welcome-view-site">
      { __( 'See the output with Preview.' ) }
      </div>
      </td>
      </tr>
      </table>
      </div>
    ]
  }

}


/**
* Register: a Gutenberg Block.
*
* @param  {string}	  name	   Block name.
* @param  {Object}	  settings Block settings.
* @return {?WPBlock}		   The block, if it has been successfully
*							   registered; otherwise `undefined`.
*/
if ( plan === 'premium' ) {

  var cmTagGroupsDPFBlock = registerBlockType( 'chatty-mango/tag-groups-premium-dpf', {
    title: __( 'Dynamic Post Filter' ),
    icon: 'filter', // Block icon from Dashicons → https://developer.wordpress.org/resource/dashicons/.
    category: 'widgets',
    description: __( 'Show a list of posts that corresponds to tags entered by your visitors.' ),
    keywords: [
      __( 'post filter' ),
      __( 'tags' ),
      'Chatty Mango',
    ],
    html: false,
    useOnce: true,
    transforms: {
      to: [
        {
          type: 'block',
          blocks: [ 'core/shortcode' ],
          transform: function( attributes ) {
            let parameters = [];
            for ( var attribute in attributes ) {
              if (attributes.hasOwnProperty( attribute )) {
                if ( null !== attributes[attribute] && '' !== attributes[ attribute ] && 'source' !== attribute && cmTagGroupsDPFBlock.attributes[ attribute ] && attributes[ attribute ] !== cmTagGroupsDPFBlock.attributes[ attribute ].default ) {
                  if ( typeof attributes[attribute] === 'number' ) {
                    parameters.push( attribute + '=' + attributes[ attribute ] );
                  } else {
                    parameters.push( attribute + '="' + attributes[ attribute ] + '"');
                  }
                }
              }
            }

            let text = '[tag_groups_dpf ' + parameters.join(' ') + ']';
            return createBlock( 'core/shortcode', {
              text
            } );
          },
        },
      ],
    },
    supports: {
      html: false,
    },

    /**
    * Attributes are the same as shortcode parameters
    **/
    attributes: {
      source: { // internal indicator to identify Gutebergb blocks
        type: 'string',
        default: ''
      },
      article_class: { // configurable in block
        type: 'string',
        default: 'tg-post'
      },
      caching_time: {// configurable in block
        type: 'integer',
        default: 10
      },
      default_option_label: {// configurable in block
        type: 'string',
        default: __( 'Filter by group' )
      },
      display_amount: {// configurable in block
        type: 'integer',
        default: 0
      },
      div_class: {// configurable in block
        type: 'string',
        default: ''
      },
      groups_header: {// configurable in block
        type: 'string',
        default: __( 'Selected groups:' )
      },
      groups_only: {// configurable in block
        type: 'integer',
        default: 0
      },
      hide_close_buttons: {// configurable in block
        type: 'integer',
        default: 0
      },
      include: {// configurable in block
        type: 'string',
        default: ''
      },
      message_only_existing: {// configurable in block
        type: 'string',
        default: __( 'Use only existing tags.' )
      },
      message_amount_plural: {// configurable in block
        type: 'string',
        default: __( '{count} posts found.' )
      },
      message_amount_singular: {// configurable in block
        type: 'string',
        default: __( '1 post found.' )
      },
      message_same_group: {// configurable in block
        type: 'string',
        default: __( 'Not allowed to use tags from other groups.' )
      },
      message_nothing_found: {// configurable in block
        type: 'string',
        default: __( 'Nothing found.' )
      },
      message_load_more: {// configurable in block
        type: 'string',
        default: __( 'Load more' )
      },
      message_go_back: {// configurable in block
        type: 'string',
        default: __( 'Go back' )
      },
      open: {// configurable in block
        type: 'string',
        default: ''
      },
      operator: {// configurable in block
        type: 'string',
        default: 'IN'
      },
      order: {// configurable in block
        type: 'string',
        default: 'DESC'
      },
      orderby: {// configurable in block
        type: 'string',
        default: 'date'
      },
      pager: {// configurable in block
        type: 'integer',
        default: 0
      },
      persistent_filter: {// configurable in block
        type: 'integer',
        default: 0
      },
      placeholder: {// only in shortcode
        type: 'string',
        default: __( 'Please enter your tags here.' )
      },
      posts_per_page: {// configurable in block
        type: 'integer',
        default: 5
      },
      posts_placeholder: {// configurable in block
        type: 'string',
        default: __( 'Please select a group.' )
      },
      static_taxonomy: {// configurable in block
        type: 'string',
        default: ''
      },
      static_terms: {// configurable in block
        type: 'string',
        default: ''
      },
      sumoselect: {// configurable in block
        type: 'integer',
        default: 1
      },
      taxonomy: {// configurable in block
        type: 'string',
        default: ''
      },
      template: {// configurable in block
        type: 'string',
        default: '' // Needs to be empty to apply default template from settings
      },
      transition: {// configurable in block
        type: 'string',
        default: ''
      },
    },

    /**
    * Composing and rendering the editor content and control elements
    */
    edit: tagGroupsPremiumDPFParameters,


    /**
    * We don't render any HTML when saving
    */
    save: function( props ) {
      return null;
    },
  } );
}
