/**
* BLOCK: chatty-mango-guten-tag-post-list
*
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2019 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @since       1.18.0
*/

//	Import CSS.
import './style.scss';
import './editor.scss';

import Select from 'react-select';
import apiFetch from '@wordpress/api-fetch';

const { __ } = wp.i18n;

const {
  createBlock,
  registerBlockType,
} = wp.blocks;

const {
  InspectorControls,
  PlainText,
} = wp.editor;

const {
  SelectControl,
  PanelBody,
  ToggleControl,
  RangeControl
} = wp.components;

const { Component } = wp.element;

const {
  siteUrl,
  siteLang,
  pluginUrl,
  premiumPluginUrl,
  hasPremium,
  plan,
} = ChattyMangoTagGroupsGlobal;

const helpUrl = 'https://documentation.chattymango.com/documentation/';
const helpProduct = 'tag-groups-premium';
const helpComponent = 'post-list/post-list-parameters/';
const logoUrl = premiumPluginUrl + '/assets/images/cm-tgp-icon-64x64.png';

class TagGroupsHelp extends Component {
  render() {
    let href = helpUrl + helpProduct + "/" + helpComponent;

    if ( '' != siteLang ) {
      href += "?lang=" + siteLang;
    }

    href += "#" + this.props.topic;

    let tooltip = __( 'Click for help!' );

    return (
      <div>
      <a href={href} target="_blank" style={{textDecoration: 'none'}} title={tooltip}>
      <span className="dashicons dashicons-editor-help tg_right chatty-mango-help-icon"></span>
      </a>
      </div>
    );
  }
}


class tagGroupsPremiumPostListParameters extends Component {

  // Method for setting the initial state.
  static getInitialState( attributes ) {
    let selectedGroups = []; // empty means all
    let selectedTaxonomies = ['post_tag'];

    // We need arrays for the select elements.
    if ( attributes.include ) {
      selectedGroups = attributes.include.split(",")
    }

    if ( attributes.taxonomy ) {
      selectedTaxonomies = attributes.taxonomy.split(",")
    }

    return {
      taxonomies: [],
      groups: [],
      selectedGroups,
    };
  }


  // Constructing our component. With super() we are setting everything to 'this'.
  // Now we can access the attributes with this.props.attributes
  constructor() {
    super( ...arguments );

    this.groupsEndPoint = siteUrl + '/wp-json/tag-groups/v1/groups';
    this.taxonomiesEndPoint = siteUrl + '/wp-json/tag-groups/v1/taxonomies';

    this.state = this.constructor.getInitialState( this.props.attributes );

    // Bind so we can use 'this' inside the method.
    // this.getAllTermsFromApi = this.getAllTermsFromApi.bind( this );
    // this.getAllTaxonomiesFromApi = this.getAllTaxonomiesFromApi.bind( this );
    this.getTaxonomiesFromApi = this.getTaxonomiesFromApi.bind( this );
    this.getGroupsFromApi = this.getGroupsFromApi.bind( this );
    this.handleChangeInclude = this.handleChangeInclude.bind( this );
    this.handleChangeTaxonomy = this.handleChangeTaxonomy.bind( this );
    this.toggleOptionPager = this.toggleOptionPager.bind( this );
    this.toggleOptionDisplayAmount = this.toggleOptionDisplayAmount.bind( this );

    this.props.attributes.operator && this.props.setAttributes( { operator: this.props.attributes.operator.toUpperCase() } );


    // Load data from REST API.
    this.getGroupsFromApi();
    this.getTaxonomiesFromApi();
  }


  handleChangeInclude( options ) {
    let selectedGroups = options.map( function( option ) {
      // if ( ! isNaN( option.value ) ) {
      return String( option.value );
      // }
    });

    // Set the state
    this.setState( { selectedGroups } );

    // Set the attributes
    this.props.setAttributes( {
      include: selectedGroups.join(',')
    } );

  }

  handleChangeTaxonomy( options ) {
    let selectedTaxonomies = options.map( function( option ) {
      if ( typeof option.value === 'string' ) {
        return option.value;
      }
    });

    // Set the state
    this.setState( { selectedTaxonomies } );

    // Set the attributes
    this.props.setAttributes( {
      taxonomy: selectedTaxonomies.join(',')
    });
  }


  /**
  * Loading Groups
  */
  getGroupsFromApi() {
    // retrieve the groups
    apiFetch( { path: this.groupsEndPoint } ).then( groups => {
      if ( groups ) {
        this.setState({ groups });
      }
    });
  }


  /**
  * Loading Taxonomies (own REST API endpoint)
  */
  getTaxonomiesFromApi() {
    // retrieve the taxonomies
    apiFetch( { path: this.taxonomiesEndPoint } ).then( taxonomies => {
      if ( taxonomies ) {
        this.setState({ taxonomies });
      }
    });
  }

  toggleOptionPager() {
    let pager = ( 1 === this.props.attributes.pager ) ? 0 : 1;
    this.props.setAttributes( { pager } );
  }

  toggleOptionDisplayAmount() {
    let display_amount = ( 1 === this.props.attributes.display_amount ) ? 0 : 1;
    this.props.setAttributes( { display_amount } );
  }

  render() {
    const {
      attributes,
      setAttributes
    } = this.props;

    const {
      article_class,
      caching_time,
      default_image_src,
      display_amount,
      div_class,
      include,
      message_amount_plural,
      message_amount_singular,
      message_nothing_found,
      message_load_more,
      message_go_back,
      operator,
      order,
      orderby,
      pager,
      posts_per_page,
      posts_placeholder,
      taxonomy,
      template,
    } = attributes;

    let optionsTaxonomies = [], optionsGroups = [];

    if( this.state.groups && this.state.groups.length > 0 ) {
      this.state.groups.forEach( ( group ) => {
        optionsGroups.push({ value:group.term_group, label:group.label });
      });
    }

    if( this.state.taxonomies && this.state.taxonomies.length > 0 ) {
      this.state.taxonomies.forEach( ( taxonomy ) => {
        optionsTaxonomies.push({ value:taxonomy.slug, label:taxonomy.name });
      });
    }

    if ( attributes.source !== 'gutenberg' ) {
      setAttributes({ source: 'gutenberg' });
    }

    return [
      (
        <InspectorControls key='inspector'>
        <div className='chatt-mango-inspector-control'>
        <PanelBody title={ __( 'Tags and Terms' ) } initialOpen={false}>
        <TagGroupsHelp topic="taxonomy"/>
        <label htmlFor="tg_input_taxonomy">
        { __( 'Include taxonomies' ) }
        </label>
        <Select
        id="tg_input_taxonomy"
        onChange={this.handleChangeTaxonomy}
        value={this.state.selectedTaxonomies}
        options={ optionsTaxonomies }
        multi={ true }
        closeOnSelect={ false}
        removeSelected={ true }
        />
        <TagGroupsHelp topic="operator"/>
        <label htmlFor="tg_input_operator">
        { __( 'Logical Operator' ) }
        </label>
        <Select
        id="tg_input_operator"
        onChange={ ( option ) => { if ( option ) setAttributes( { operator: option.value } ) } }
        value={ ( operator && ( typeof operator === 'string' ) ) ? operator : 'IN' }
        options={ [
          { value:'IN', label:__( 'any group' ) },
          { value:'AND', label:__( 'every groups' ) },
        ] }
        />
        </PanelBody>

        <PanelBody title={ __( 'Groups' ) } initialOpen={false}>
        <TagGroupsHelp topic="include"/>
        <label htmlFor="tg_input_include">
        { __( 'Include groups' ) }
        </label>
        <Select
        id="tg_input_include"
        onChange={ this.handleChangeInclude }
        value={ this.state.selectedGroups }
        options={ optionsGroups }
        multi={ true }
        closeOnSelect={ false}
        removeSelected={ true }
        />
        </PanelBody>

        <PanelBody title={ __( 'Posts' ) } initialOpen={false}>
        <TagGroupsHelp topic="posts_per_page"/>
        <RangeControl
        label={ __( 'Posts per page' ) }
        value={ posts_per_page ? Number( posts_per_page ) : 22 }
        onChange={ ( posts_per_page ) => setAttributes( { posts_per_page } ) }
        min={ 1 }
        max={ 100 }
        />
        <TagGroupsHelp topic="pager"/>
        <ToggleControl
        label={ __( 'Use a pager.' ) }
        checked={ pager }
        onChange={ this.toggleOptionPager }
        />
        <TagGroupsHelp topic="caching_time"/>
        <RangeControl
        label={ __( 'Caching time (minutes)' ) }
        value={ caching_time ? Number( caching_time ) : 10 }
        onChange={ ( caching_time ) => setAttributes( { caching_time } ) }
        min={ 0 }
        max={ 600 }
        />
        <TagGroupsHelp topic="orderby"/>
        <label htmlFor="tg_input_orderby">
        { __( 'Order posts by' ) }
        </label>
        <Select
        id="tg_input_orderby"
        onChange={ ( option ) => { if ( option ) setAttributes( { orderby: option.value } ) } }
        value={ orderby && typeof orderby === 'string' ? orderby : 'name' }
        options={ [
          { value:'author', label:__('Author') },
          { value:'date', label:__('Date') },
          { value:'modified', label:__('Modified') },
          { value:'title', label:__('Title') },
          { value:'comment_count', label:__('Comment count') },
        ] }
        />
        <TagGroupsHelp topic="order"/>
        <label htmlFor="tg_input_order">
        { __( 'Sort order' ) }
        </label>
        <Select
        id="tg_input_order"
        onChange={ ( option ) => { if ( option ) setAttributes( { order: option.value } ) } }
        value={ order && typeof order === 'string' ? order.toUpperCase() : 'DESC' }
        options={ [
          { value:'ASC', label:__('Ascending') },
          { value:'DESC', label:__('Descending') }
        ] }
        />
        <TagGroupsHelp topic="display_amount"/>
        <ToggleControl
        label={ __( 'Display the total amount of posts.' ) }
        checked={ display_amount }
        onChange={ this.toggleOptionDisplayAmount }
        />
        <TagGroupsHelp topic="template"/>
        <label htmlFor="tg_input_template">
        { __( 'Post template' ) }
        </label>
        <PlainText
        id="tg_input_template"
        className="input-control"
        value={ template ? template : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( template ) => setAttributes( { template } ) }
        className="cm-three-line-input"
        />
        <TagGroupsHelp topic="default_image_src"/>
        <label htmlFor="tg_input_default_image_src">
        { __( 'Default image src attribute' ) }
        </label>
        <PlainText
        id="tg_input_default_image_src"
        className="input-control"
        value={ default_image_src ? default_image_src : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( default_image_src ) => setAttributes( { default_image_src } ) }
        />
        </PanelBody>

        <PanelBody title={ __( 'Labels and Messages' ) } initialOpen={false}>
        <TagGroupsHelp topic="message_nothing_found"/>
        <label htmlFor="tg_input_message_nothing_found">
        { __( 'Message if nothing was found' ) }
        </label>
        <PlainText
        id="tg_input_message_nothing_found"
        className="input-control"
        value={ message_nothing_found ? message_nothing_found : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( message_nothing_found ) => setAttributes( { message_nothing_found } ) }
        />
        { pager == true &&
          <div>
          <TagGroupsHelp topic="message_load_more"/>
          <label htmlFor="tg_input_message_load_more">
          { __( 'Link to load more posts' ) }
          </label>
          <PlainText
          id="tg_input_message_load_more"
          className="input-control"
          value={ message_load_more ? message_load_more : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_load_more ) => setAttributes( { message_load_more } ) }
          />
          <TagGroupsHelp topic="message_go_back"/>
          <label htmlFor="tg_input_message_go_back">
          { __( 'Link to load previous posts' ) }
          </label>
          <PlainText
          id="tg_input_message_go_back"
          className="input-control"
          value={ message_go_back ? message_go_back : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_go_back ) => setAttributes( { message_go_back } ) }
          />
          </div>
        }
        { display_amount == true &&
          <div>
          <TagGroupsHelp topic="message_amount_singular"/>
          <label htmlFor="tg_input_message_amount_singular">
          { __( 'Message for one post in total' ) }
          </label>
          <PlainText
          id="tg_input_message_amount_singular"
          className="input-control"
          value={ message_amount_singular ? message_amount_singular : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_amount_singular ) => setAttributes( { message_amount_singular } ) }
          />
          <TagGroupsHelp topic="message_amount_plural"/>
          <label htmlFor="tg_input_message_amount_plural">
          { __( 'Message for {count} posts in total' ) }
          </label>
          <PlainText
          id="tg_input_message_amount_plural"
          className="input-control"
          value={ message_amount_plural ? message_amount_plural : '' }
          placeholder={ __( 'Write here or leave empty.' ) }
          onChange={ ( message_amount_plural ) => setAttributes( { message_amount_plural } ) }
          />
          </div>
        }
        </PanelBody>

        <PanelBody title={ __( 'Advanced Styling' ) } initialOpen={false}>
        <div>
        <TagGroupsHelp topic="div_class"/>
        <label htmlFor="tg_input_div_class">
        { '<div class="...">' }
        </label>
        </div>
        <PlainText
        id="tg_input_div_class"
        className="input-control"
        value={ div_class ? div_class : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( div_class ) => setAttributes( { div_class } ) }
        />
        <div>
        <TagGroupsHelp topic="article_class"/>
        <label htmlFor="tg_input_article_class">
        { '<article class="...">' }
        </label>
        </div>
        <PlainText
        id="tg_input_article_class"
        className="input-control"
        value={ article_class ? article_class : '' }
        placeholder={ __( 'Write here or leave empty.' ) }
        onChange={ ( article_class ) => setAttributes( { article_class } ) }
        />
        </PanelBody>
        <div className="chatty-mango-help-transform">
        <TagGroupsHelp topic="transform-your-block-for-more-options"/>
        <div dangerouslySetInnerHTML={{ __html: __( 'If you want to customize further options, you need to transform the block into a <b>shortcode block</b>.' ) }}>
        </div>
        </div>
        </div>
        </InspectorControls>
      ),
      <div className="chatty-mango-editor">
      <table style={{border:'none'}}>
      <tr>
      <td>
      <img src={logoUrl} alt='logo' style={{float:'left', margin:15}}/>
      </td>
      <td>
      <h3>{ __( 'Post List' ) }</h3>
      <div className="cm-gutenberg dashicons-before dashicons-admin-generic">
      { __( 'Select this block and customize the filter in the Inspector.' ) }
      </div>
      <div className="cm-gutenberg dashicons-before dashicons-welcome-view-site">
      { __( 'See the output with Preview.' ) }
      </div>
      </td>
      </tr>
      </table>
      </div>
    ]
  }

}


/**
* Register: a Gutenberg Block.
*
* @param  {string}	  name	   Block name.
* @param  {Object}	  settings Block settings.
* @return {?WPBlock}		   The block, if it has been successfully
*							   registered; otherwise `undefined`.
*/
if ( plan === 'premium' ) {

  var cmTagGroupsPostFilterBlock = registerBlockType( 'chatty-mango/tag-groups-premium-post-filter', {
    title: __( 'Post List' ),
    icon: 'list-view', // Block icon from Dashicons → https://developer.wordpress.org/resource/dashicons/.
    category: 'widgets',
    description: __( 'Show a list of posts that corresponds to tag groups.' ),
    keywords: [
      __( 'posts' ),
      __( 'tags' ),
      'Chatty Mango',
    ],
    html: false,
    useOnce: false,
    transforms: {
      to: [
        {
          type: 'block',
          blocks: [ 'core/shortcode' ],
          transform: function( attributes ) {
            let parameters = [];
            for ( var attribute in attributes ) {
              if (attributes.hasOwnProperty( attribute )) {
                if ( null !== attributes[attribute] && '' !== attributes[ attribute ] && 'source' !== attribute && cmTagGroupsPostFilterBlock.attributes[ attribute ] && attributes[ attribute ] !== cmTagGroupsPostFilterBlock.attributes[ attribute ].default ) {
                  if ( typeof attributes[attribute] === 'number' ) {
                    parameters.push( attribute + '=' + attributes[ attribute ] );
                  } else {
                    parameters.push( attribute + '="' + attributes[ attribute ] + '"');
                  }
                }
              }
            }

            let text = '[tag_groups_post_list ' + parameters.join(' ') + ']';
            return createBlock( 'core/shortcode', {
              text
            } );
          },
        },
      ],
    },
    supports: {
      html: false,
    },

    /**
    * Attributes are the same as shortcode parameters
    **/
    attributes: {
      source: { // internal indicator to identify Gutebergb blocks
        type: 'string',
        default: ''
      },
      article_class: { // configurable in block
        type: 'string',
        default: 'tg-post'
      },
      caching_time: {// configurable in block
        type: 'integer',
        default: 10
      },
      default_image_src: {// configurable in block
        type: 'string',
        default: ''
      },
      display_amount: {// configurable in block
        type: 'integer',
        default: 0
      },
      div_class: {// configurable in block
        type: 'string',
        default: ''
      },
      include: {// configurable in block
        type: 'string',
        default: ''
      },
      message_amount_plural: {// configurable in block
        type: 'string',
        default: __( '{count} posts found.' )
      },
      message_amount_singular: {// configurable in block
        type: 'string',
        default: __( '1 post found.' )
      },
      message_nothing_found: {// configurable in block
        type: 'string',
        default: __( 'Nothing found.' )
      },
      message_load_more: {// configurable in block
        type: 'string',
        default: __( 'Load more' )
      },
      message_go_back: {// configurable in block
        type: 'string',
        default: __( 'Go back' )
      },
      operator: {// configurable in block
        type: 'string',
        default: 'IN'
      },
      order: {// configurable in block
        type: 'string',
        default: 'DESC'
      },
      orderby: {// configurable in block
        type: 'string',
        default: 'date'
      },
      pager: {// configurable in block
        type: 'integer',
        default: 0
      },
      posts_per_page: {// configurable in block
        type: 'integer',
        default: 5
      },
      taxonomy: {// configurable in block
        type: 'string',
        default: ''
      },
      template: {// configurable in block
        type: 'string',
        default: '' // Needs to be empty to apply default template from settings
      },
    },

    /**
    * Composing and rendering the editor content and control elements
    */
    edit: tagGroupsPremiumPostListParameters,


    /**
    * We don't render any HTML when saving
    */
    save: function( props ) {
      return null;
    },
  } );
}
