/**
* BLOCK: chatty-mango-guten-tag-accorion
*
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)

*/

//	Import CSS.
import './style.scss';
import './editor.scss';

import Select from 'react-select';
import apiFetch from '@wordpress/api-fetch';

const { __ } = wp.i18n;

const {
  createBlock,
  registerBlockType,
} = wp.blocks;

const {
  InspectorControls,
  PlainText,
} = wp.editor;

const {
  SelectControl,
  PanelBody,
  ToggleControl,
  RangeControl
} = wp.components;

const { Component } = wp.element;

const {
  siteUrl,
  siteLang,
  pluginUrl,
  premiumPluginUrl,
  hasPremium,
} = ChattyMangoTagGroupsGlobal;

const helpUrl = 'https://documentation.chattymango.com/documentation/';
const helpProduct = 'tag-groups-premium';
const helpComponent = 'shuffle-box/shuffle-box-parameters/';
const logoUrl = premiumPluginUrl + '/assets/images/cm-tgp-icon-64x64.png';

class TagGroupsHelp extends Component {
  render() {
    let href = helpUrl + helpProduct + "/" + helpComponent;

    if ( '' != siteLang ) {
      href += "?lang=" + siteLang;
    }

    href += "#" + this.props.topic;

    let tooltip = __( 'Click for help!' );

    return (
      <div>
      <a href={href} target="_blank" style={{textDecoration: 'none'}} title={tooltip}>
      <span className="dashicons dashicons-editor-help tg_right chatty-mango-help-icon"></span>
      </a>
      </div>
    );
  }
}


class tagGroupsPremiumShuffleBoxParameters extends Component {

  // Method for setting the initial state.
  static getInitialState( attributes ) {
    let selectedGroups = []; // empty means all
    let selectedTaxonomies = ['post_tag'];

    // We need arrays for the select elements.
    if ( attributes.include ) {
      selectedGroups = attributes.include.split(",")
    }

    if ( attributes.taxonomy ) {
      selectedTaxonomies = attributes.taxonomy.split(",")
    }

    return {
      groups: [],
      taxonomies: [],
      selectedGroups: selectedGroups, // array representation
      selectedTaxonomies: selectedTaxonomies, // array representation
    };
  }


  // Constructing our component. With super() we are setting everything to 'this'.
  // Now we can access the attributes with this.props.attributes
  constructor() {
    super( ...arguments );

    this.groupsEndPoint = siteUrl + '/wp-json/tag-groups/v1/groups';
    // this.termsEndPoint = siteUrl + '/wp-json/tag-groups/v1/terms';
    this.taxonomiesEndPoint = siteUrl + '/wp-json/tag-groups/v1/taxonomies';

    this.state = this.constructor.getInitialState( this.props.attributes );

    // Bind so we can use 'this' inside the method.
    this.getGroupsFromApi = this.getGroupsFromApi.bind( this );
    this.getTaxonomiesFromApi = this.getTaxonomiesFromApi.bind( this );
    this.getPostsFromApi = this.getPostsFromApi.bind( this );
    this.handleChangeInclude = this.handleChangeInclude.bind( this );
    this.handleChangeTaxonomy = this.handleChangeTaxonomy.bind( this );
    // this.handleChangeInitialGroup = this.handleChangeInitialGroup.bind( this );
    this.toggleOptionHideEmpty = this.toggleOptionHideEmpty.bind( this );
    this.toggleOptionShowGroupFilter = this.toggleOptionShowGroupFilter.bind( this );
    this.toggleOptionShowTextFilter = this.toggleOptionShowTextFilter.bind( this );
    this.toggleOptionAddPremiumFilter = this.toggleOptionAddPremiumFilter.bind( this );
    this.toggleOptionShowTagCount = this.toggleOptionShowTagCount.bind( this );
    this.toggleOptionShowFilterAllGroups = this.toggleOptionShowFilterAllGroups.bind( this );

    // Load data from REST API.
    this.getGroupsFromApi();
    this.getTaxonomiesFromApi();
    this.getPostsFromApi();
  }


  handleChangeInclude( options ) {
    let selectedGroups = options.map( function( option ) {
      if ( ! isNaN( option.value ) ) {
        return option.value;
      }
    });

    // Set the state
    this.setState( { selectedGroups: selectedGroups } );

    // Set the attributes
    this.props.setAttributes( {
      include: selectedGroups.join(',')
    } );
  }

  handleChangeTaxonomy( options ) {
    let selectedTaxonomies = options.map( function( option ) {
      if ( typeof option.value === 'string' ) {
        return option.value;
      }
    });

    // Set the state
    this.setState( { selectedTaxonomies } );

    // Set the attributes
    this.props.setAttributes( {
      taxonomy: selectedTaxonomies.join(',')
    });
  }


  /**
  * Loading Groups
  */
  getGroupsFromApi() {
    // retrieve the groups
    apiFetch( { path: this.groupsEndPoint } ).then( groups => {
      if ( groups ) {
        this.setState({ groups });
      }
    });
  }


  /**
  * Loading Taxonomies (own REST API endpoint)
  */
  getTaxonomiesFromApi() {
    // retrieve the taxonomies
    apiFetch( { path: this.taxonomiesEndPoint } ).then( taxonomies => {
      if ( taxonomies ) {
        this.setState({ taxonomies });
      }
    });
  }


  /**
  * Loading Posts
  */
  getPostsFromApi() {
    apiFetch( { path: siteUrl + '/wp-json/wp/v2/posts?per_page=100' } ).then( fullPosts => {
      if ( fullPosts ) {
        let posts = [
          { value: -1, label: __('off') },
          { value: 0, label: __('use this post') }
        ];
        fullPosts.forEach( ( fullPost ) => {
          posts.push({
            value: fullPost.id,
            label: fullPost.title.rendered
          });
        } );
        this.setState({ posts });
      }
    });
  }


  toggleOptionHideEmpty( ) {
    let hide_empty = ( 1 === this.props.attributes.hide_empty ) ? 0 : 1;
    this.props.setAttributes( { hide_empty } );
  }

  toggleOptionShowGroupFilter( ) {
    let show_group_filter = ( 1 === this.props.attributes.show_group_filter ) ? 0 : 1;
    this.props.setAttributes( { show_group_filter } );
  }

  toggleOptionShowTextFilter() {
    let show_text_filter = ( 1 === this.props.attributes.show_text_filter ) ? 0 : 1;
    this.props.setAttributes( { show_text_filter } );
  }

  toggleOptionAddPremiumFilter( key ) {
    let add_premium_filter = ( 1 === this.props.attributes.add_premium_filter ) ? 0 : 1;
    this.props.setAttributes( { add_premium_filter } );
  }

  toggleOptionShowTagCount( ) {
    let show_tag_count = ( 1 === this.props.attributes.show_tag_count ) ? 0 : 1;
    this.props.setAttributes( { show_tag_count } );
  }

  toggleOptionShowFilterAllGroups( ) {
    let show_filter_all_groups = ( 1 === this.props.attributes.show_filter_all_groups ) ? 0 : 1;
    this.props.setAttributes( { show_filter_all_groups } );
  }

  render() {
    const {
      attributes,
      setAttributes
    } = this.props;

    const {
      add_premium_filter,
      amount,
      append,
      custom_title,
      div_class,
      div_id,
      groups_post_id,
      hide_empty,
      initial_group,
      largest,
      layout_mode,
      link_append,
      link_target,
      not_assigned_name,
      order,
      orderby,
      placeholder,
      prepend,
      show_all_name,
      show_filter_all_groups,
      show_group_filter,
      show_tag_count,
      show_text_filter,
      smallest,
      tags_post_id,
      // taxonomy,
    } = attributes;

    let optionsGroups = [], optionsTaxonomies = [], optionsInitialGroup = [];

    if ( this.state.groups && this.state.groups.length > 0 ) {
      this.state.groups.forEach( ( group ) => {
        optionsGroups.push({ value:group.term_group, label:group.label });
      });
    }

    if ( this.state.taxonomies && this.state.taxonomies.length > 0 ) {
      this.state.taxonomies.forEach( ( taxonomy ) => {
        optionsTaxonomies.push({ value:taxonomy.slug, label:taxonomy.name });
      });
    }

    if ( show_filter_all_groups ) {
      optionsInitialGroup.push({ value:-1, label: __( 'All Groups' ) });
    }

    if ( this.state.groups && this.state.groups.length > 0 ) {
      this.state.groups.forEach( ( group ) => {
        if ( this.state.selectedGroups.length == 0 || this.state.selectedGroups.indexOf( String( group.term_group ) ) > -1 || this.state.selectedGroups.indexOf( group.term_group ) > -1 ) {
        optionsInitialGroup.push({ value:group.term_group, label:group.label });
        }
      });
    }

    if ( attributes.source !== 'gutenberg' ) {
      setAttributes({ source: 'gutenberg' });
    }


    return [
      (
        <InspectorControls key='inspector'>
          <div className='chatt-mango-inspector-control'>
            <PanelBody title={ __( 'Tags and Terms' ) } initialOpen={false}>
              <TagGroupsHelp topic="layout_mode"/>
              <label htmlFor="tg_input_layout_mode">
              { __( 'Layout Mode' ) }
              </label>
              <Select
                id="tg_input_layout_mode"
                onChange={ ( option ) => { if ( option ) setAttributes( { layout_mode:option.value } ) } }
                value={ ( layout_mode && ( typeof layout_mode === 'string' ) ) ? layout_mode : 'fitRows' }
                options={ [
                  { value:'fitRows', label:'fitRows' },
                  { value:'masonry', label:'masonry' },
                  { value:'vertical', label:'vertical' },
                ] }
              />
              <TagGroupsHelp topic="taxonomy"/>
              <label htmlFor="tg_input_taxonomy">
  						{ __( 'Include taxonomies' ) }
              </label>
              <Select
                id="tg_input_taxonomy"
                onChange={this.handleChangeTaxonomy}
                value={this.state.selectedTaxonomies}
                options={ optionsTaxonomies }
                multi={ true }
                closeOnSelect={ false}
                removeSelected={ true }
              />
              <TagGroupsHelp topic="smallest"/>
              <RangeControl
  							label={ __( 'Smallest font size' ) }
  							value={ smallest ? Number( smallest ) : 12 }
  							onChange={ ( value ) => { if ( value <= largest && value < 73 ) setAttributes( { smallest: value } ) } }
  							min={ 6 }
  							max={ 72 }
              />
              <TagGroupsHelp topic="largest"/>
              <RangeControl
  							label={ __( 'Largest font size' ) }
  							value={ largest ? Number( largest ) : 22 }
  							onChange={ ( value ) => { if ( smallest <= value && value > 5 ) setAttributes( { largest: value } ) } }
  							min={ 6 }
  							max={ 72 }
              />
              <TagGroupsHelp topic="amount"/>
              <RangeControl
  							label={ __( 'Amount of tags per group' ) + ( amount == 0 ? ': ' + __( 'unlimited' ) : '' ) }
  							value={ amount ? Number( amount ) : 0 }
  							onChange={ ( amount ) => setAttributes( { amount } ) }
  							min={ 0 }
  							max={ 200 }
              />
              <TagGroupsHelp topic="orderby"/>
              <label htmlFor="tg_input_orderby">
  						{ __( 'Order tags by' ) }
              </label>
              <Select
                id="tg_input_orderby"
                onChange={ ( option ) => { if ( option ) setAttributes( { orderby: option.value } ) } }
                value={ orderby && typeof orderby === 'string' ? orderby : 'name' }
                options={ [
                  { value:'name', label:__('Name') },
                  { value:'natural', label:__('Natural sorting') },
                  { value:'count', label:__('Post count') },
                  { value:'slug', label:__('Slug') },
                  { value:'term_id', label:__('Term ID') },
                  { value:'description', label:__('Description') },
                ] }
              />
              <TagGroupsHelp topic="order"/>
              <label htmlFor="tg_input_order">
  						{ __( 'Sort order' ) }
              </label>
              <Select
                id="tg_input_order"
                onChange={ ( option ) => { if ( option ) setAttributes( { order: option.value } ) } }
                value={ order && typeof order === 'string' ? order.toUpperCase() : 'ASC' }
                options={ [
                  { value:'ASC', label:__('Ascending') },
                  { value:'DESC', label:__('Descending') }
                ] }
              />
              <TagGroupsHelp topic="hide_empty"/>
              <ToggleControl
                label={ __( 'Hide empty tags' ) }
                checked={ hide_empty }
                onChange={ this.toggleOptionHideEmpty }
              />
              <TagGroupsHelp topic="prepend"/>
              <div>
              <label htmlFor="tg_input_prepend">
  						{ __( 'Prepend' ) }
              </label>
              </div>
              <PlainText
                id="tg_input_prepend"
    						className="input-control"
    						value={ prepend ? prepend : '' }
    						placeholder={ __( 'Write here or leave empty.' ) }
    						onChange={ ( prepend ) => setAttributes( { prepend } ) }
              />
              <TagGroupsHelp topic="append"/>
              <div>
              <label htmlFor="tg_input_append">
  						{ __( 'Append' ) }
              </label>
              </div>
              <PlainText
                id="tg_input_append"
    						className="input-control"
    						value={ append ? append : '' }
    						placeholder={ __( 'Write here or leave empty.' ) }
    						onChange={ ( append ) => setAttributes( { append } ) }
              />
              { ! custom_title &&
                <div>
              <TagGroupsHelp topic="show_tag_count"/>
                <ToggleControl
                  label={ __( 'Show the post count in the tooltip' ) }
                  checked={ show_tag_count }
                  onChange={ this.toggleOptionShowTagCount }
                />
                </div>
                }
                <div>
                <TagGroupsHelp topic="custom_title"/>
                <label htmlFor="tg_input_custom_title">
    						{ __( 'Custom title' ) }
                </label>
                </div>
                <PlainText
                  id="tg_input_custom_title"
      						className="input-control"
      						value={ custom_title ? custom_title : '' }
      						placeholder={ __( 'Write here or leave empty.' ) }
      						onChange={ ( custom_title ) => setAttributes( { custom_title } ) }
                />
              <TagGroupsHelp topic="link_target"/>
              <label htmlFor="tg_input_link_target">
  						{ __( 'Link target' ) }
              </label>
              <Select
                id="tg_input_link_target"
                onChange={ ( option ) => { if ( option ) setAttributes( { link_target:option.value } ) } }
                value={ ( link_target && ( typeof link_target === 'string' ) ) ? link_target : '_self' }
                options={ [
                  { value:'_self', label:'_self' },
                  { value:'_blank', label:'_blank' },
                  { value:'_parent', label:'_parent' },
                  { value:'_top', label:'_top' },
                ] }
              />
              <div>
              <label htmlFor="tg_input_link_append">
  						{ __( 'Append to the link' ) }
              </label>
              </div>
              <PlainText
                id="tg_input_link_append"
    						className="input-control"
    						value={ link_append ? link_append : '' }
    						placeholder={ __( 'Write here or leave empty.' ) }
    						onChange={ ( link_append ) => setAttributes( { link_append } ) }
              />
              { hasPremium &&
                <div>
                <TagGroupsHelp topic="add_premium_filter"/>
                <ToggleControl
                  label={ __( 'Add filter to tags for multiple groups.' ) }
                  checked={ add_premium_filter }
                  onChange={ this.toggleOptionAddPremiumFilter }
                />
                </div>
              }
              <TagGroupsHelp topic="tags_post_id"/>
              <label htmlFor="tg_input_tags_post_id">
              { __( 'Use tags of the following post:' ) }
              </label>
              <Select
                id="tg_input_tags_post_id"
                onChange={ ( option ) => { if ( option ) setAttributes( { tags_post_id:option.value } ) } }
                value={ tags_post_id }
                options={ this.state.posts }
              />
            </PanelBody>

            <PanelBody title={ __( 'Groups' ) } initialOpen={false}>
              <TagGroupsHelp topic="include"/>
              <label htmlFor="tg_input_include">
  						{ __( 'Include groups' ) }
              </label>
              <Select
                id="tg_input_include"
                onChange={ this.handleChangeInclude }
                value={ this.state.selectedGroups }
                options={ optionsGroups }
                multi={ true }
                closeOnSelect={ false}
                removeSelected={ true }
              />
              { this.state.selectedGroups.indexOf(-1) > -1 &&
                <div>
                <div>
                <label htmlFor="tg_input_not_assigned_name">
    						{ __( 'Label on button for not assigned tags' ) }
                </label>
                </div>
                <PlainText
                  id="tg_input_not_assigned_name"
      						className="input-control"
      						value={ not_assigned_name ? not_assigned_name : 'not assigned' }
      						placeholder={ __( 'Write here or leave empty.' ) }
      						onChange={ ( not_assigned_name ) => setAttributes( { not_assigned_name } ) }
                />
                </div>
              }
              <TagGroupsHelp topic="groups_post_id"/>
              <label htmlFor="tg_input_group_post_id">
  						{ __( 'Use groups of the following post:' ) }
              </label>
              <Select
                id="tg_input_group_post_id"
                onChange={ ( option ) => { if ( option ) setAttributes( { groups_post_id:option.value } ) } }
                value={ groups_post_id }
                options={ this.state.posts }
              />
            </PanelBody>

            <PanelBody title={ __( 'Filters' ) } initialOpen={false}>
            <TagGroupsHelp topic="show_text_filter"/>
            <ToggleControl
            label={ __( 'Show input field to filter by text.' ) }
            checked={ show_text_filter }
            onChange={ this.toggleOptionShowTextFilter }
            />
            <TagGroupsHelp topic="show_group_filter"/>
            <ToggleControl
              label={ __( 'Show buttons to filter by group.' ) }
              checked={ show_group_filter }
              onChange={ this.toggleOptionShowGroupFilter }
            />
            <TagGroupsHelp topic="show_filter_all_groups"/>
            <ToggleControl
              label={ __( 'Show Filter "All Groups"' ) }
              checked={ show_filter_all_groups }
              onChange={ this.toggleOptionShowFilterAllGroups }
            />
            <TagGroupsHelp topic="initial_group"/>
            <label htmlFor="tg_input_initial_group">
            { __( 'Initially selected filter' ) }
            </label>
            <Select
              id="tg_input_initial_group"
              onChange={ ( option ) => { if ( option ) setAttributes( { initial_group:option.value } ) } }
              value={ ( initial_group ) ? initial_group : -1 }
              options={ optionsInitialGroup }
              multi={ false }
            />
            </PanelBody>

            { ( show_group_filter === 1 || show_text_filter === 1 ) &&
            <PanelBody title={ __( 'Labels and Messages' ) } initialOpen={false}>
            {show_text_filter === 1 &&
              <div>
            <TagGroupsHelp topic="placeholder"/>
            <label htmlFor="tg_input_placeholder">
            { __( 'Placeholder for tag field' ) }
            </label>
            <PlainText
              id="tg_input_placeholder"
              className="input-control"
              value={ placeholder ? placeholder : '' }
              placeholder={ __( 'Write here or leave empty.' ) }
              onChange={ ( placeholder ) => setAttributes( { placeholder } ) }
            />
            </div>
            }
            {show_group_filter === 1 &&
              <div>
            <TagGroupsHelp topic="not_assigned_name"/>
            <label htmlFor="tg_input_not_assigned_name">
            { __( 'Label for "not assigned" button' ) }
            </label>
            <PlainText
              id="tg_input_not_assigned_name"
              className="input-control"
              value={ not_assigned_name ? not_assigned_name : '' }
              placeholder={ __( 'Write here or leave empty.' ) }
              onChange={ ( not_assigned_name ) => setAttributes( { not_assigned_name } ) }
            />
            <TagGroupsHelp topic="show_all_name"/>
            <label htmlFor="tg_input_show_all_name">
            { __( 'Label for "show all" button' ) }
            </label>
            <PlainText
              id="tg_input_show_all_name"
              className="input-control"
              value={ show_all_name ? show_all_name : '' }
              placeholder={ __( 'Write here or leave empty.' ) }
              onChange={ ( show_all_name ) => setAttributes( { show_all_name } ) }
            />
            </div>
            }
            </PanelBody>
          }

            <PanelBody title={ __( 'Advanced Styling' ) } initialOpen={false}>
            <div>
            <TagGroupsHelp topic="div_id"/>
            <label htmlFor="tg_input_div_id">
            { '<div id="...">' }
            </label>
            </div>
            <PlainText
              id="tg_input_div_id"
              className="input-control"
              value={ div_id ? div_id : '' }
              placeholder={ __( 'Write here or leave empty.' ) }
              onChange={ ( div_id ) => setAttributes( { div_id } ) }
            />
            <div>
            <TagGroupsHelp topic="div_class"/>
            <label htmlFor="tg_input_div_class">
            { __( '<div class="...">' ) }
            </label>
            </div>
            <PlainText
              id="tg_input_div_class"
              className="input-control"
              value={ div_class ? div_class : '' }
              placeholder={ __( 'Write here or leave empty.' ) }
              onChange={ ( div_class ) => setAttributes( { div_class } ) }
            />
            </PanelBody>
          </div>
        </InspectorControls>
      ),
      <div className="chatty-mango-editor">
      <table style={{border:'none'}}>
      <tr>
      <td>
        <img src={logoUrl} alt='logo' style={{float:'left', margin:15}}/>
      </td>
      <td>
          <h3>{ __( 'Shuffle Box' ) }</h3>
          <div className="cm-gutenberg dashicons-before dashicons-admin-generic">
            { __( 'Select this block and customize the Shuffle Box in the Inspector.' ) }
          </div>
          <div className="cm-gutenberg dashicons-before dashicons-welcome-view-site">
            { __( 'See the output with Preview.' ) }
          </div>
          </td>
          </tr>
          </table>
      </div>
    ]
  }

}


/**
* Register: a Gutenberg Block.
*
* @param  {string}	  name	   Block name.
* @param  {Object}	  settings Block settings.
* @return {?WPBlock}		   The block, if it has been successfully
*							   registered; otherwise `undefined`.
*/
var cmTagGroupsTableBlock = registerBlockType( 'chatty-mango/tag-groups-premium-shuffle-box', {
  title: __( 'Shuffle Box' ),
  icon: 'tagcloud', // Block icon from Dashicons → https://developer.wordpress.org/resource/dashicons/.
  category: 'widgets',
  description: __( 'Show your tags in a searchable tag cloud.' ),
  keywords: [
    __( 'shuffle' ),
    __( 'tag cloud' ),
    'Chatty Mango',
  ],
  html: false,
  useOnce: false,
  transforms: {
    to: [
      {
        type: 'block',
        blocks: [ 'core/shortcode' ],
        transform: function( attributes ) {
          let parameters = [];
          for ( var attribute in attributes ) {
            if (attributes.hasOwnProperty( attribute )) {
              if ( null !== attributes[attribute] && '' !== attributes[ attribute ] && 'source' !== attribute && cmTagGroupsTableBlock.attributes[ attribute ] && attributes[ attribute ] !== cmTagGroupsTableBlock.attributes[ attribute ].default ) {
                if ( typeof attributes[attribute] === 'number' ) {
                  parameters.push( attribute + '=' + attributes[ attribute ] );
                } else {
                  parameters.push( attribute + '="' + attributes[ attribute ] + '"');
                }
              }
            }
          }

          let text = '[tag_groups_shuffle_box ' + parameters.join(' ') + ']';
          return createBlock( 'core/shortcode', {
            text
          } );
        },
      },
    ],
  },
  supports: {
    html: false,
  },

  /**
  * Attributes are the same as shortcode parameters
  **/
  attributes: {
    source: { // internal indicator to identify Gutebergb blocks
      type: 'string',
      default: ''
    },
    add_premium_filter: {// configurable in block
      type: 'integer',
      default: 0
    },
    amount: {// configurable in block
      type: 'integer',
      default: 0
    },
    append: {// configurable in block
      type: 'string',
      default: ''
    },
    custom_title: {// configurable in block
      type: 'string',
      default: ''
    },
    div_class: {// configurable in block
      type: 'string',
      default: 'tag-groups-theme-default'
    },
    div_id: {// configurable in block
      type: 'string',
      default: ''
    },
    groups_post_id: {// configurable in block
      type: 'integer',
      default: -1
    },
    hide_empty: {// configurable in block
      type: 'integer',
      default: 1
    },
    include: {// configurable in block
      type: 'string',
      default: ''
    },
    initial_group: {// configurable in block
      type: 'integer',
      default: -1
    },
    largest: {// configurable in block
      type: 'integer',
      default: 22
    },
    layout_mode: {// configurable in block
      type: 'string',
      default: 'fitRows'
    },
    link_append: {// configurable in block
      type: 'string',
      default: ''
    },
    link_target: {// configurable in block
      type: 'string',
      default: '_self'
    },
    not_assigned_name: {// configurable in block
      type: 'string',
      default: __( 'not assigned' )
    },
    order: {// configurable in block
      type: 'string',
      default: 'ASC'
    },
    orderby: {// configurable in block
      type: 'string',
      default: 'name'
    },
    placeholder: {// configurable in block
      type: 'string',
      default: __( 'search' )
    },
    prepend: {// configurable in block
      type: 'string',
      default: ''
    },
    show_all_name: {// configurable in block
      type: 'string',
      default: __( 'all groups' )
    },
    show_filter_all_groups: {// configurable in block
      type: 'integer',
      default: 1
    },
    show_group_filter: {// configurable in block
      type: 'integer',
      default: 1
    },
    show_tag_count: { // configurable in block
      type: 'integer',
      default: 1
    },
    show_text_filter: {// configurable in block
      type: 'integer',
      default: 1
    },
    smallest: {// configurable in block
      type: 'integer',
      default: 12
    },
    tags_post_id: {// configurable in block
      type: 'integer',
      default: -1
    },
    taxonomy: {// configurable in block
      type: 'string',
      default: ''
    },
  },

  /**
  * Composing and rendering the editor content and control elements
  */
  edit: tagGroupsPremiumShuffleBoxParameters,


  /**
  * We don't render any HTML when saving
  */
  save: function( props ) {
    return null;
  },
} );
