<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2017 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*
*/

// Don't call this file directly
if ( ! defined( 'ABSPATH') ) {

  die;

}


/**
* This plugin's absolute path on this server - starting from root.
*/
define( "TAG_GROUPS_PREMIUM_PLUGIN_ABSOLUTE_PATH", dirname( __FILE__ ) );

/**
* This plugin's last piece of the path, i.e. basically the plugin's name
*/
define( "TAG_GROUPS_PREMIUM_PLUGIN_RELATIVE_PATH", basename( dirname( __FILE__ ) ) );

/*
* Default lifetime of elements in ChattyMango_Cache.
* in seconds
*/
define ( "TAG_GROUPS_PREMIUM_CACHE_LIFETIME", 600 );

/*
* The full URL (including protocol) of the RSS channel that informas about updates.
*/
// define ( "TAG_GROUPS_PREMIUM_UPDATES_RSS_URL", "https://chattymango.com/news/feed/?cat=3,4");
define ( "TAG_GROUPS_PREMIUM_UPDATES_RSS_URL", "https://chattymango.com/blog/feed/?cat=4");

/**
* Time interval to refresh the available tags in the meta box; in milliseconds; default: 5 minutes
*
*/
define ( "TAG_GROUPS_PREMIUM_MB_RELOAD_TAGS", 300000 );



/**
* add Gutenberg functionality
*/
require_once TAG_GROUPS_PREMIUM_PLUGIN_ABSOLUTE_PATH . '/src/init.php';

/**
*  Do initial variable setups, register hooks etc.
*
*  @param void
*  @return void
*/
function tag_groups_premium_init() {

  /**
  * Running the activation hook - we need some classes for that
  */
  require_once dirname( __FILE__ ) . '/include/class.premium_loader.php';

  $tag_groups_premium_loader = new TagGroups_Premium_Loader( TAG_GROUPS_PREMIUM_PLUGIN_ABSOLUTE_PATH );

  $tag_groups_premium_loader->require_classes();


  // register_activation_hook( __FILE__, array( 'TagGroups_Premium', 'on_activation' ) );


  /*
  * 3rd party code Plugin Update Checker Library 2.0.0 (Update routine), kernl.us
  */
  if ( ! defined( 'CM_UPDATE_CHECK' ) || CM_UPDATE_CHECK ) {

    if ( defined( 'CM_TGP_BETA_PLUGIN_UUID' ) && CM_TGP_BETA_PLUGIN_UUID ) {

      $kernl_plugin_uuid = CM_TGP_BETA_PLUGIN_UUID;

    } else if ( defined( 'CM_TGP_KERNL_UUID' ) && CM_TGP_KERNL_UUID ) {

      $kernl_plugin_uuid = CM_TGP_KERNL_UUID;

    }


    if ( file_exists( dirname( __FILE__ ) . '/plugin_update_check.php' ) && ! empty( $kernl_plugin_uuid ) && WP_PLUGIN_DIR . '/' . TAG_GROUPS_PLUGIN_BASENAME != '' ) {

      require_once dirname( __FILE__ ) . '/plugin_update_check.php';

      $MyUpdateChecker = new PluginUpdateChecker_2_0 (
        'https://kernl.us/api/v1/updates/' . $kernl_plugin_uuid . '/',
        WP_PLUGIN_DIR . '/' . TAG_GROUPS_PLUGIN_BASENAME,
        'tag-groups-premium',
        1
      );

    }

  }


  /*
  * The function plugins_url() is available only now.
  */
  define ( "TAG_GROUPS_PREMIUM_PLUGIN_URL", plugins_url( '', __FILE__ ) );

  
  $tag_groups_premium_loader->check_preconditions();

  $tag_groups_premium_loader->provide_globals();

  $tag_groups_premium_loader->add_hooks();

  $tag_groups_premium_loader->register_shortcodes_and_blocks();

  $tag_groups_premium_loader->register_CRON();

  $tag_groups_premium_loader->maybe_run_version_update();


}


/**
* Internationalization before WP 4.6
*
* Doesn't work as method
*
* @param void
* @return void
*/
function tag_groups_premium_load_plugin_textdomain() {

  load_plugin_textdomain( 'tag-groups-premium', false, TAG_GROUPS_PREMIUM_PLUGIN_RELATIVE_PATH . '/languages/' );

}

add_action( 'init', 'tag_groups_premium_load_plugin_textdomain' );



/*
* Add here all API functions that must be publically available
*/
