<?php
/**
* Product attributes
*
* Modified to allow for clickable attribute values.
*
*
* @see 	    https://docs.woocommerce.com/document/template-structure/
* @author 		WooThemes
* @version     3.1.0
*
*
*	Modifications:
* @author		Chatty Mango
* @since			1.15.0
*/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
?>
<table class="shop_attributes">
	<?php if ( $display_dimensions && $product->has_weight() ) : ?>
		<tr>
			<th><?php _e( 'Weight', 'woocommerce' ) ?></th>
			<td class="product_weight"><?php echo esc_html( wc_format_weight( $product->get_weight() ) ); ?></td>
		</tr>
	<?php endif; ?>

	<?php if ( $display_dimensions && $product->has_dimensions() ) : ?>
		<tr>
			<th><?php _e( 'Dimensions', 'woocommerce' ) ?></th>
			<td class="product_dimensions"><?php echo esc_html( wc_format_dimensions( $product->get_dimensions( false ) ) ); ?></td>
		</tr>
	<?php endif; ?>

	<?php foreach ( $attributes as $attribute ) : ?>
		<tr>
			<th><?php echo wc_attribute_label( $attribute->get_name() ); ?></th>
			<td><?php
			$values = array();
			if ( $attribute->is_taxonomy() ) {
				$attribute_taxonomy = $attribute->get_taxonomy_object();
				$attribute_values = wc_get_product_terms( $product->get_id(), $attribute->get_name(), array( 'fields' => 'all' ) );
				foreach ( $attribute_values as $attribute_value ) {
					$value_name = esc_html( $attribute_value->name );
					if ( $attribute_taxonomy->attribute_public ) {
						$values[] = '<a href="' . esc_url( get_term_link( $attribute_value->term_id, $attribute->get_name() ) ) . '" rel="tag">' . $value_name . '</a>';
					} else {
						$values[] = $value_name;
					}
				}
			} elseif ( method_exists( $attribute, 'get_terms' ) ) {
				// modified here to keep the links
				$terms = $attribute->get_terms();
				foreach ( $terms as $term ) {
					$values[] = '<a href="' . esc_url( get_term_link( $term ) ) . '" rel="tag">' . esc_html( $term->name ) . '</a>';
				}
			} else {
				$values = $attribute->get_options();
				foreach ( $values as &$value ) {
					if ( strpos( $value, '<a' ) === false ) {
						$value = make_clickable( esc_html( $value ) );
					}
				}
			}
			echo apply_filters( 'woocommerce_attribute', wpautop( wptexturize( implode( $separator, $values ) ) ), $attribute, $values );
			?></td>
		</tr>
	<?php endforeach; ?>
</table>
<!-- template woocommerce/single-product/product-attributes-with-links.php end marker -->
